package com.c503.ca.cadv.codegen;

import lombok.Getter;
import lombok.SneakyThrows;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.util.Properties;

@Getter
public class CodeGenConfig {

    public final static String[] commonColumns = {"id_", "version_", "deleted_", "created_by", "created_date", "last_modified_by", "last_modified_date"};
    /**
     * 数据库配置
     */
    private String databaseUrl = "jdbc:mysql://xx.xx.xx.xx:3306/xxx";
    private String databaseUsername = "root";
    private String databasePassword = "ABC.123456";
    private String databaseSchemeName = "public";
    //"org.postgresql.Driver";
    private String databaseDriverName = "com.mysql.cj.jdbc.Driver";

    private String superControllerClass = "DataVOResource";
    private String superMapperClass = "BaseRepository";
    private String superServiceClass = "DataVOService";
    private String superEntityClass = "IdAutoEntity";
    private String superEntityVoClass = "DataEntityVO";
    private String moduleParent = "com.c503.htphy.xx.module";

    private String moduleParentPath = "/java/com/c503/htphy/xx/module/";
    private String moduleMappingPath = "/resources/mappings/module/";

    private String outputDir = "/codegen-result/src/main";

    public CodeGenConfig() {
        this.loadConfigFile();
    }

    @SneakyThrows
    public void loadConfigFile() {
        Properties properties = new Properties();
        String fileName = "codegen.properties";
        InputStream inputStream = null;
        try {
            inputStream = getClass().getClassLoader().getResourceAsStream(fileName);
            if (inputStream != null) {
                properties.load(inputStream);
            } else {
                throw new FileNotFoundException("property file '" + fileName + "' not found in the classpath");
            }
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            if (inputStream != null) {
                try {
                    inputStream.close();
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
        }

        this.databaseUrl = properties.getProperty("databaseUrl", this.databaseUrl);
        this.databaseUsername = properties.getProperty("databaseUsername", this.databaseUsername);
        this.databasePassword = properties.getProperty("databasePassword", this.databasePassword);
        this.databaseSchemeName = properties.getProperty("databaseSchemeName", this.databaseSchemeName);
        this.databaseDriverName = properties.getProperty("databaseDriverName", this.databaseDriverName);

        this.superControllerClass = properties.getProperty("superControllerClass", this.superControllerClass);
        this.superMapperClass = properties.getProperty("superMapperClass", this.superMapperClass);
        this.superServiceClass = properties.getProperty("superServiceClass", this.superServiceClass);
        this.superEntityClass = properties.getProperty("superEntityClass", this.superEntityClass);
        this.superEntityVoClass = properties.getProperty("superEntityVoClass", this.superEntityVoClass);
        this.moduleParent = properties.getProperty("moduleParent", this.moduleParent);
        this.moduleParentPath = properties.getProperty("moduleParentPath", this.moduleParentPath);
        this.moduleMappingPath = properties.getProperty("moduleMappingPath", this.moduleMappingPath);
        this.outputDir = properties.getProperty("outputDir", this.outputDir);
    }

}
