package com.c503.ca.cadv.codegen;

import com.baomidou.mybatisplus.core.exceptions.MybatisPlusException;
import com.baomidou.mybatisplus.core.toolkit.StringPool;
import com.baomidou.mybatisplus.core.toolkit.StringUtils;
import com.baomidou.mybatisplus.generator.AutoGenerator;
import com.baomidou.mybatisplus.generator.InjectionConfig;
import com.baomidou.mybatisplus.generator.config.*;
import com.baomidou.mybatisplus.generator.config.po.TableInfo;
import com.baomidou.mybatisplus.generator.config.rules.NamingStrategy;
import com.baomidou.mybatisplus.generator.engine.FreemarkerTemplateEngine;

import java.util.*;

/**
 * 代码生成工具
 *
 * @author liaow
 */
public class CodeGenerator {

    /**
     * RUN THIS
     */
    public static void main(String[] args) {
        AutoGenerator generator = getAutoGenerator();
        generator.execute();
    }

    private static AutoGenerator getAutoGenerator() {
        //获取配置信息
        CodeGenConfig genConfig = new CodeGenConfig();
        // 代码生成器
        AutoGenerator generator = new AutoGenerator();

        //输入参数
        String author = scanner("开发人员代号", System.getProperty("user.name"));
        System.out.println("开发人员代号:" + author);
//        String serviceName = scanner("服务简称");
//        System.out.println("服务简称:" + serviceName);
        String moduleName = scanner("模块名");
        System.out.println("模块名:" + moduleName);
        String tableNames = scanner("表名");
        System.out.println("表名:" + tableNames);
        String tablePrefix = scanner("表名前缀", moduleName + "_");
        System.out.println("表名前缀:" + tablePrefix);

        // 全局配置
        GlobalConfig globalConfig = new GlobalConfig();
        String projectPath = System.getProperty("user.dir");
        globalConfig.setOutputDir(projectPath + genConfig.getOutputDir() + "/java");
        globalConfig.setFileOverride(true);
        globalConfig.setAuthor(author);
        globalConfig.setOpen(false);
        globalConfig.setSwagger2(true);
        globalConfig.setEntityName("%s");
        globalConfig.setControllerName("%sResource");
        globalConfig.setMapperName("%sRepository");
        globalConfig.setServiceName("%sService");
        globalConfig.setBaseResultMap(true);
        globalConfig.setBaseColumnList(true);
        globalConfig.setActiveRecord(false);
        generator.setGlobalConfig(globalConfig);

        // 数据源配置
        DataSourceConfig dsc = new DataSourceConfig();
        dsc.setUrl(genConfig.getDatabaseUrl());
        if (StringUtils.isNotBlank(genConfig.getDatabaseSchemeName())) {
            dsc.setSchemaName(genConfig.getDatabaseSchemeName());
        }
        dsc.setDriverName(genConfig.getDatabaseDriverName());
        dsc.setUsername(genConfig.getDatabaseUsername());
        dsc.setPassword(genConfig.getDatabasePassword());
        generator.setDataSource(dsc);

        // 包配置
        PackageConfig packageConfig = new PackageConfig();
        packageConfig.setModuleName(moduleName);
        packageConfig.setParent(genConfig.getModuleParent());
        packageConfig.setMapper("repository");
        packageConfig.setController("web");
        packageConfig.setEntity("domain");
        generator.setPackageInfo(packageConfig);

        // 策略配置
        StrategyConfig strategy = new StrategyConfig();
        strategy.setNaming(NamingStrategy.underline_to_camel);
        strategy.setColumnNaming(NamingStrategy.underline_to_camel);
        strategy.setSuperEntityClass(genConfig.getSuperEntityClass());
        strategy.setEntityTableFieldAnnotationEnable(true);
        strategy.setEntityLombokModel(true);
        strategy.setSuperControllerClass(genConfig.getSuperControllerClass());
        strategy.setSuperMapperClass(genConfig.getSuperMapperClass());
        strategy.setSuperServiceClass(genConfig.getSuperServiceClass());

        strategy.setInclude(tableNames == null ? null : tableNames.split(","));
        strategy.setSuperEntityColumns(CodeGenConfig.commonColumns);
        strategy.setControllerMappingHyphenStyle(true);
        strategy.setRestControllerStyle(true);
        strategy.setTablePrefix(tablePrefix);
        generator.setStrategy(strategy);

        //模板配置
        TemplateConfig templateConfig = new TemplateConfig();
        templateConfig.setController("codegen-template/controller.java");
        templateConfig.setEntity("codegen-template/entity.java");
        templateConfig.setService("codegen-template/service.java");
        templateConfig.setMapper("codegen-template/mapper.java");
        templateConfig.setServiceImpl(null);
        templateConfig.setXml(null);
        generator.setTemplate(templateConfig);

        // 自定义配置
        InjectionConfig cfg = new InjectionConfig() {
            @Override
            public void initMap() {
                // to do nothing
            }
        };
        List<FileOutConfig> focList = new ArrayList<>();
        focList.add(new FileOutConfig("codegen-template/mapper.xml.ftl") {
            @Override
            public String outputFile(TableInfo tableInfo) {
                // 自定义输入文件名称
                return projectPath + genConfig.getOutputDir() + genConfig.getModuleMappingPath() + moduleName
                        + "/" + tableInfo.getEntityName() + "Mapper" + StringPool.DOT_XML;
            }
        });
        focList.add(new FileOutConfig("codegen-template/entityVO.java.ftl") {
            @Override
            public String outputFile(TableInfo tableInfo) {
                // 自定义输入文件名称
                return projectPath + genConfig.getOutputDir() + genConfig.getModuleParentPath() + moduleName
                        + "/" + packageConfig.getEntity() + "/" + "vo/" + tableInfo.getEntityName() + "VO" + StringPool.DOT_JAVA;
            }
        });
        cfg.setFileOutConfigList(focList);
        Map<String, Object> customVariable = new HashMap<>(8);
        customVariable.put("superEntityVoClass", genConfig.getSuperEntityVoClass());
        cfg.setMap(customVariable);
        generator.setCfg(cfg);

        // 选择 freemarker 引擎需要指定如下加，注意 pom 依赖必须有！
        generator.setTemplateEngine(new FreemarkerTemplateEngine());
        return generator;
    }

    public static String scanner(String tip) {
        return scanner(tip, null);
    }

    public static String scanner(String tip, String defaultContent) {
        Scanner scanner = new Scanner(System.in);
        StringBuilder help = new StringBuilder();
        if (StringUtils.isNotBlank(defaultContent)) {
            help.append("请输入").append(tip).append("(").append(defaultContent).append(")").append("：");
        } else {
            help.append("请输入").append(tip).append("：");
        }
        System.out.println(help.toString());
        if (scanner.hasNextLine()) {
            String ipt = scanner.nextLine();
            if (StringUtils.isNotBlank(ipt)) {
                return ipt;
            } else {
                return defaultContent;
            }
        }
        throw new MybatisPlusException("请输入正确的" + tip + "！");
    }

}
