/*
* Copyright &copy; 2020-2021  c503-htphy All rights reserved.
*/
package ${package.Controller};

import com.fasterxml.jackson.databind.JsonNode;
import ResultBuilder;
import JsonUtils;
import StringUtils;
import Globals;
import PageModel;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.http.MediaType;
import com.google.common.collect.Lists;
import javax.validation.Valid;
import ${package.Entity}.vo.${entity}VO;
import ${package.Service}.${table.serviceName};
<#if restControllerStyle>
import org.springframework.web.bind.annotation.RestController;
<#else>
import org.springframework.stereotype.Controller;
</#if>
<#if swagger2>
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
</#if>
<#if superControllerClassPackage??>
import ${superControllerClassPackage};
</#if>

/**
* <p>
* ${table.comment!} 控制器
* </p>
*
* @author ${author}
*/
<#if restControllerStyle>
@RestController
<#else>
@Controller
</#if>
@RequestMapping("<#if package.ModuleName??>/${package.ModuleName}</#if>/<#if controllerMappingHyphenStyle??>${controllerMappingHyphen}<#else>${table.entityPath}</#if>")
<#if swagger2>
@Api(tags = "<#if package.ModuleName??>${package.ModuleName}</#if>_${entity}")
</#if>
<#if kotlin>
class ${table.controllerName}<#if superControllerClass??> : ${superControllerClass}()</#if>
<#else>
    <#if superControllerClass??>
public class ${table.controllerName} extends ${superControllerClass}<${table.serviceName}, ${entity}VO> {
<#else>
public class ${table.controllerName} {
    </#if>
    public ${table.controllerName}(${table.serviceName} service) {
        super(service);
    }

    @GetMapping(value = "/")
    @ApiOperation(value = "获取${table.comment!}列表", notes = "根据分页参数，获取分页列表")
    public ResponseEntity<CustomMessage<JsonNode>> getPage(PageModel pm) {
        service.findVOPage(pm);
        JsonNode rs = JsonUtils.getInstance().toJsonObject(pm);
        return ResultBuilder.buildObject(rs);
    }

    /**
    * 保存 ${table.comment!}
    */
    @PostMapping(value = "/", produces = MediaType.APPLICATION_JSON_VALUE)
    @ApiOperation(value = "保存${table.comment!}", notes = "")
    public ResponseEntity<CustomMessage<${entity}VO>> save(@Valid @RequestBody ${entity}VO vo) {
        boolean success = service.saveOrUpdate(vo);
        if (success) {
            return ResultBuilder.buildOk(vo, "保存成功");
        } else {
            return ResultBuilder.buildFailed("保存失败");
        }
    }

    /**
    * 批量删除
    */
    @DeleteMapping(value = "/{ids:" + BaseEntity.ENTITY_ID_REGEX + "}")
    @ApiOperation(value = "删除实体", notes = "批量删除，ID通过逗号分隔")
    public ResponseEntity<CustomMessage<String>> delete(@PathVariable String ids) {
        service.removeByIds(Lists.newArrayList(ids.split(StringUtils.SPLIT_DEFAULT)));
        return ResultBuilder.buildOk("删除成功");
    }

}
</#if>
