package com.c503.ca.cadv.common.config;

import com.c503.ca.cadv.common.util.domain.PageModel;
import com.fasterxml.classmate.ResolvedType;
import com.fasterxml.classmate.TypeResolver;
import org.springframework.core.annotation.Order;
import org.springframework.stereotype.Component;
import springfox.documentation.builders.RequestParameterBuilder;
import springfox.documentation.schema.ScalarType;
import springfox.documentation.service.ParameterStyle;
import springfox.documentation.service.ParameterType;
import springfox.documentation.service.RequestParameter;
import springfox.documentation.service.ResolvedMethodParameter;
import springfox.documentation.spi.DocumentationType;
import springfox.documentation.spi.service.OperationBuilderPlugin;
import springfox.documentation.spi.service.contexts.OperationContext;
import springfox.documentation.swagger.common.SwaggerPluginSupport;

import java.util.List;

import static com.google.common.collect.Lists.newArrayList;

/**
 * 自定义分页接口参数
 *
 * @author liaow
 */
@Component
@Order(SwaggerPluginSupport.SWAGGER_PLUGIN_ORDER + 1000)
public class SwaggerPageableParameterReader implements OperationBuilderPlugin {

    private final ResolvedType pageableType;

    private final TypeResolver resolver;

    public SwaggerPageableParameterReader(TypeResolver resolver) {
        this.resolver = resolver;
        this.pageableType = this.resolver.resolve(PageModel.class);
    }

    @Override
    public void apply(OperationContext context) {
        List<ResolvedMethodParameter> methodParameters = context.getParameters();
        List<RequestParameter> parameters = newArrayList();

        for (ResolvedMethodParameter methodParameter : methodParameters) {
            ResolvedType resolvedType = methodParameter.getParameterType();

            if (pageableType.getErasedType().equals(resolvedType.getErasedType())) {
                parameters.add(new RequestParameterBuilder()
                    .in(ParameterType.QUERY)
                    .name("page")
                    .query(q -> q.model(m -> m.scalarModel(ScalarType.INTEGER)))
                    .description("Page number of the requested page").build());
                parameters.add(new RequestParameterBuilder()
                    .in(ParameterType.QUERY)
                    .name("size")
                    .query(q -> q.style(ParameterStyle.SIMPLE)
                        .model(m -> m.scalarModel(ScalarType.INTEGER)))
                    .description("Size of a page，填写 -1 则不分页").build());
                parameters.add(new RequestParameterBuilder()
                    .in(ParameterType.QUERY)
                    .name("queryConditionJson")
                    .query(q -> q.style(ParameterStyle.SIMPLE)
                        .model(m -> m.scalarModel(ScalarType.STRING)))
                    .description("查询条件:" +
                        "\n* [{\"fieldName\":\"name\",\"operate\":\"eq\",\"value\":\"g\"},{},...,{}]\r\n" +
                        "<br></br>" +
                        "\n* 等于:eq，不等于:ne" +
                        "\n* 大于:gt，小于:lt，大于等于:ge，小于等于:le" +
                        "\n* 类似:like，不类似:notLike" +
                        "\n* 区间:between，两个值分别用 value,endValue" +
                        "\n* 包含:in，不包含:notIn" +
                        "\n* 为Null:isNull，不为Null:isNotNull").build());
                parameters.add(
                    new RequestParameterBuilder().in(ParameterType.QUERY)
                        .name("sortName")
                        .query(q -> q.style(ParameterStyle.SIMPLE)
                            .model(m -> m.scalarModel(ScalarType.STRING)))
                        .description("排序方式：如多排序条件,分隔符为英文半角逗号：name asc,date desc")
                        .build());
                context.operationBuilder().requestParameters(parameters);
            }
        }
    }

    @Override
    public boolean supports(DocumentationType delimiter) {
        return SwaggerPluginSupport.pluginDoesApply(delimiter);
    }
}
