package com.c503.ca.cadv.common.config;

import com.c503.ca.cadv.common.util.TimeZoneEnum;
import com.c503.ca.cadv.common.util.spring.SpringContextHolder;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;
import java.util.TimeZone;

/**
 * 系统相关配置
 *
 * @author liaow
 */
@ConfigurationProperties(prefix = "system", ignoreInvalidFields = true)
@Data
@Component
@Slf4j
public class SystemProperties {

    private static SystemProperties instance;
    private String systemBaseClassPath = "com.c503.ca";
    /**
     * 系统调试模式
     */
    private boolean debugMode = true;
    /**
     * 系统名称，整个系统的代号，不是各个服务名称
     */
    private String systemName = "cadv";

    /**
     * 文件上传路径
     */
    private String fileUploadPath = "./.data/upload";
    private String fileDownloadPath = "./.data/download";
    private TimeZone businessTimeZone = TimeZoneEnum.CN.value();
    private Locale systemLocale = Locale.SIMPLIFIED_CHINESE;
    private List<String> languageSupport = initLanguageSupport();

    /**
     * fileExpirTime 文件过期时间(秒) 默认10小时
     */
    private long fileExpirTime = 36000;
    /** 生数和注入两个操作的间隔时间（先生数后注入）/毫秒 默认1小时 */
    private long genParameterInterval = 3600000;
    /** 启动流程与生数两个操作的间隔时间（先启动流程后生数）/毫秒 默认1小时 */
    private long startFlowInterval = 3600000;

    public static SystemProperties getInstance() {
        if (instance == null) {
            synchronized (SystemProperties.class) {
                if (instance == null) {
                    instance = SpringContextHolder.getBean(SystemProperties.class);
                }
            }
        }
        return instance;
    }

    private List<String> initLanguageSupport() {
        List<String> languageSupport = new ArrayList<>();
        languageSupport.add(Locale.SIMPLIFIED_CHINESE.toString());
        languageSupport.add(Locale.US.toString());
        return languageSupport;
    }

}
