package com.c503.ca.cadv.common.util;

import com.c503.ca.cadv.common.util.exception.RuntimeWarnException;
import com.google.common.net.InetAddresses;
import org.apache.commons.net.telnet.TelnetClient;

import java.io.IOException;
import java.net.InetAddress;

/**
 * 网络工具类
 *
 * @author liaow
 */
public class NetworkUtils {

    /**
     * 判断IP地址是否可达
     *
     * @param host    目标IP地址
     * @param timeout 等待超时时间
     */
    public static boolean isHostReachable(String host, Integer timeout) {
        try {
            return InetAddress.getByName(host).isReachable(timeout);
        } catch (IOException ignored) {
            System.out.println("判断IP地址是否可达时异常");
        }
        return false;
    }

    /**
     * 判断IP地址是否为组播地址
     *
     * @param ip 字符串IP地址
     */
    public static boolean isMulticaseAddress(String ip) {
        if (StringUtils.isNotEmpty(ip) && InetAddresses.isInetAddress(ip)) {
            InetAddress address = InetAddresses.forString(ip);
            int ipInt = InetAddresses.coerceToInteger(address);
            return ipInt >= 0xE00000FF && ipInt <= 0xEFFFFFFF;
        }

        return false;
    }

    public static boolean telnet(String ip, int port) {
        boolean isConnect;
        try {
            TelnetClient client = new TelnetClient();

            client.setDefaultTimeout(3000);

            client.connect(ip, port);

            isConnect = true;
        } catch (Exception e) {
            throw new RuntimeWarnException("连接失败");
        }

        return isConnect;
    }

}
