package com.c503.ca.cadv.common.util;

import com.c503.ca.cadv.common.util.spring.SpringContextHolder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.*;
import org.springframework.web.client.RestClientException;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.util.UriComponentsBuilder;

import java.util.HashMap;

/**
 * 发送rest请求工具
 *
 * @author liaow
 */
public class RestRequestUtils {

    protected static Logger logger = LoggerFactory.getLogger(RestRequestUtils.class);
    private static RestTemplate restTemplate;

    public static RestTemplate getRestTemplate() {
        if (restTemplate == null) {
            restTemplate = SpringContextHolder.getBean(RestTemplate.class);
        }
        return restTemplate;
    }

    public static <T> ResponseEntity<T> request(String requestMethod, String url,
                                                HashMap<String, Object> params, Class<T> clazz) throws RestClientException {
        HttpMethod httpMethod = resolveHttpMethod(requestMethod);
        return request(httpMethod, url, params, clazz);
    }

    public static <T> ResponseEntity<T> request(HttpMethod requestMethod, String url,
                                                HashMap<String, Object> params, Class<T> clazz) throws RestClientException {

        HttpHeaders requestHeaders = new HttpHeaders();
        requestHeaders.setContentType(MediaType.APPLICATION_JSON);

        HttpEntity<String> requestEntity = new HttpEntity<String>(null, requestHeaders);

        if (requestMethod.equals(HttpMethod.GET) || requestMethod.equals(HttpMethod.DELETE)
            || requestMethod.equals(HttpMethod.PUT)) {
            return get(requestMethod, url, requestEntity, params, clazz);
        } else {
            return post(requestMethod, url, requestEntity, params, clazz);
        }
    }

    private static <T> ResponseEntity<T> post(HttpMethod requestMethod, String url,
                                              HttpEntity<String> requestEntity, HashMap<String, Object> params, Class<T> clazz) {

        requestEntity = new HttpEntity<String>(JacksonUtils.toJsonString(params), requestEntity.getHeaders());

        return getRestTemplate().exchange(url, requestMethod, requestEntity, clazz);
    }

    public static <T> ResponseEntity<T> get(HttpMethod requestMethod, String url,
                                            HttpEntity<String> requestEntity, HashMap<String, Object> params, Class<T> clazz)
        throws RestClientException {

        if (url.contains("{")) {
            StringUtils.remove(url, "{");
            StringUtils.remove(url, "}");
            for (String key : params.keySet()) {
                if (StringUtils.countMatches(url, key) > 1) {
                    logger.error(
                        "接口路径(" + url + ")中，PathVariable关键词(" + key + ")与路径其他内容重复，请检查!!");
                    throw new NullPointerException(
                        "接口路径(" + url + ")中，PathVariable关键词(" + key + ")与路径其他内容重复，请检查!!");
                } else {
                    StringUtils.replace(url, key, (String) params.get(key));
                }
            }
        } else {
            UriComponentsBuilder builder = UriComponentsBuilder.fromHttpUrl(url);
            params.forEach((k, v) -> builder.queryParam(k, v));

            url = builder.toUriString();
        }
        return getRestTemplate().exchange(url, requestMethod, requestEntity, clazz);
    }

    private static HttpMethod resolveHttpMethod(String requestMethod) {
        HttpMethod httpMethod = HttpMethod.resolve(requestMethod.trim().toUpperCase());
        if (httpMethod != null) {
            return httpMethod;
        } else {
            throw new NullPointerException("请求方法错误！");
        }
    }

}
