package com.c503.ca.cadv.common.util;

import com.c503.ca.cadv.common.constant.SecurityConstants;
import com.c503.ca.cadv.common.util.exception.RuntimeWarnException;
import com.c503.ca.cadv.common.util.spring.SpringContextHolder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.crypto.password.PasswordEncoder;

import javax.servlet.http.HttpServletRequest;

/**
 * Utility class for Spring Security.
 *
 * @author liaow
 */
public class SecurityUtils {

    protected static Logger logger = LoggerFactory.getLogger(SecurityUtils.class);

    public static boolean isAdmin(String loginId) {
        return SecurityConstants.ADMIN_LOGIN_ID.equals(loginId);
    }

    /**
     * 根据request获取请求token
     */
    public static String getToken(HttpServletRequest request) {
        if (request == null) {
            return null;
        }
        String token = request.getHeader(SecurityConstants.HEADER);
        if (StringUtils.isNotEmpty(token) && token.startsWith(SecurityConstants.TOKEN_PREFIX)) {
            token = token.replace(SecurityConstants.TOKEN_PREFIX, "");
        }
        return token;
    }

    /**
     * 判断密码是否相同
     *
     * @param rawPassword     真实密码
     * @param encodedPassword 加密后字符
     * @return 结果
     */
    public static boolean matchesPassword(String rawPassword, String encodedPassword) {
        PasswordEncoder passwordEncoder = SpringContextHolder.getBean(PasswordEncoder.class);
        return passwordEncoder.matches(rawPassword, encodedPassword);
    }

    /**
     * 获取用户登录名称
     */
    public static String getUsername() {
        HttpServletRequest request = RequestUtils.getRequest();
        if (request != null) {
            Object obj = request.getAttribute(SecurityConstants.HEADER_USER_NAME);
            if (obj == null) return null;
            return obj.toString();
        }
        return null;
    }

    /**
     * 获取用户ID
     */
    public static String getUserId() {
        HttpServletRequest request = RequestUtils.getRequest();
        if (request != null) {
            Object obj = request.getAttribute(SecurityConstants.HEADER_USER_ID);
            if(obj != null) {
                return obj.toString();
            }
        }
        throw new RuntimeWarnException("当前请求上下文为空，无法获取用户ID！");
    }

    /**
     * 获取请求token
     */
    public static String getToken() {
        HttpServletRequest request = RequestUtils.getRequest();
        if (request != null) {
            return getToken(request);
        }
        throw new RuntimeWarnException("当前请求上下文为空，无法获取Token！");
    }

    /**
     * 生成BCryptPasswordEncoder密码
     *
     * @param password 密码
     * @return 加密字符串
     */
    public static String encryptPassword(String password) {
        BCryptPasswordEncoder passwordEncoder = new BCryptPasswordEncoder();
        return passwordEncoder.encode(password);
    }

}
