package com.c503.ca.cadv.common.util;

import com.google.common.collect.Lists;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.text.StringEscapeUtils;

import java.io.UnsupportedEncodingException;
import java.security.SecureRandom;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 字符串工具类, 继承org.apache.commons.lang.StringUtils类
 */
@Slf4j
@SuppressWarnings({"unused"})
public class StringUtils extends org.apache.commons.lang.StringUtils {
    public static final String SPLIT_DEFAULT = ",";
    public static final String SPLIT_LINE = "-";
    public static final String SPLIT_UNDERLINE = "_";
    public static final String SPLIT_COLON = ":";
    public static final String SPACE = " ";
    public static final String SPLIT_FILE_DEFAULT = "|";
    public static final String SPLIT_FILE = "/";
    public static final Pattern PATTERN = Pattern.compile("<([a-zA-Z]+)[^<>]*>");
    private static final char SEPARATOR = '_';
    private static final String CHARSET_NAME = "UTF-8";

    /**
     * 获取一定位数的随机字符串
     ***/
    public static String getRandomString(int size) {
        char[] c = {'1', '2', '3', '4', '5', '6', '7', '8', '9', '0', 'q', 'w', 'e', 'r', 't', 'y', 'u', 'i', 'o', 'p', 'a', 's', 'd', 'f', 'g', 'h', 'j', 'k', 'l', 'z', 'x', 'c', 'v', 'b', 'n', 'm'};
        SecureRandom random = new SecureRandom();
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < size; ++i) {
            sb.append(c[(Math.abs(random.nextInt()) % c.length)]);
        }

        return sb.toString();
    }

    public static String[] splitDefault(final String str) {
        return split(str, SPLIT_DEFAULT);
    }

    /**
     * 转换为字节数组
     *
     * @param str 字符串
     * @return 字节数组
     */
    public static byte[] getBytes(String str) {
        if (str != null) {
            try {
                return str.getBytes(CHARSET_NAME);
            } catch (UnsupportedEncodingException e) {
                return null;
            }
        } else {
            return null;
        }
    }

    /**
     * 转换为字节数组
     *
     * @param bytes 字节数组
     * @return 字符串
     */
    public static String toString(byte[] bytes) {
        try {
            return new String(bytes, CHARSET_NAME);
        } catch (UnsupportedEncodingException e) {
            return EMPTY;
        }
    }

    /**
     * 是否包含字符串
     *
     * @param str  验证字符串
     * @param strs 字符串组
     * @return 包含返回true
     */
    public static boolean inString(String str, String... strs) {
        if (str != null) {
            for (String s : strs) {
                if (str.equals(trim(s))) {
                    return true;
                }
            }
        }
        return false;
    }

    public static Map<String, String> removeDashKey(Map<String, String> map) {
        Map<String, String> result = new HashMap<>();

        for (Map.Entry<String, String> entry : map.entrySet()) {
            String key = entry.getKey();
            if (key.contains("-")) {
                StringBuilder sb = new StringBuilder(key.length());

                for (int i = 0; i < key.length(); ++i) {
                    if (key.charAt(i) == '-' && i + 1 < key.length() && Character.isAlphabetic(key.charAt(i + 1))) {
                        ++i;
                        char upperChar = Character.toUpperCase(key.charAt(i));
                        sb.append(upperChar);
                    } else {
                        sb.append(key.charAt(i));
                    }
                }

                key = sb.toString();
            }
            result.put(key, entry.getValue());
        }
        return result;
    }

    /**
     * 替换掉HTML标签方法
     */
    public static String replaceHtml(String html) {
        if (isBlank(html)) {
            return "";
        }
        String regEx = "<.+?>";
        Pattern p = Pattern.compile(regEx);
        Matcher m = p.matcher(html);
        return m.replaceAll("");
    }

    /**
     * 缩略字符串（不区分中英文字符）
     *
     * @param str    目标字符串
     * @param length 截取长度
     * @return 缩略结果
     */
    public static String abbr(String str, int length) {
        if (str == null) {
            return "";
        }
        try {
            StringBuilder sb = new StringBuilder();
            int currentLength = 0;
            for (char c : replaceHtml(StringEscapeUtils.unescapeHtml4(str)).toCharArray()) {
                currentLength += String.valueOf(c).getBytes("GBK").length;
                if (currentLength <= length - 3) {
                    sb.append(c);
                } else {
                    sb.append("...");
                    break;
                }
            }
            return sb.toString();
        } catch (UnsupportedEncodingException e) {
            log.warn("缩略字符串失败", e);
        }
        return "";
    }

    public static String abbr2(String param, int length) {
        if (param == null) {
            return "";
        }
        StringBuilder result = new StringBuilder();
        int n = 0;
        char temp;
        boolean isCode = false; // 是不是HTML代码
        boolean isHTML = false; // 是不是HTML特殊字符,如&nbsp;
        for (int i = 0; i < param.length(); i++) {
            temp = param.charAt(i);
            if (temp == '<') {
                isCode = true;
            } else if (temp == '&') {
                isHTML = true;
            } else if (temp == '>' && isCode) {
                n = n - 1;
                isCode = false;
            } else if (temp == ';' && isHTML) {
                isHTML = false;
            }
            try {
                if (!isCode && !isHTML) {
                    n += String.valueOf(temp).getBytes("GBK").length;
                }
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
            }

            if (n <= length - 3) {
                result.append(temp);
            } else {
                result.append("...");
                break;
            }
        }
        // 取出截取字符串中的HTML标记
        String tempResult = result.toString().replaceAll("(>)[^<>]*(<?)", "$1$2");
        // 去掉不需要结素标记的HTML标记
        tempResult = tempResult.replaceAll(
                "</?(AREA|BASE|BASEFONT|BODY|BR|COL|COLGROUP|DD|DT|FRAME|HEAD|HR|HTML|IMG|INPUT|ISINDEX|LI|LINK|META|OPTION|P|PARAM|TBODY|TD|TFOOT|TH|THEAD|TR|area|base|basefont|body|br|col|colgroup|dd|dt|frame|head|hr|html|img|input|isindex|li|link|meta|option|p|param|tbody|td|tfoot|th|thead|tr)[^<>]*/?>",
                "");
        // 去掉成对的HTML标记
        tempResult = tempResult.replaceAll("<([a-zA-Z]+)[^<>]*>(.*?)</\\1>", "$2");
        // 用正则表达式取出标记
        Matcher m = PATTERN.matcher(tempResult);
        List<String> endHTML = Lists.newArrayList();
        while (m.find()) {
            endHTML.add(m.group(1));
        }
        // 补全不成对的HTML标记
        for (int i = endHTML.size() - 1; i >= 0; i--) {
            result.append("</");
            result.append(endHTML.get(i));
            result.append(">");
        }
        return result.toString();
    }

    /**
     * 转换为Double类型
     */
    public static Double toDouble(Object val) {
        if (val == null) {
            return 0D;
        }
        try {
            return Double.valueOf(trim(val.toString()));
        } catch (Exception e) {
            return 0D;
        }
    }

    /**
     * 转换为Float类型
     */
    public static Float toFloat(Object val) {
        return toDouble(val).floatValue();
    }

    /**
     * 转换为Long类型
     */
    public static Long toLong(Object val) {
        return toDouble(val).longValue();
    }

    /**
     * 转换为Integer类型,为空设置默认值
     */
    public static Integer toInteger(Object val, int defaultIntValue) {
        if (val == null) {
            return defaultIntValue;
        }
        return toLong(val).intValue();
    }

    /**
     * 转换为Integer类型
     */
    public static Integer toInteger(Object val) {
        return toLong(val).intValue();
    }

    /**
     * 驼峰命名法工具
     *
     * @return toCamelCase(" hello_world ") == "helloWorld" toCapitalizeCamelCase("hello_world") == "HelloWorld" toUnderScoreCase("helloWorld") = "hello_world"
     */
    public static String toCamelCase(String s) {
        return toCamelCase(s, SEPARATOR);
    }


    public static String toCamelCase(String s, Character... splits) {
        if (s == null) {
            return null;
        }
        StringBuilder sb = new StringBuilder(s.length());
        boolean upperCase = false;
        List<Character> splitList = Lists.newArrayList(splits);
        for (int i = 0; i < s.length(); i++) {
            char c = s.charAt(i);
            if (splitList.contains(c)) {
                upperCase = true;
            } else if (upperCase) {
                sb.append(Character.toUpperCase(c));
                upperCase = false;
            } else {
                sb.append(c);
            }
        }

        return sb.toString();
    }

    /**
     * 驼峰命名法工具
     *
     * @return toCamelCase(" hello_world ") == "helloWorld" toCapitalizeCamelCase("hello_world") == "HelloWorld" toUnderScoreCase("helloWorld") = "hello_world"
     */
    public static String toCapitalizeCamelCase(String s) {
        if (s == null) {
            return null;
        }
        s = toCamelCase(s);
        return s.substring(0, 1).toUpperCase() + s.substring(1);
    }

    /**
     * 首字母小写
     *
     * @return toFirstUpperCase(" HelloWorld ") == "helloWorld"
     */
    public static String toFirstLowerCase(String s) {
        if (s == null) {
            return null;
        }
        char c = s.charAt(0);
        return Character.toLowerCase(c) + s.substring(1);
    }

    /**
     * 驼峰命名法工具
     *
     * @return toCamelCase(" hello_world ") == "helloWorld" toCapitalizeCamelCase("hello_world") == "HelloWorld" toUnderScoreCase("helloWorld") = "hello_world"
     */
    public static String toUnderScoreCase(String s) {
        if (s == null) {
            return null;
        }
        StringBuilder sb = new StringBuilder();
        boolean upperCase = false;
        for (int i = 0; i < s.length(); i++) {
            char c = s.charAt(i);

            boolean nextUpperCase = true;

            if (i < (s.length() - 1)) {
                nextUpperCase = Character.isUpperCase(s.charAt(i + 1));
            }

            if ((i > 0) && Character.isUpperCase(c)) {
                if (!upperCase || !nextUpperCase) {
                    sb.append(SEPARATOR);
                }
                upperCase = true;
            } else {
                upperCase = false;
            }

            sb.append(Character.toLowerCase(c));
        }

        return sb.toString();
    }

    /**
     * 转换为JS获取对象值，生成三目运算返回结果
     *
     * @param objectString 对象串 例如：row.user.id 返回：!row?'':!row.user?'':!row.user.id?'':row.user.id
     */
    public static String jsGetVal(String objectString) {
        StringBuilder result = new StringBuilder();
        StringBuilder val = new StringBuilder();
        String[] vals = split(objectString, ".");
        for (String s : vals) {
            val.append(".").append(s);
            result.append("!").append(val.substring(1)).append("?'':");
        }
        result.append(val.substring(1));
        return result.toString();
    }

    public static String findCommonStartStr(String oriStr, String newStr, Character separate) {
        if (StringUtils.isEmpty(oriStr) || StringUtils.isEmpty(newStr)) {
            return null;
        }
        if (StringUtils.isNotEmpty(oriStr)) {
            String minStr = oriStr.length() > newStr.length() ? newStr : oriStr;
            String maxStr = oriStr.length() > newStr.length() ? oriStr : newStr;
            while (true) {
                if (maxStr.startsWith(minStr)) {
                    break;
                }
                minStr = minStr.substring(0, minStr.length() - 1);
                if (minStr.length() == 0) {
                    break;
                }
            }
            if (StringUtils.isNotEmpty(minStr) && separate != null && separate.equals(minStr.charAt(minStr.length() - 1))) {
                minStr = minStr.substring(0, minStr.length() - 1);
            }
            return minStr;
        }
        return null;
    }

    public static int countMatches(String str, String sub) {
        if (!isEmpty(str) && !isEmpty(sub)) {
            int count = 0;

            for (int idx = 0; (idx = str.indexOf(sub, idx)) != -1; idx += sub.length()) {
                ++count;
            }

            return count;
        } else {
            return 0;
        }
    }

    public static String toAppendStr(String... strs) {
        StringBuilder sb = new StringBuilder();
        for (String str : strs) {
            if (StringUtils.isNotEmpty(str)) {
                sb.append(str);
            }
        }
        return sb.toString();
    }

    public static String toString(Object obj) {
        if (obj != null) {
            return String.valueOf(obj);
        }
        return "";
    }

    /**
     * 获取固定长度的str
     */
    public static String getFixedLengthStr(String s, int length) {
        if (s == null || length <= 0) {
            return "";
        }
        if (s.length() < length) {
            int remain = length - s.length();
            for (int i = 0; i < remain; i++) {
                s += " ";
            }
            return s;
        }

        if (s.length() >= length) {
            return s.substring(0, length);
        }
        return s;
    }

    /**
     * 十六进制字符串转二进制字符串
     *
     * @param hexString
     * @return
     */
    public static String hexStrToBinaryStr(String hexString) {
        if (isEmpty(hexString)) {
            return null;
        }
        StringBuilder builder = new StringBuilder();
        for (int i = 0; i < hexString.length(); i++) {
            String indexStr = hexString.charAt(i) + "";
            String binaryStr = Integer.toBinaryString(Integer.parseInt(indexStr, 16));
            while (binaryStr.length() < 4) {
                binaryStr = "0" + binaryStr;
            }
            builder.append(binaryStr);
        }
        return builder.toString();
    }

    public static String cutStr(String str, int byteLength) {
        String finalStr = "";
        if (str == null || str.getBytes().length <= byteLength) {
            return str == null ? "" : str;
        }
        int sum = 0;
        for (int i = 0; i < str.length(); i++) {
            String s = str.substring(i, i + 1);
            sum += s.getBytes().length;
            if (sum > byteLength) {
                finalStr = str.substring(0, i) + "...";
                break;
            }
        }
        return finalStr;
    }

    public static String convertEmptyStr(String str) {
        if (str == null) {
            return "";
        }
        return str;
    }
}
