package com.c503.ca.cadv.common.util;

import org.apache.poi.hwpf.HWPFDocument;
import org.apache.poi.hwpf.model.StyleDescription;
import org.apache.poi.hwpf.model.StyleSheet;
import org.apache.poi.hwpf.usermodel.*;
import org.apache.poi.poifs.filesystem.POIFSFileSystem;
import org.apache.poi.xwpf.usermodel.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;


/**
 * word文档表格读取工具类
 */

public class WordReaderUtil {


    private static final Logger log = LoggerFactory.getLogger(WordReaderUtil.class);


    /**
     * 读取文档中表格
     * @param filePath 文档路径
     * @param orderNum 设置需要读取的第几个表格
     */
    public static List<List<String>> getWorTableList(String filePath, Integer orderNum){
            if(!filePath.isEmpty()&filePath.toLowerCase(Locale.ENGLISH).endsWith("docx")){
                return readableFromDocx(filePath,orderNum);
            }else if(!filePath.isEmpty()&filePath.toLowerCase(Locale.ENGLISH).endsWith("doc")){
                return readableFromDoc(filePath,orderNum);
            }else{
                log.info("不支持的文档格式");
                return null;
            }

    }



    // 获取word文档标题
    public static List<String> getWordTitles(String filePath) {

        if(!filePath.isEmpty()&filePath.toLowerCase(Locale.ENGLISH).endsWith("docx")){
            return getWordTitlesDocx(filePath);
        }else if(!filePath.isEmpty()&filePath.toLowerCase(Locale.ENGLISH).endsWith("doc")){
            return getWordTitlesDoc(filePath);
        }else{
            log.info("不支持的文档格式");
            return null;
        }
    }





    //根据指定位置获取word文档表格数据-2007之后版本（xxx.docx）
    private static List<List<String>>  readableFromDocx(String filePath, Integer orderNum) {
        try {
            FileInputStream in = new FileInputStream(filePath);//载入文档
            //word 2007 图片不会被读取， 表格中的数据会被放在字符串的最后
            XWPFDocument xwpf = new XWPFDocument(in);//得到word文档的信息
            Iterator<XWPFTable> itpre = xwpf.getTablesIterator();//得到word中的表格
            int total = 0;
            while (itpre.hasNext()) {
                itpre.next();
                total += 1;
            }
            Iterator<XWPFTable> it = xwpf.getTablesIterator();//得到word中的表格
            // 设置需要读取的表格  set是设置需要读取的第几个表格，total是文件中表格的总数
            int set = orderNum;
            int num = set;
            // 过滤前面不需要的表格
            for (int i = 0; i < set-1; i++) {
                it.hasNext();
                it.next();
            }
            List<List<String>> tableList = new ArrayList<>();
            while(it.hasNext()){
                XWPFTable table = it.next();
                //System.out.println("这是第" + num + "个表的数据");
                List<XWPFTableRow> rows = table.getRows();
                //读取每一行数据
                for (int i = 0; i < rows.size(); i++) {
                    XWPFTableRow row = rows.get(i);

                    //读取每一列数据
                    List<XWPFTableCell> cells = row.getTableCells();
                    List<String> rowList = new ArrayList<>();
                    for (int j = 0; j < cells.size(); j++) {
                        XWPFTableCell cell = cells.get(j);
                        rowList.add(cell.getText());

                    }
                    tableList.add(rowList);
                    //System.out.println();
                }
                // 过滤多余的表格
                while (num < total) {
                    it.hasNext();
                    it.next();
                    num += 1;
                }
            }
            //关闭文件流
            in.close();
            return tableList;

        }catch (Exception e){
            e.printStackTrace();
            log.debug("读取文件失败：" + e.getMessage());
            return  null;
        }
    }

    //根据指定位置获取word文档表格数据-2007之后版本（xxx.doc）
    private static List<List<String>>  readableFromDoc(String filePath, Integer orderNum) {
        try {
            FileInputStream in = new FileInputStream(filePath);//载入文档

            // 处理doc格式 即office2003版本
            POIFSFileSystem pfs = new POIFSFileSystem(in);
            HWPFDocument hwpf = new HWPFDocument(pfs);
            Range range = hwpf.getRange();//得到文档的读取范围
            TableIterator itpre = new TableIterator(range);//得到word中的表格
            int total = 0;
            while (itpre.hasNext()) {
                itpre.next();
                total += 1;
            }
            TableIterator it = new TableIterator(range);
            // 迭代文档中的表格
            // 如果有多个表格只读取需要的一个 set是设置需要读取的第几个表格，total是文件中表格的总数
            int set = orderNum;
            int num = set;
            for (int i = 0; i < set-1; i++) {
                it.hasNext();
                it.next();
            }
            List<List<String>> tableList = new ArrayList<>();
            while (it.hasNext()) {
                Table tb = (Table) it.next();
                //System.out.println("这是第" + num + "个表的数据");
                //迭代行，默认从0开始,可以依据需要设置i的值,改变起始行数，也可设置读取到那行，只需修改循环的判断条件即可
                for (int i = 0; i < tb.numRows(); i++) {
                    List<String> rowList = new ArrayList<>();
                    TableRow tr = tb.getRow(i);
                    //迭代列，默认从0开始
                    for (int j = 0; j < tr.numCells(); j++) {
                        TableCell td = tr.getCell(j);//取得单元格
                        //取得单元格的内容
                        for(int k = 0; k < td.numParagraphs(); k++){
                            Paragraph para = td.getParagraph(k);
                            String s = para.text();
                            //去除后面的特殊符号
                            if(null != s && s.length() > 0){
                                s = s.substring(0, s.length()-1);
                            }
                            rowList.add(s);
                            // System.out.print(s+"["+i+","+j+"]" + "\t");
                        }
                    }
                    tableList.add(rowList);
                    // System.out.println();
                }
                // 过滤多余的表格
                while (num < total) {
                    it.hasNext();
                    it.next();
                    num += 1;
                }
            }
            //关闭文件流
            in.close();
            return tableList;
        }catch (Exception e){
            e.printStackTrace();
            log.debug("读取文件失败：" + e.getMessage());
            return  null;
        }
    }


    // 获取2007版word文档的标题
    public static List<String> getWordTitlesDocx(String path) {
        try {
            InputStream in = new FileInputStream(path);
            List<String> list = new ArrayList<String>();
            XWPFDocument doc = new XWPFDocument(in);
            List<XWPFParagraph> paras = doc.getParagraphs();
            for (XWPFParagraph graph : paras) {
                String text = graph.getParagraphText();
                String style = graph.getStyle();
                if ("1".equals(style)) {
                    list.add(text);
                }else if ("2".equals(style)) {
                    list.add(text);
                }else if ("3".equals(style)) {
                    list.add(text);
                }else{
                    continue;
                }
            }
            in.close();
            return list;
        }catch (Exception e){
            e.printStackTrace();
            log.debug("读取文件失败：" + e.getMessage());
            return null;
        }
    }


    // 获取2003版word文档的标题
    public static List<String> getWordTitlesDoc(String path) {
        try {
            InputStream in = new FileInputStream(path);
            HWPFDocument doc = new HWPFDocument(in);
            Range r = doc.getRange();
            List<String> list = new ArrayList<String>();
            List<String> titles = new ArrayList<String>();
            List<String> context = new ArrayList<String>();
            for (int i = 0; i < r.numParagraphs(); i++) {
                Paragraph p = r.getParagraph(i);
                // check if style index is greater than total number of styles
                int numStyles = doc.getStyleSheet().numStyles();
                int styleIndex = p.getStyleIndex();
                String contexts = p.text();
                list.add(contexts); // 标题+内容
                if (numStyles > styleIndex) {
                    StyleSheet style_sheet = doc.getStyleSheet();
                    StyleDescription style = style_sheet.getStyleDescription(styleIndex);
                    String styleName = style.getName();
                    if (styleName != null && styleName.contains("标题")) {
                        String text = p.text();
                        titles.add(text);
                    } else if (styleName != null && styleName.contains("正文")) {
                        String text = p.text();
                        context.add(text);
                    }
                }
            }
            in.close();
            return titles;
        }catch (Exception e){
            e.printStackTrace();
            log.debug("读取文件失败：" + e.getMessage());
            return null;
        }
    }


    //根据的表头获取word文档表格数据-2007之后版本（xxx.docx）
    public static List<List<String>>  readableByTitleDocx(String filePath,String[] titles) {
        try {
            FileInputStream in = new FileInputStream(filePath);//载入文档
            //word 2007 图片不会被读取， 表格中的数据会被放在字符串的最后
            XWPFDocument xwpf = new XWPFDocument(in);//得到word文档的信息

            Iterator<XWPFTable> it = xwpf.getTablesIterator();//得到word中的表格


            List<List<String>> tableList = new ArrayList<>();
            boolean isTitleSuit=false;
            while(it.hasNext()){
                XWPFTable table = it.next();
                //System.out.println("这是第" + num + "个表的数据");
                List<XWPFTableRow> rows = table.getRows();
                //读取每一行数据

                isTitleSuit=checkTitle(titles,rows.get(0));
                for (int i = 0; i < rows.size(); i++) {
                    XWPFTableRow row = rows.get(i);
                    //读取每一列数据
                    if(isTitleSuit){
                        List<XWPFTableCell> cells = row.getTableCells();
                        List<String> rowList = new ArrayList<>();
                        for (int j = 0; j < cells.size(); j++) {
                            XWPFTableCell cell = cells.get(j);
                            rowList.add(cell.getText());

                        }
                        tableList.add(rowList);
                    }

                }

            }
            //关闭文件流
            in.close();
            return tableList;

        }catch (Exception e){
            e.printStackTrace();
            log.debug("读取文件失败：" + e.getMessage());
            return  null;
        }
    }


    //校验title是否匹配
    public static boolean   checkTitle(String[] titles,XWPFTableRow row){
        boolean isChecked=true;
        if( row.getTableCells().size()==titles.length){
            List<XWPFTableCell> cells = row.getTableCells();
            for (int j = 0; j < cells.size(); j++) {
                XWPFTableCell cell = cells.get(j);
                if(!titles[j].equals(cell.getText().trim())){
                    isChecked=false;
                }
            }
        }else{
            isChecked= false;
        }
       return isChecked;
    }


    }
