package com.c503.ca.cadv.common.util;

import java.io.*;
import java.util.List;

import net.lingala.zip4j.core.ZipFile;
import net.lingala.zip4j.exception.ZipException;
import net.lingala.zip4j.model.FileHeader;
import net.lingala.zip4j.model.ZipParameters;
import net.lingala.zip4j.util.Zip4jConstants;

import org.apache.commons.compress.archivers.tar.TarArchiveEntry;
import org.apache.commons.compress.archivers.tar.TarArchiveInputStream;
import org.apache.commons.compress.utils.IOUtils;
import org.apache.commons.compress.utils.Lists;
import org.springframework.web.multipart.MultipartFile;

/**
 * 压缩解压缩文件 工具类
 *
 * @author zxh
 *
 */
public class ZipUtil {

    /**
     * 压缩文件 （压缩后 删除原有的文件）
     *
     * @param path
     *            压缩文件\文件夹路径
     */
    public static File zip(String path) {
        return zip(path,true);
    }

    /**
     * 压缩文件
     *
     * @param path
     *            压缩文件\文件夹路径
     * @param isDelete
     *            压缩后是否删除原文件\文件夹
     */
    public static File zip(String path, Boolean isDelete) {
        ZipFile zipFile = null;
        try {
            ZipParameters parameters = new ZipParameters();
            parameters.setCompressionMethod(Zip4jConstants.COMP_DEFLATE);
            parameters.setCompressionLevel(Zip4jConstants.DEFLATE_LEVEL_NORMAL);
            File file = new File(path);
            if (file.isDirectory()) {
                zipFile = new ZipFile(new File(path + ".zip"));
                zipFile.setFileNameCharset("utf-8");
                zipFile.addFolder(path, parameters);
            } else {
                zipFile = new ZipFile(new File(path.split(".")[0] + ".zip"));
                zipFile.setFileNameCharset("utf-8");
                zipFile.addFile(file, parameters);
            }

            if (isDelete) {
                FileUtils.deleteDir(file);
            }
            return zipFile.getFile();
        } catch (ZipException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 压缩文件夹(加密)
     *
     * @param path
     *            压缩文件\文件夹路径
     * @param isDelete
     *            压缩后是否删除原文件\文件夹
     * @param password
     *            加密密码
     */
    public static void zipSetPass(String path, Boolean isDelete, String password) {
        ZipFile zipFile = null;
        try {
            ZipParameters parameters = new ZipParameters();
            parameters.setCompressionMethod(Zip4jConstants.COMP_DEFLATE);
            parameters.setCompressionLevel(Zip4jConstants.DEFLATE_LEVEL_NORMAL);
            // 设置密码
            parameters.setEncryptFiles(true);
            parameters.setEncryptionMethod(Zip4jConstants.ENC_METHOD_STANDARD);
            parameters.setPassword(password);
            File file = new File(path);
            if (file.isDirectory()) {
                zipFile = new ZipFile(new File(path + ".zip"));
                zipFile.setFileNameCharset("utf-8");
                zipFile.addFolder(path, parameters);
            } else {
                zipFile = new ZipFile(new File(path.split(".")[0] + ".zip"));
                zipFile.setFileNameCharset("utf-8");
                zipFile.addFile(file, parameters);
            }
            if (isDelete) {
                FileUtils.deleteDir(new File(path));
            }
        } catch (ZipException e) {
            e.printStackTrace();
        }

    }

    /**
     * 解压压缩文件
     *
     * @param filePath
     *            压缩文件路径
     * @param toPath
     *            解压到的文件夹路径
     * @param password
     *            密码 没有密码设置为""
     */
    public static String unZip(String filePath, String toPath, String password) {
        try {
            ZipFile zipFile = new ZipFile(filePath);
            zipFile.setFileNameCharset("utf-8");
            return	unZipFile(zipFile, toPath, password);
        } catch (ZipException e) {
            e.printStackTrace();
        }
        return "";
    }


    /**
     *	解压文件(导入导出专用)
     * @throws IOException
     * @throws IllegalStateException
     */
    @SuppressWarnings("unchecked")
    public static List<String> unZipFile(MultipartFile multipartFile, String toPath) throws Exception {
        //转存文件
        String originalFilename = multipartFile.getOriginalFilename();
        String destPath = toPath + originalFilename;

        createFilePath(destPath, originalFilename);
        File file = new File(destPath);
        if (file.exists())
            file.delete();
        multipartFile.transferTo(file);
        ZipFile zipFile = new ZipFile(file);
        zipFile.setFileNameCharset("GBK");       // 设置文件名编码，在GBK系统中需要设置
        if (zipFile.isEncrypted())
            zipFile.setPassword("");
        if (!zipFile.isValidZipFile())
            throw new ZipException("压缩文件不合法,可能被损坏.");
        List<String> fileDir = Lists.newArrayList();
        zipFile.extractAll(toPath);      // 将文件抽出到解压目录(解压)

        List<FileHeader> fileHeaderList = zipFile.getFileHeaders();
        for (FileHeader fileHeader : fileHeaderList) {
            String dirName =fileHeader.getFileName();
            if(!fileHeader.isDirectory()){
                //				if(dirName.endsWith("\\"))//如果是widow下的压缩的
                //					fileDir = dirName.substring(0, dirName.lastIndexOf("\\"));
                //				else if(dirName.endsWith("/")) //如果是linux下的压缩的
                //					fileDir = dirName.substring(0, dirName.lastIndexOf("/"));
                //				else//如果都不是有问题。用系统替换
                //					fileDir = dirName.substring(0, dirName.lastIndexOf(File.separator));
                fileDir.add(dirName);
            }
        }
        //删除解压文件
        FileUtils.deleteDir(file);
        //返回文件的路径
        return  fileDir;
    }


    /**
     * 解压压缩文件
     * @param multipartFile
     * @param toPath
     * @param password
     */
    public static void unZipFile(MultipartFile multipartFile, String toPath,
                                 String password) {
        String originalFilename = multipartFile.getOriginalFilename();
        String destPath = toPath + originalFilename;
        try {
            createFilePath(destPath, originalFilename);
            File file = new File(destPath);
            if (file.exists())
                file.delete();
            multipartFile.transferTo(file);
            unZipFile(new ZipFile(file), toPath, password);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 解压压缩文件
     * @param zipFile
     * @param toPath
     * @param password
     */
    public static String unZipFile(ZipFile zipFile, String toPath, String password) {
        try {
            String filePath = "";
            if (zipFile.isEncrypted()) {
                zipFile.setPassword(password);
            }
            List<?> fileHeaderList = zipFile.getFileHeaders();
            for (Object o : fileHeaderList) {
                FileHeader fileHeader = (FileHeader) o;
                zipFile.extractFile(fileHeader, toPath);
                filePath = fileHeader.getFileName();
            }
            if(filePath.contains("/")) {
                return filePath.substring(0,filePath.indexOf("/"));
            }
        } catch (ZipException e) {
            e.printStackTrace();
        }
        return "";
    }
    /**
     * 创建文件目录
     *
     * @param tempPath
     * @param fileName
     *            文件名称
     * @return 文件的完整目录
     */
    public static String createFilePath(String tempPath, String fileName) {
        File file = new File(tempPath);
        // 文件夹不存在创建
        if (!file.exists())
            file.mkdirs();
        return file.getPath() + File.separator + fileName;
    }

    /**
     * 解压tar
     *
     * @param tarGzFile tarGzFile
     * @param destDir   destDir
     */
    public static void deCompressGZipFile(String tarGzFile, String destDir) {
        //建立输出流，用于将从压缩文件中读出的文件写如磁盘
        TarArchiveEntry entry;
        TarArchiveEntry[] subEntries;
        File subEntryFile = null;
        try (FileInputStream fis = new FileInputStream(tarGzFile);
             //             GZIPInputStream gis = new GZIPInputStream(fis);
             TarArchiveInputStream taris = new TarArchiveInputStream(fis);) {
            while ((entry = taris.getNextTarEntry()) != null) {
                StringBuilder entryFileName = new StringBuilder();
                entryFileName.append(destDir).append(File.separator).append(entry.getName());
                File entryFile = new File(entryFileName.toString());
                if (entry.isDirectory()) {
                    if (!entryFile.exists())
                        entryFile.mkdirs();
                    subEntries = entry.getDirectoryEntries();
                    for (int i = 0; i < subEntries.length; i++) {
                        try (OutputStream out = new FileOutputStream(subEntryFile)) {
                            subEntryFile = new File(entryFileName + File.separator + subEntries[i].getName());
                            IOUtils.copy(taris, out);
                        } catch (Exception e) {
                            e.printStackTrace();
                        }
                    }
                } else {
                    checkFileExists(entryFile);
                    OutputStream out = new FileOutputStream(entryFile);
                    IOUtils.copy(taris, out);
                    out.close();
                    //如果时gz文件进行递归解压
                    if (entryFile.getName().endsWith(".gz")) {
                        deCompressGZipFile(entryFile.getPath(), destDir);
                    }
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void checkFileExists(File file) {
        //判断是否是目录
        if (file.isDirectory()) {
            if (!file.exists()) {
                file.mkdirs();
            }
        } else {
            //判断父目录是否存在，如果不存在，则创建
            if (file.getParentFile() != null && !file.getParentFile().exists())
                file.getParentFile().mkdirs();
            try {
                file.createNewFile();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

}
