package com.c503.ca.cadv.common.util.domain;

import com.c503.ca.cadv.common.util.LocaleMessageUtils;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonPropertyOrder;
import org.assertj.core.util.Lists;
import org.springframework.http.HttpStatus;
import org.springframework.util.ObjectUtils;

import java.io.Serializable;
import java.util.List;

/**
 * 基础控制器支持类
 */
@JsonPropertyOrder({"status", "code", "message", "data"})
public class CustomMessage<T> implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * 消息状态  0信息 1成功 2警告 3错误
     */
    private int status;
    private StatusEmun statusEmun;

    /**
     * http请求状态码
     */
    private HttpStatus httpStatus;

    private String[] messages = {};
    private T data;

    public CustomMessage() {
    }

    public CustomMessage(T data, StatusEmun statusEmun, String... messages) {
        this.setStatusEmun(statusEmun);
        this.messages = translateMessages(messages);
        this.data = data;
    }

    public static <T> CustomMessage<T> create(T data, StatusEmun statusEmunEmun, String... message) {
        return new CustomMessage<>(data, statusEmunEmun, message);
    }

    public static <T> CustomMessage<T> createSuccessData(T data, String... message) {
        return create(data, StatusEmun.MSG_TYPE_SUCCESS, message);
    }

    public static <T> CustomMessage<T> createSuccess(String... message) {
        return create(null, StatusEmun.MSG_TYPE_SUCCESS, message);
    }

    public static <T> CustomMessage<T> createWarn(T data, String... message) {
        return create(data, StatusEmun.MSG_TYPE_WARNING, message);
    }

    public static <T> CustomMessage<T> createWarn(String... message) {
        return create(null, StatusEmun.MSG_TYPE_WARNING, message);
    }

    public static <T> CustomMessage<T> createError(T data, String... messages) {
        return create(data, StatusEmun.MSG_TYPE_ERROR, messages);
    }

    public static <T> CustomMessage<T> createError(String... messages) {
        return create(null, StatusEmun.MSG_TYPE_ERROR, messages);
    }

    private String[] translateMessages(String[] messages) {
        List<String> messageList = Lists.newArrayList();
        for (String message : messages) {
            messageList.add(LocaleMessageUtils.get(message));
        }
        this.messages = messageList.toArray(new String[0]);
        return this.messages;
    }

    public String getMessage() {
        return readMessages();
    }

    public void setMessage(String message) {
        addMessage(message);
    }

    public int getStatus() {
        return status;
    }

    public void setStatus(int status) {
        this.status = status;
        this.statusEmun = StatusEmun.valueOf(status);
    }


    public String readMessages() {
        StringBuilder sb = new StringBuilder();
        for (String message : messages) {
            sb.append(message);
        }
        return sb.toString();
    }

    public void addMessage(String message) {
        String localeMessage = LocaleMessageUtils.get(message);
        this.messages = ObjectUtils.addObjectToArray(messages, localeMessage);
    }

    @JsonIgnore
    public HttpStatus getHttpStatus() {
        return httpStatus;
    }

    public CustomMessage<T> setHttpStatus(HttpStatus status) {
        this.httpStatus = status;
        return this;
    }

    @JsonIgnore
    public StatusEmun getStatusEmun() {
        if (this.statusEmun == null) {
            this.statusEmun = StatusEmun.valueOf(status);
        }

        return this.statusEmun;
    }

    public void setStatusEmun(StatusEmun statusEmun) {
        this.statusEmun = statusEmun;
        this.status = statusEmun.getStatus();
    }

    @JsonIgnore
    public String getStatusName() {
        return this.statusEmun.name();
    }

    public T getData() {
        return data;
    }

    public void setData(T data) {
        this.data = data;
    }

}
