package com.c503.ca.cadv.common.util.domain;

import com.c503.ca.cadv.common.util.LocaleMessageUtils;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;

/**
 * 对象验证返回对象
 *
 * @author liaow
 */
@Data
@NoArgsConstructor
@AllArgsConstructor
public class VerifyResult<T> {

    private String separator = ",";
    private boolean success;
    private String message;
    private StringBuilder messageBuilder = new StringBuilder();
    private T data;

    public VerifyResult(boolean success) {
        this.success = success;
    }

    public VerifyResult(boolean success, String initMessage) {
        this(success);
        this.message = LocaleMessageUtils.get(initMessage);
        if (initMessage != null) {
            this.messageBuilder = new StringBuilder(message);
        }
    }

    public VerifyResult(boolean success, String initMessage, String separator) {
        this(success, initMessage);
        this.separator = separator;
    }

    public void addMessage(String message) {
        String localeMessage = LocaleMessageUtils.get(message);
        this.messageBuilder.append(localeMessage).append(separator);
    }

    public String getMessage() {
        if (this.messageBuilder.length() > 1 &&
            this.separator.equals(String.valueOf(messageBuilder.charAt(messageBuilder.length() - 1)))) {
            //裁剪分隔符
            messageBuilder.deleteCharAt(messageBuilder.length() - 1);
        }
        return this.messageBuilder.toString();
    }

    public void setMessage(String message) {
        this.messageBuilder = new StringBuilder(message);
    }

}
