package com.c503.ca.cadv.common.util.jackson;

import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.databind.SerializerProvider;
import com.fasterxml.jackson.databind.ser.std.StdSerializer;
import lombok.extern.slf4j.Slf4j;
import org.springframework.boot.jackson.JsonComponent;

import java.io.IOException;
import java.time.format.DateTimeFormatter;
import java.time.format.DateTimeFormatterBuilder;
import java.time.temporal.ChronoUnit;
import java.util.Date;

/**
 * 自定义 Date 类型的序列化
 *
 * @author liaow
 */
@JsonComponent
@Slf4j
public class DateSerializer extends StdSerializer<Date> {

    private static final DateTimeFormatter ISO_INSTANT;

    static {
        ISO_INSTANT = new DateTimeFormatterBuilder().appendInstant(3).toFormatter();
    }

    protected DateSerializer(Class<Date> t) {
        super(t);
    }

    public DateSerializer() {
        this(null);
    }

    @Override
    public void serialize(Date obj, JsonGenerator gen, SerializerProvider provider) {
        try {
            String resultVal = ISO_INSTANT.format(obj.toInstant().truncatedTo(ChronoUnit.MILLIS));
            gen.writeObject(resultVal);
        } catch (IOException e) {
            log.error("serialize error:{}", e.toString());
        }
    }

}
