package com.c503.ca.cadv.common.util.orika;

import com.c503.ca.cadv.common.util.spring.SpringContextHolder;
import ma.glasnost.orika.MapperFacade;
import ma.glasnost.orika.MapperFactory;
import ma.glasnost.orika.impl.DefaultMapperFactory;
import ma.glasnost.orika.property.RegexPropertyResolver;
import net.rakugakibox.spring.boot.orika.OrikaProperties;

/**
 * Orika Bean组件获取帮助类
 *
 * @author liaow
 * @since 2020/10/22
 */
public class OrikaBeanHelper {

    private volatile static MapperFacade mapperFacade;
    private volatile static MapperFacade mapperFacadeNoMapNulls;
    private volatile static MapperFactory orikaMapperFactory;
    private volatile static MapperFactory orikaMapperFactory2;
    private volatile static OrikaProperties orikaProperties;

    public static MapperFacade getMapperFacade() {
        if (mapperFacade == null) {
            synchronized (OrikaBeanHelper.class) {
                if (mapperFacade == null) {
                    mapperFacade = SpringContextHolder.getBean(MapperFacade.class);
                }
            }
        }
        return mapperFacade;
    }

    public static MapperFacade getMapperFacadeNoMapNulls() {
        if (mapperFacadeNoMapNulls == null) {
            synchronized (OrikaBeanHelper.class) {
                if (mapperFacadeNoMapNulls == null) {
                    MapperFactory mapperFactory = orikaMapperFactoryBuilder().mapNulls(false).build();
                    mapperFacadeNoMapNulls = mapperFactory.getMapperFacade();
                }
            }
        }
        return mapperFacadeNoMapNulls;
    }

    private static MapperFactory getOrikaMapperFactory() {
        if (orikaMapperFactory == null) {
            synchronized (OrikaBeanHelper.class) {
                if (orikaMapperFactory == null) {
                    orikaMapperFactory = SpringContextHolder.getBean(MapperFactory.class);
                }
            }
        }
        return orikaMapperFactory;
    }

    private static MapperFactory getOrikaMapperFactory2() {
        if (orikaMapperFactory2 == null) {
            synchronized (OrikaBeanHelper.class) {
                if (orikaMapperFactory2 == null) {
                    orikaMapperFactory2 = new DefaultMapperFactory.Builder().propertyResolverStrategy(
                            new RegexPropertyResolver("get|is|has([^h][\\w]+)",
                                    "set([^h][\\w]+)", true, true)
                    ).build();
                }
            }
        }
        return orikaMapperFactory2;
    }

    private static OrikaProperties getOrikaProperties() {
        if (orikaProperties == null) {
            synchronized (OrikaBeanHelper.class) {
                if (orikaProperties == null) {
                    orikaProperties = SpringContextHolder.getBean(OrikaProperties.class);
                }
            }
        }
        return orikaProperties;
    }

    public static DefaultMapperFactory.MapperFactoryBuilder<?, ?> orikaMapperFactoryBuilder() {
        //初始化使用配置内容
        OrikaProperties orikaProperties = getOrikaProperties();
        DefaultMapperFactory.Builder orikaMapperFactoryBuilder = new DefaultMapperFactory.Builder();
        if (orikaProperties.getUseBuiltinConverters() != null) {
            orikaMapperFactoryBuilder.useBuiltinConverters(orikaProperties.getUseBuiltinConverters());
        }
        if (orikaProperties.getUseAutoMapping() != null) {
            orikaMapperFactoryBuilder.useAutoMapping(orikaProperties.getUseAutoMapping());
        }
        if (orikaProperties.getMapNulls() != null) {
            orikaMapperFactoryBuilder.mapNulls(orikaProperties.getMapNulls());
        }
        if (orikaProperties.getDumpStateOnException() != null) {
            orikaMapperFactoryBuilder.dumpStateOnException(orikaProperties.getDumpStateOnException());
        }
        if (orikaProperties.getFavorExtension() != null) {
            orikaMapperFactoryBuilder.favorExtension(orikaProperties.getFavorExtension());
        }
        if (orikaProperties.getCaptureFieldContext() != null) {
            orikaMapperFactoryBuilder.captureFieldContext(orikaProperties.getCaptureFieldContext());
        }

        return orikaMapperFactoryBuilder;
    }


}
