package com.c503.ca.cadv.common.event;

import com.c503.ca.cadv.common.util.StringUtils;
import com.c503.ca.cadv.enhance.redis.util.RedisUtils;
import org.springframework.stereotype.Component;
import org.springframework.transaction.event.TransactionPhase;
import org.springframework.transaction.event.TransactionalEventListener;

/**
 * 事务监听器
 * 缓存相关处理
 *
 * @author liaow
 */
@Component
public class CacheEventListener {

    /**
     * 事务提交后，清理缓存
     */
    @TransactionalEventListener(phase = TransactionPhase.AFTER_COMMIT)
    public void cacheEvictEvent(CacheEvictEvent event) {
        //通知客户端
        String cacheKey = event.getCacheKey();
        String hashKey = event.getHashKey();
        if (StringUtils.isEmpty(hashKey)) {
            RedisUtils.del(cacheKey);
        } else {
            RedisUtils.hDel(cacheKey, hashKey);
        }
    }

}
