package com.c503.ca.cadv.common.mvc.service;

import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.c503.ca.cadv.common.mvc.QueryDetail;
import com.c503.ca.cadv.common.mvc.domain.DataEntity;
import com.c503.ca.cadv.common.mvc.domain.vo.DataEntityVO;
import com.c503.ca.cadv.common.mvc.repository.BaseRepository;
import com.c503.ca.cadv.common.util.BeanConvertUtils;
import com.c503.ca.cadv.common.util.StringUtils;
import com.c503.ca.cadv.common.util.domain.PageModel;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.springframework.transaction.annotation.Transactional;

import java.io.Serializable;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;

/**
 * Service基类
 */
@Data
@EqualsAndHashCode(callSuper = true)
public abstract class DataVOService<Repository extends BaseRepository<T>, T extends DataEntity, V extends DataEntityVO>
    extends BaseService<Repository, T> {

    private Class<V> entityVOClz;

    public DataVOService() {
        super();
        Class<?> c = getClass();
        Type type = c.getGenericSuperclass();
        if (type instanceof ParameterizedType) {
            Type[] parameterizedType = ((ParameterizedType) type).getActualTypeArguments();
            entityVOClz = (Class<V>) parameterizedType[2];
        }
    }

    public PageModel<V> findVOPage(PageModel<V> pm) {
        PageModel<T> pageModel = findPage(new PageModel<>(pm));
        List<T> list = pageModel.getData();
        List<V> listVO = copyBeanToVO(list);
        pm.setData(listVO);
        pm.setRecordsTotal(pageModel.getRecordsTotal());
        return pm;
    }

    @Transactional(readOnly = true, rollbackFor = Exception.class)
    public V findOneVO(Serializable id) {
        return copyBeanToVO(findOne(id));
    }

    public void copyBeanToVO(T module, V result) {
        if (result != null && module != null) {
            BeanConvertUtils.map(module, result);
        }
    }

    public V copyBeanToVO(T module) {
        V result = null;
        if (module != null) {
            try {
                result = entityVOClz.newInstance();
                copyBeanToVO(module, result);
            } catch (Exception e) {
                log.error("{}", e.toString());
            }
        }
        return result;
    }

    public List<V> copyBeanToVO(List<T> modules) {
        return BeanConvertUtils.mapAsList(modules, entityVOClz);
    }

    public void copyVOToBean(V form, T entity) {
        if (form != null && entity != null) {
            //VO To Bean 不映射null
            BeanConvertUtils.map(form, entity, false);
        }
    }

    public T copyVOToBean(V form) {
        T result = null;
        if (form != null && getEntityClass() != null) {
            try {
                result = getEntityClass().newInstance();
                copyVOToBean(form, result);
            } catch (Exception e) {
                log.error("{}", e.toString());
            }
        }
        return result;
    }

    public boolean save(V form) {
        T newEntity = null;
        try {
            newEntity = getEntityClass().newInstance();
            if (form != null && form.getId() != null && StringUtils.isNotEmpty(form.getId().toString())) {
                T originEntity = super.findOne(form.getId());
                if (originEntity != null) {
                    //深拷贝一次，避免直接使用originEntity导致后续的引用问题
                    BeanConvertUtils.map(originEntity, newEntity);
                }
            }

            copyVOToBean(form, newEntity);
        } catch (Exception e) {
            log.warn("{}", e.toString());
        }
        boolean success = saveOrUpdate(newEntity);

        if (success) {
            copyBeanToVO(newEntity, form);
        }
        return success;
    }

    public boolean saveOrUpdate(V form) {
        return this.save(form);
    }

    @Transactional(readOnly = true, rollbackFor = Exception.class)
    public List<V> findAllVO() {
        List<V> resultVOs = new ArrayList<>();
        List<T> result = findAll();
        if (result != null && !result.isEmpty()) {
            resultVOs = BeanConvertUtils.mapAsList(result, entityVOClz);
        }
        return resultVOs;
    }

    @Transactional(readOnly = true, rollbackFor = Exception.class)
    public List<V> findAllVO(QueryDetail<T> queryDetail) {
        return findAllVO(queryDetail.getWrapper());
    }

    @Transactional(readOnly = true, rollbackFor = Exception.class)
    public List<V> findAllVO(Wrapper<T> wrapper) {
        List<V> resultVOs = new ArrayList<>();
        List<T> result = super.findAll(wrapper);
        if (result != null && !result.isEmpty()) {
            resultVOs = BeanConvertUtils.mapAsList(result, entityVOClz);
        }
        return resultVOs;
    }

}
