package com.c503.ca.cadv.common.mvc.service;

import com.c503.ca.cadv.common.mvc.domain.TreeEntity;
import com.c503.ca.cadv.common.mvc.domain.vo.TreeEntityVO;
import com.c503.ca.cadv.common.mvc.repository.TreeRepository;
import com.c503.ca.cadv.common.util.BeanConvertUtils;
import com.c503.ca.cadv.common.util.StringUtils;
import com.c503.ca.cadv.common.util.domain.PageModel;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.transaction.annotation.Transactional;

import java.io.Serializable;
import java.lang.reflect.ParameterizedType;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

/**
 * 树类型VOService类
 */
@Data
@EqualsAndHashCode(callSuper = true)
@Slf4j
public class TreeVOService<Repository extends TreeRepository<T>,
    T extends TreeEntity, V extends TreeEntityVO> extends TreeService<Repository, T> {

    private Class<V> entityVOClz;

    public TreeVOService() {
        super();
        Class<?> c = getClass();
        Type type = c.getGenericSuperclass();
        if (type instanceof ParameterizedType) {
            Type[] parameterizedType = ((ParameterizedType) type).getActualTypeArguments();
            entityVOClz = (Class<V>) parameterizedType[2];
        }
    }

    public PageModel<V> findVOPage(PageModel<V> pm) {
        PageModel<T> pageModel = findPage(new PageModel<>(pm));
        List<T> list = pageModel.getData();
        List<V> listVO = copyBeanToVO(list);
        pm.setData(listVO);
        pm.setRecordsTotal(pageModel.getRecordsTotal());
        return pm;
    }

    @Transactional(readOnly = true, rollbackFor = Exception.class)
    public V findOneVO(Serializable id) {
        T entity = findTreeOne(id);
        if (entity != null && entity.getParentId() != null) {
            entity.setParent(findOne(entity.getParentId()));
        }
        return copyBeanToVO(entity);
    }

    public void copyBeanToVO(T module, V result) {
        if (result != null && module != null) {
            BeanConvertUtils.map(module, result);
            if (module.getParent() != null) {
                result.setParentName(module.getParent().getName());
            }
        }
    }

    public V copyBeanToVO(T module) {
        V result = null;
        if (module != null) {
            try {
                result = entityVOClz.newInstance();
                copyBeanToVO(module, result);
            } catch (Exception e) {
                log.error("{}", e.toString());
            }
        }
        return result;
    }

    public List<V> copyBeanToVO(List<T> modules) {
        List<V> result = new ArrayList<>();
        if (modules != null && !modules.isEmpty()) {
            for (T module : modules) {
                V resultOne = copyBeanToVO(module);
                result.add(resultOne);
            }
        }
        return result;
    }

    public void copyVOToBean(V form, T entity) {
        if (form != null && entity != null) {
            //VO To Bean 不映射null
            BeanConvertUtils.map(form, entity, false);
        }
    }

    public T copyVOToBean(V form) {
        T result = null;
        if (form != null && getEntityClass() != null) {
            try {
                result = getEntityClass().newInstance();
                copyVOToBean(form, result);
            } catch (Exception e) {
                log.error("{}", e.toString());
            }
        }
        return result;
    }

    @SneakyThrows
    public boolean save(V form) {
        T newEntity;
        try {
            newEntity = getEntityClass().newInstance();
            if (StringUtils.isNotEmpty(form.getId())) {
                T originEntity = super.findOne(form.getId());
                if (originEntity != null) {
                    BeanConvertUtils.map(originEntity, newEntity);
                }
            }

            copyVOToBean(form, newEntity);
        } catch (Exception e) {
            log.warn("{}", e.toString());
            throw e;
        }
        boolean success = saveOrUpdate(newEntity);
        if (success) {
            copyBeanToVO(newEntity, form);
        }
        return success;
    }

    public boolean saveOrUpdate(V form) {
        return this.save(form);
    }

    @Transactional(readOnly = true, rollbackFor = Exception.class)
    public Optional<V> findOptionalTopByParentId(String parentId) {
        List<T> tempList = super.findByParentIdOrderBySortDesc(parentId);
        if (tempList != null && !tempList.isEmpty()) {
            T entity = tempList.get(0);
            entity.setParent(getById(entity.getParentId()));
            return Optional.of(copyBeanToVO(entity));
        }
        return Optional.empty();
    }

}
