package com.c503.ca.cadv.common.mvc.util;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.c503.ca.cadv.common.mvc.QueryDetail;
import com.c503.ca.cadv.common.util.DateUtils;
import com.c503.ca.cadv.common.util.StringUtils;
import com.c503.ca.cadv.common.constant.SystemConstants;
import com.c503.ca.cadv.common.util.domain.QueryCondition;
import com.google.common.collect.Lists;
import lombok.SneakyThrows;
import org.apache.commons.lang3.math.NumberUtils;
import org.springframework.data.domain.Sort;

import java.util.Collection;
import java.util.Iterator;
import java.util.List;

/**
 * QueryWrapper条件转换工具
 *
 * @author liaow
 */
@SuppressWarnings({"unchecked", "rawtypes", "unused"})
public class QueryWrapperUtil {

    private static Collection handlerQueryConditionCollectionValue(QueryCondition queryCondition) {
        Collection col = null;
        if (queryCondition.getValue() instanceof String) {
            String val = String.valueOf(queryCondition.getValue());
            col = val.contains(",") ? Lists.newArrayList(val.split(","))
                : Lists.newArrayList(val);
        }
        if (queryCondition.getValue() instanceof Collection) {
            col = (Collection) queryCondition.getValue();
        }
        return col;
    }

    private static String handlerQueryConditionLikeValue(QueryCondition queryCondition) {
        String val = (String) queryCondition.getValue();
        val = SqlUtils.escapeWildcardsForMySql(val);
        return val;
    }

    /**
     * 比较条件转换
     */
    private static void convertCondition(QueryDetail queryDetail, QueryWrapper queryWrapper, QueryCondition queryCondition) {
        Object queryValue = getQueryValue(queryCondition, null);

        String sqlColumn = queryDetail.getSqlColumnByProperty(queryCondition.getFieldName());
        if (queryCondition.getOperate() != null) {
            switch (queryCondition.getOperate()) {
                case notIn:
                    queryWrapper.notIn(sqlColumn, handlerQueryConditionCollectionValue(queryCondition));
                    break;
                case in:
                    queryWrapper.in(sqlColumn, handlerQueryConditionCollectionValue(queryCondition));
                    break;
                case findInSet:
                    queryWrapper.apply(StringUtils.toAppendStr("find_in_set(",sqlColumn, ",", "{0})"), queryValue);
                    break;
                case like:
                    queryWrapper.like(sqlColumn, handlerQueryConditionLikeValue(queryCondition));
                    break;
                case notLike:
                    queryWrapper.notLike(sqlColumn, handlerQueryConditionLikeValue(queryCondition));
                    break;
                case between:
                    queryWrapper.between(sqlColumn
                        , queryValue
                        , getQueryValue(queryCondition, queryCondition.getEndValue()));
                    break;
                case isNull:
                    queryWrapper.isNull(sqlColumn);
                    break;
                case isNotNull:
                    queryWrapper.isNotNull(sqlColumn);
                    break;
                default:
                    queryWrapper.apply(StringUtils.toAppendStr(
                        sqlColumn, " ",
                        queryCondition.getOperate().getOperator(), " {0} "
                    ), queryValue);
                    break;
            }
        } else {
            throw new IllegalArgumentException("查询操作符无效");
        }
    }

    /**
     * 转换条件规则
     */
    public static void convertQueryDetail(QueryDetail queryDetail) {
        QueryWrapper queryWrapper = queryDetail.getRealQueryWrapper();
        QueryWrapperUtil.convertQueryDetail(queryDetail, queryWrapper);
    }

    public static void convertQueryDetail(QueryDetail queryDetail, QueryWrapper queryWrapper) {
        //and 条件转换
        List<QueryCondition> andQueryConditions = queryDetail.getAndQueryConditions();
        if (andQueryConditions != null && !andQueryConditions.isEmpty()) {
            Iterator<QueryCondition> andIterator = andQueryConditions.iterator();
            while (andIterator.hasNext()) {
                QueryCondition queryCondition = andIterator.next();
                convertCondition(queryDetail, queryWrapper, queryCondition);
                andIterator.remove();
            }
            queryDetail.setAndQueryConditions(andQueryConditions);
        }

        //排序条件转换
        List<Sort.Order> orders = queryDetail.getOrders();
        if (orders != null && !orders.isEmpty()) {
            Iterator<Sort.Order> orderIterator = orders.iterator();
            while (orderIterator.hasNext()) {
                Sort.Order order = orderIterator.next();
                String fieldName = queryDetail.getSqlColumnByProperty(order.getProperty());
                switch (order.getDirection()) {
                    case ASC:
                        queryWrapper.orderByAsc(fieldName);
                        break;
                    case DESC:
                        queryWrapper.orderByDesc(fieldName);
                        break;
                    default:
                        break;
                }
                orderIterator.remove();
            }
        }
    }

    /**
     * 查询条件值，类型转换
     */
    @SneakyThrows
    public static Object getQueryValue(QueryCondition queryCondition, Object val) {
        String type = queryCondition.getAttrType();
        if (val == null) {
            val = queryCondition.getValue();
        }
        if (StringUtils.isNotEmpty(type) && val != null) {
            if (SystemConstants.TYPE_INTEGER.equalsIgnoreCase(type) || SystemConstants.TYPE_INT.equalsIgnoreCase(type)) {
                val = NumberUtils.toInt(String.valueOf(val));
            } else if (SystemConstants.TYPE_LONG.equalsIgnoreCase(type)) {
                val = NumberUtils.toLong(String.valueOf(val));
            } else if (SystemConstants.TYPE_SHORT.equalsIgnoreCase(type)) {
                val = NumberUtils.toShort(String.valueOf(val));
            } else if (SystemConstants.TYPE_FLOAT.equalsIgnoreCase(type)) {
                val = NumberUtils.toFloat(String.valueOf(val));
            } else if (SystemConstants.TYPE_DOUBLE.equalsIgnoreCase(type)) {
                val = NumberUtils.toDouble(String.valueOf(val));
            } else if (SystemConstants.TYPE_DATE.equalsIgnoreCase(type)) {
                val = StringUtils.isNotEmpty(queryCondition.getFormat()) ? DateUtils.parseDate(String.valueOf(val), queryCondition.getFormat()) : DateUtils.parseDate(String.valueOf(val));
            }
        }
        return val;
    }

}
