package com.c503.ca.cadv.common.mvc.util;

import com.baomidou.mybatisplus.core.toolkit.LambdaUtils;
import com.baomidou.mybatisplus.core.toolkit.support.ColumnCache;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.stereotype.Component;

import java.util.Locale;
import java.util.Map;

@Component
@ConfigurationProperties(prefix = "jdbc")
public class SqlUtils {

    /**
     * 根据数据表实体类，获取属性对应的sql字段
     *
     * @param clazz        实体类，//一般指有@TableName注解 并且 extends GeneralEntity
     * @param propertyName 属性名称，如：name,status
     * @return fieldsSql 属性对应的sql字段
     */
    public static <T> String getSqlColumnByProperty(Class<T> clazz,
                                                    String propertyName) {
        Map<String, ColumnCache> columnMap = LambdaUtils.getColumnMap(clazz);
        if (columnMap != null && !columnMap.isEmpty()) {
            ColumnCache columnCache = columnMap.get(propertyName.toUpperCase(Locale.ENGLISH));
            if (columnCache != null) {
                return columnCache.getColumn();
            }
        }
        return propertyName;
    }

    /**
     * 解决数据库保留字段未转换引号问题
     *
     * @Deprecated 各数据库保留字段不一，不再转换，请在TableFiled字段中用(反)引号包含内容
     */
    @Deprecated
    public static String sqlWordConvert(String convertStr) {
        return convertStr;
    }

    private static String escapeStringForMySql(String s) {
        return s.replaceAll("\\\\", "\\\\\\\\")
            .replaceAll("\b", "\\b")
            .replaceAll("\n", "\\n")
            .replaceAll("\r", "\\r")
            .replaceAll("\t", "\\t")
            .replaceAll("\\x1A", "\\Z")
            .replaceAll("\\x00", "\\0")
            .replaceAll("'", "\\\\'")
            .replaceAll("\"", "\\\\\"");
    }

    public static String escapeWildcardsForMySql(String s) {
        return escapeStringForMySql(s)
            .replaceAll("%", "\\\\%")
            .replaceAll("_", "\\\\_");
    }

}
