package com.c503.ca.cadv.common.rest.base;

import com.c503.ca.cadv.common.mvc.domain.BaseEntity;
import com.c503.ca.cadv.common.mvc.domain.vo.DataEntityVO;
import com.c503.ca.cadv.common.mvc.service.DataVOService;
import com.c503.ca.cadv.common.util.domain.CustomMessage;
import com.c503.ca.cadv.common.util.domain.ResultBuilder;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiOperation;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;

import java.util.Optional;

/**
 * 基础控制器支持类
 */
public class DataVOResource<Service extends DataVOService, V extends DataEntityVO>
    extends BaseResource {

    protected final Service service;

    public DataVOResource(Service service) {
        this.service = service;
    }

    /**
     * @param id id
     * @return 对象
     */
    @GetMapping("/{id:" + BaseEntity.ENTITY_ID_REGEX + "}")
    @ApiImplicitParam(name = "id", value = "实体ID", required = true, dataTypeClass = String.class, paramType = "path")
    @ApiOperation(value = "获取实体详情", notes = "")
    public ResponseEntity<CustomMessage<V>> get(@PathVariable String id) {
        log.debug("REST request to get Entity : {}", id);
        return ResultBuilder.wrapOrNotFound(Optional.ofNullable((V) service.findOneVO(id)));
    }

}
