package com.c503.ca.cadv.common.rest.handler;

import com.c503.ca.cadv.common.util.domain.ResultBuilder;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.web.servlet.error.ErrorAttributes;
import org.springframework.boot.web.servlet.error.ErrorController;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.context.request.WebRequest;

import javax.servlet.http.HttpServletResponse;
import java.util.Map;

@Slf4j
@Controller
public class HttpErrorHandler implements ErrorController {

    public static final String PATH = "/error";

    @Autowired
    private ErrorAttributes errorAttributes;

    @RequestMapping(value = PATH, produces = {MediaType.APPLICATION_JSON_VALUE})
    ResponseEntity error(WebRequest request, HttpServletResponse response) {
        return buildBody(request, true);
    }

    private ResponseEntity buildBody(WebRequest request, Boolean includeStackTrace) {
        Map<String, Object> errorAttributes = getErrorAttributes(request, includeStackTrace);
        Integer status = (Integer) errorAttributes.get("status");
        String path = (String) errorAttributes.get("path");
        String messageFound = (String) errorAttributes.get("message");
        String message = "", trace;
        if (!StringUtils.isEmpty(path)) {
            message = String.format("Requested path %s with result %s", path, messageFound);
        }
        if (includeStackTrace) {
            trace = (String) errorAttributes.get("trace");
            if (!StringUtils.isEmpty(trace)) {
                message += String.format(" and trace %s", trace);
            }
        }
        if (!path.contains("/ws/")) {
            log.debug("错误请求：{} - {}", status, message);
        }
        return ResultBuilder.buildFailed(HttpStatus.valueOf(status), message);
    }

    @Override
    public String getErrorPath() {
        return PATH;
    }

    protected Map<String, Object> getErrorAttributes(WebRequest request, boolean includeStackTrace) {
//        RequestAttributes requestAttributes = new ServletRequestAttributes(request);
        return errorAttributes.getErrorAttributes(request, includeStackTrace);
    }
}
