package com.c503.ca.cadv.enhance.arthas;

import com.c503.ca.cadv.common.util.IdGen;
import com.c503.ca.cadv.common.util.StringUtils;
import com.taobao.arthas.agent.attach.ArthasAgent;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnMissingBean;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.env.ConfigurableEnvironment;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

@ConditionalOnProperty(name = "spring.arthas.enabled")
@EnableConfigurationProperties({ArthasProperties.class})
@Configuration
public class ArthasConfiguration {
    private static final Logger logger = LoggerFactory.getLogger(ArthasConfiguration.class);
    public static String AGENT_ID;
    @Autowired
    ConfigurableEnvironment environment;

    public ArthasConfiguration() {
    }

    @ConfigurationProperties(prefix = "arthas")
    @ConditionalOnMissingBean
    @Bean
    public HashMap<String, String> arthasConfigMap() {
        return new HashMap<>();
    }

    @ConditionalOnMissingBean
    @Bean
    public ArthasAgent arthasAgent(@Autowired Map<String, String> arthasConfigMap, @Autowired ArthasProperties arthasProperties) throws Throwable {
        arthasConfigMap = StringUtils.removeDashKey(arthasConfigMap);

        String appName = this.environment.getProperty("spring.application.name");
        if (arthasConfigMap.get("appName") == null && appName != null) {
            arthasConfigMap.put("appName", appName);
        }

        Map<String, String> mapWithPrefix = new HashMap<>(arthasConfigMap.size());
        for (Entry<String, String> entry : arthasConfigMap.entrySet()) {
            mapWithPrefix.put("arthas." + entry.getKey(), entry.getValue());
        }

        AGENT_ID = IdGen.uuid();
        mapWithPrefix.put("arthas.agentId", AGENT_ID);

        ArthasAgent arthasAgent = new ArthasAgent(mapWithPrefix, arthasProperties.getHome(), arthasProperties.isSlientInit(), null);
        arthasAgent.init();
        logger.info("Arthas agent start success. Agent id is: {}", AGENT_ID);
        return arthasAgent;
    }
}
