package com.c503.ca.cadv.enhance.ftp.service;

import com.c503.ca.cadv.common.mvc.domain.IdAutoEntity;
import com.c503.ca.cadv.common.mvc.service.DataVOService;
import com.c503.ca.cadv.common.util.NetworkUtils;
import com.c503.ca.cadv.common.util.domain.CacheConstants;
import com.c503.ca.cadv.enhance.ftp.domain.FtpConfig;
import com.c503.ca.cadv.enhance.ftp.domain.vo.FtpConfigVO;
import com.c503.ca.cadv.enhance.ftp.repository.FtpConfigRepository;
import com.c503.ca.cadv.enhance.redis.util.RedisUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.Serializable;
import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

/**
 * <p>
 * FTP配置 服务类
 * </p>
 *
 * @author liaow
 * @since 2020-03-10
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class FtpConfigService extends DataVOService<FtpConfigRepository, FtpConfig, FtpConfigVO> {

    public void initOrFreshFtpConfigMap() {
        RedisUtils.del(CacheConstants.CACHE_FTP_ID_,CacheConstants.CACHE_FTP_NAME_);
        List<FtpConfigVO> ftpConfigVOList = findAllVO();
        for (FtpConfigVO ftpConfigVO : ftpConfigVOList) {
            RedisUtils.hSet(CacheConstants.CACHE_FTP_ID_,ftpConfigVO.getId(),ftpConfigVO);
            RedisUtils.hSet(CacheConstants.CACHE_FTP_NAME_,ftpConfigVO.getName(),ftpConfigVO);
        }
    }

    @Override
    public FtpConfig findOne(Serializable id) {
        FtpConfig result;
        FtpConfigVO cacheObj = (FtpConfigVO) RedisUtils.hGet(CacheConstants.CACHE_FTP_ID_,id.toString());
        if (cacheObj != null) {
            result = copyVOToBean(cacheObj);
        } else {
            result = super.getById(id);
            if (result != null) {
                RedisUtils.hSet(CacheConstants.CACHE_FTP_ID_,result.getId(),copyBeanToVO(result));
            }
        }

        return result;
    }

    @Override
    public boolean saveOrUpdate(FtpConfig entity) {
        boolean success = super.saveOrUpdate(entity);
        if (success) {
            initOrFreshFtpConfigMap();
        }
        return success;
    }

    @Override
    public boolean removeByIds(Collection<? extends Serializable> idList) {
        boolean success = super.removeByIds(idList);
        if (success) {
            initOrFreshFtpConfigMap();
        }
        return success;
    }

    /**
     * 获取可连接的FTP ID
     *
     * @return 可用的FTP ID列表
     */
    public List<String> getWorkableFtpIds() {
        List<FtpConfig> configs = findAll();
        return configs.stream().filter(config ->
                NetworkUtils.isHostReachable(config.getHost(), 3000)
        ).map(IdAutoEntity::getId).collect(Collectors.toList());
    }

    public List<FtpConfigVO> getList() {
        return findAllVO();
    }

    public FtpConfigVO findByHostName(String hostName) {
        if (!RedisUtils.hasKey(CacheConstants.CACHE_FTP_NAME_)) {
            initOrFreshFtpConfigMap();
        }
        return (FtpConfigVO) RedisUtils.hGet(CacheConstants.CACHE_FTP_NAME_,hostName);
    }

}
