/**
 * Copyright &copy; 2019-2020  All rights reserved.
 */
package com.c503.ca.cadv.enhance.ftp.web;

import com.c503.ca.cadv.common.rest.base.DataVOResource;
import com.c503.ca.cadv.common.util.JacksonUtils;
import com.c503.ca.cadv.common.util.StringUtils;
import com.c503.ca.cadv.common.util.domain.CustomMessage;
import com.c503.ca.cadv.common.util.domain.PageModel;
import com.c503.ca.cadv.common.util.domain.ResultBuilder;
import com.c503.ca.cadv.enhance.ftp.domain.vo.FtpConfigVO;
import com.c503.ca.cadv.enhance.ftp.service.FtpConfigService;
import com.fasterxml.jackson.databind.JsonNode;
import com.google.common.collect.Lists;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

/**
 * <p>
 * FTP配置 控制器
 * </p>
 *
 * @author liaow
 * @since 2020-03-10
 */
@RestController
@RequestMapping("/ftp/ftp-config")
@Api(tags = "ftp_FtpConfig")
public class FtpConfigResource extends DataVOResource<FtpConfigService, FtpConfigVO> {
    public FtpConfigResource(FtpConfigService service) {
        super(service);
    }

    @GetMapping(value = "/")
    @ApiOperation(value = "获取FTP配置列表", notes = "根据分页参数，获取分页列表")
    public ResponseEntity<CustomMessage<PageModel<FtpConfigVO>>> getPage(PageModel<FtpConfigVO> pm) {
        return ResultBuilder.buildObject(service.findVOPage(pm));
    }

    /**
     * 保存 FTP配置
     */
    @PostMapping(value = "/", produces = MediaType.APPLICATION_JSON_VALUE)
    @ApiOperation(value = "保存FTP配置", notes = "")
    public ResponseEntity<CustomMessage<FtpConfigVO>> save(@Valid @RequestBody FtpConfigVO vo) {
        FtpConfigVO ftpConfigVO = service.findByHostName(vo.getName());
        if (StringUtils.isNotEmpty(vo.getId())) {
            if (ftpConfigVO != null && !vo.getId().equals(ftpConfigVO.getId())){
                return ResultBuilder.buildFailed("更新失败， FTP名称已存在");
            }
        }else {
            if (ftpConfigVO != null) {
                return ResultBuilder.buildFailed("新增失败，FTP名称已存在");
            }
        }
        service.saveOrUpdate(vo);
        return ResultBuilder.buildOk(vo, "保存成功");
    }

    /**
     * 批量删除
     */
    @PostMapping(value = "/del")
    public ResponseEntity<CustomMessage<String>> delete(@RequestBody String[] ids) {
        boolean success = service.physicalRemoveByIds(Lists.newArrayList(ids));
        service.initOrFreshFtpConfigMap();
        return success ? ResultBuilder.buildOk("删除成功") : ResultBuilder.buildOk("删除失败");
    }

    @GetMapping(value = "/list")
    @ApiOperation(value = "获取FTP列表", notes = "获取FTP列表")
    public ResponseEntity<CustomMessage<JsonNode>> getList() {
        return ResultBuilder.buildObject(JacksonUtils.toJsonNode(service.getList()));
    }
}
