/**
 * Copyright &copy; 2018-2019 c503-boss All rights reserved.
 */
package com.c503.ca.cadv.enhance.quartz.domain;

import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableName;
import com.c503.ca.cadv.common.mvc.domain.IdAutoEntity;
import com.c503.ca.cadv.common.util.StringUtils;
import com.c503.ca.cadv.common.util.annotation.DictType;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import org.hibernate.validator.constraints.Length;

import java.time.ZoneId;
import java.util.TimeZone;

/**
 * 定时任务Entity 定时任务
 *
 * @author admin
 */
@TableName(value = "quartz_task_schedule_job_t")
@Data
@EqualsAndHashCode(callSuper = true)
@NoArgsConstructor
@AllArgsConstructor
public class TaskScheduleJob extends IdAutoEntity<String> {

    private static final long serialVersionUID = 1L;

    //columns START
    /**
     * name 名称
     */
    @Length(max = 255)
    @TableField("name_")
    private String name;

    /**
     * group 分组
     */
    @Length(max = 255)
    @TableField("group_")
    private String group;

    /**
     * jobStatus 任务状态
     */
    @Length(max = 255)
    @TableField("job_status")
    private String jobStatus;

    /**
     * cronExpression cron表达式
     */
    @Length(max = 255)
    @TableField("cron_expression")
    private String cronExpression;

    /**
     * 设置任务的时区
     **/
    @TableField(exist = false)
    private TimeZone timeZone;

    @TableField("time_zone_id")
    private String timeZoneId;

    /**
     * isConcurrent 是否当前任务
     */
    @DictType(name = "sys_yes_no")
    @TableField("is_concurrent")
    private Integer isConcurrent;

    /**
     * springId springId
     */
    @Length(max = 255)
    @TableField("spring_id")
    private String springId;

    /**
     * sourceId 业务编号
     */
    @Length(max = 255)
    @TableField("source_id")
    private String sourceId;

    /**
     * beanClass 调用类名
     */
    @Length(max = 255)
    @TableField("bean_class")
    private String beanClass;

    /**
     * methodname 调用方法名
     */
    @Length(max = 255)
    @TableField("method_name")
    private String methodName;

    /**
     * methodParams 方法参数
     */
    @Length(max = 255)
    @TableField("method_params")
    private String methodParams;

    /**
     * description 描述
     */
    @Length(max = 65535)
    @TableField("description_")
    private String description;

    //columns END
    public TimeZone getTimeZone() {
        if (this.timeZone != null && StringUtils.isNotEmpty(this.timeZoneId)) {
            timeZone = TimeZone.getTimeZone(ZoneId.of(this.timeZoneId));
        }
        return timeZone;
    }

    public void setTimeZone(TimeZone timeZone) {
        this.timeZone = timeZone;
        this.timeZoneId = timeZone == null ? null : timeZone.getID();
    }

    protected String getTimeZoneId() {
        if (StringUtils.isEmpty(this.timeZoneId)) {
            timeZoneId = this.timeZone == null ? null : this.timeZone.getID();
        }
        return timeZoneId;
    }

    protected void setTimeZoneId(String timeZoneId) {
        this.timeZoneId = timeZoneId;
        this.timeZone = TimeZone.getTimeZone(ZoneId.of(this.timeZoneId));
    }

}
