/**
 * Copyright &copy; 2018-2019 c503-boss All rights reserved.
 */
package com.c503.ca.cadv.enhance.quartz.domain.vo;

import com.c503.ca.cadv.common.mvc.domain.vo.DataEntityVO;
import com.c503.ca.cadv.common.util.StringUtils;
import io.swagger.annotations.ApiModelProperty;
import lombok.*;
import org.hibernate.validator.constraints.Length;

import java.time.ZoneId;
import java.util.TimeZone;

/**
 * 定时任务EntityVO 定时任务
 *
 * @author admin
 */
@Data
@Builder
@EqualsAndHashCode(callSuper = true)
@NoArgsConstructor
@AllArgsConstructor
public class TaskScheduleJobVO extends DataEntityVO<String> {

    private static final long serialVersionUID = 1L;

    //columns START
    /**
     * name 名称
     */
    @Length(max = 255)
    @ApiModelProperty(value = "名称")
    private String name;

    /**
     * group 分组
     */
    @Length(max = 255)
    @ApiModelProperty(value = "分组")
    private String group;

    /**
     * SystemConfig.STR_NO  SystemConfig.STR_YES 初始化创建一个任务用这个构造
     * 初始化的时候 一定是SystemConfig.STR_YES 否则该定时任务不会被添加
     * **/
    /**
     * jobStatus 任务状态
     */
    @Length(max = 255)
    @ApiModelProperty(value = "任务状态")
    private String jobStatus;

    /**
     * cronExpression cron表达式
     */
    @Length(max = 255)
    @ApiModelProperty(value = "cron表达式")
    private String cronExpression;

    /**
     * timeZone 时区配置，传了就为传的时区，不传默认为TimeZone.getDefault()
     */
    @ApiModelProperty(value = "时区配置，传了就为传的时区，不传默认为TimeZone.getDefault()")
    private TimeZone timeZone;

    @ApiModelProperty(value = "时区配置的id")
    private String timeZoneId;
    /**
     * isConcurrent 是否当前任务
     */
    @ApiModelProperty(value = "是否当前任务", example = "1")
    private Integer isConcurrent;

    /**
     * springId springId
     */
    @Length(max = 255)
    @ApiModelProperty(value = "springId")
    private String springId;

    /**
     * sourceId 业务编号
     */
    @Length(max = 255)
    @ApiModelProperty(value = "业务编号")
    private String sourceId;

    /**
     * beanClass 调用类名
     */
    @Length(max = 255)
    @ApiModelProperty(value = "调用类名")
    private String beanClass;

    /**
     * methodname 调用方法名
     */
    @Length(max = 255)
    @ApiModelProperty(value = "调用方法名")
    private String methodName;

    /**
     * methodParams 方法参数
     */
    @Length(max = 255)
    @ApiModelProperty(value = "方法参数")
    private String methodParams;

    /**
     * description 描述
     */
    @Length(max = 65535)
    @ApiModelProperty(value = "描述")
    private String description;

    public TaskScheduleJobVO(String id) {
        this.setId(id);
    }

    //columns END
    public TimeZone getTimeZone() {
        if (this.timeZone != null && StringUtils.isNotEmpty(this.timeZoneId)) {
            timeZone = TimeZone.getTimeZone(ZoneId.of(this.timeZoneId));
        }
        return timeZone;
    }

    public void setTimeZone(TimeZone timeZone) {
        this.timeZone = timeZone;
        this.timeZoneId = timeZone == null ? null : timeZone.getID();
    }

    protected String getTimeZoneId() {
        if (StringUtils.isEmpty(this.timeZoneId)) {
            timeZoneId = this.timeZone == null ? null : this.timeZone.getID();
        }
        return timeZoneId;
    }

    protected void setTimeZoneId(String timeZoneId) {
        this.timeZoneId = timeZoneId;
        this.timeZone = TimeZone.getTimeZone(ZoneId.of(this.timeZoneId));
    }


}
