package com.c503.ca.cadv.enhance.quartz.util;

import com.c503.ca.cadv.common.util.JacksonUtils;
import com.c503.ca.cadv.common.util.StringUtils;
import com.c503.ca.cadv.common.util.base.Reflections;
import com.c503.ca.cadv.common.util.spring.SpringContextHolder;
import com.c503.ca.cadv.enhance.quartz.domain.TaskScheduleJob;
import com.google.common.collect.Lists;
import org.apache.commons.lang3.StringEscapeUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.List;

public class TaskScheduleUtils {
    protected static Logger logger = LoggerFactory.getLogger(TaskScheduleUtils.class);

    /**
     * 通过反射调用scheduleJob中定义的方法
     *
     * @param scheduleJob
     */
    public static void invokMethod(TaskScheduleJob scheduleJob) {
        Object object = null;
        Class clazz = null;
        if (StringUtils.isNotBlank(scheduleJob.getSpringId())) {
            object = SpringContextHolder.getBean(scheduleJob.getSpringId());
            if (object != null && object.getClass().getName().contains("$Proxy")) {
                try {
                    object = Reflections.getTargetClass(object).newInstance();
                } catch (Exception e) {
                    logger.warn("msg : {}", e.getMessage());
                }
            }
        } else if (StringUtils.isNotBlank(scheduleJob.getBeanClass())) {
            try {
                clazz = Class.forName(scheduleJob.getBeanClass());
                object = SpringContextHolder.getBean(clazz);
                if (object != null) {
                    object = clazz.newInstance();
                }
            } catch (Exception e) {
                logger.warn("msg : {}", e.getMessage());
            }

        }
        if (object == null) {
            logger.error("任务名称 = [{}]---------------未启动成功，请检查是否配置正确！！！", scheduleJob.getName());
            return;
        }
        try {
            List<Object> paramsList = Lists.newArrayList();
            String params = StringEscapeUtils.unescapeHtml4(scheduleJob.getMethodParams());
            if (StringUtils.isNotEmpty(params)) {
                try {
                    if (params.startsWith("[")) {
                        JacksonUtils.deserializeList(params, TaskTargetParam.class).forEach(item -> {
                            paramsList.add(item.getRelValue());
                        });
                    } else {
                        TaskTargetParam targetParam = JacksonUtils.fromJsonString(params, TaskTargetParam.class);
                        paramsList.add(targetParam.getRelValue());
                    }
                } catch (Exception e) {
                    logger.warn("msg {}", e);
                }
            }
            logger.info("定时任务 = [{}] [{}]--------启动调度", scheduleJob.getGroup(), scheduleJob.getName());
            Reflections.invokeMethodByName(object, scheduleJob.getMethodName(), paramsList.toArray());
        } catch (SecurityException e) {
            e.printStackTrace();
            logger.warn("定时任务 = [{}] [{}]--------调度失败，异常:{}", scheduleJob.getGroup(), scheduleJob.getName(), e.getMessage());
        }
        logger.info("定时任务 = [{}] [{}]--------调度完成", scheduleJob.getGroup(), scheduleJob.getName());
    }
}
