/**
 * Copyright &copy; 2018-2019 c503-boss All rights reserved.
 */
package com.c503.ca.cadv.enhance.quartz.web;

import com.c503.ca.cadv.common.mvc.domain.BaseEntity;
import com.c503.ca.cadv.common.rest.base.DataVOResource;
import com.c503.ca.cadv.common.util.JacksonUtils;
import com.c503.ca.cadv.common.util.StringUtils;
import com.c503.ca.cadv.common.util.domain.CustomMessage;
import com.c503.ca.cadv.common.util.domain.PageModel;
import com.c503.ca.cadv.common.util.domain.ResultBuilder;
import com.c503.ca.cadv.enhance.quartz.domain.vo.TaskScheduleJobVO;
import com.c503.ca.cadv.enhance.quartz.service.TaskScheduleJobService;
import com.google.common.collect.Lists;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

/**
 * 定时任务Controller 定时任务
 *
 * @author admin
 */
@RestController
@RequestMapping(value = "/taskScheduleJob")
@Api(tags = "quartz_TaskScheduleJob")
public class TaskScheduleJobResource extends DataVOResource<TaskScheduleJobService, TaskScheduleJobVO> {

    public TaskScheduleJobResource(TaskScheduleJobService service) {
        super(service);
    }

    /**
     * GET / : get all taskScheduleJob.
     *
     * @param pm the pagination information
     * @return the ResponseEntity with status 200 (OK) and with body all taskScheduleJob
     */
    @GetMapping(value = "/")
    @ApiOperation(value = "获取定时任务列表", notes = "根据分页参数获取分页列表")
    public ResponseEntity<CustomMessage<String>> getPage(PageModel pm) {
        service.findVOPage(pm);
        String json = JacksonUtils.toJsonString(pm);
        return ResultBuilder.buildObject(json);
    }

    /**
     * POST / : Save a taskScheduleJobVO.
     *
     * @param taskScheduleJobVO the HTTP taskScheduleJob
     */
    @PostMapping(value = "/", produces = MediaType.APPLICATION_JSON_VALUE)
    @ApiOperation(value = "保存定时任务", notes = "")
    public ResponseEntity<CustomMessage<String>> save(@Valid @RequestBody TaskScheduleJobVO taskScheduleJobVO) {
        service.saveTaskScheduleJob(taskScheduleJobVO);
        return ResultBuilder.buildOk("保存定时任务成功");
    }

    /**
     * DELETE //:id : delete the "id" TaskScheduleJob.
     *
     * @param ids the id of the taskScheduleJob to delete
     * @return the ResponseEntity with status 200 (OK)
     */
    @DeleteMapping(value = "/{ids:" + BaseEntity.ENTITY_ID_REGEX
        + "}")
    @ApiOperation(value = "删除定时任务", notes = "")
    public ResponseEntity<CustomMessage<String>> delete(@PathVariable String ids) {
        service.removeByIds(Lists.newArrayList(ids.split(StringUtils.SPLIT_DEFAULT)));
        return ResultBuilder.buildOk("删除定时任务成功");
    }

    @DeleteMapping(value = "/delete/{id:" + BaseEntity.ENTITY_ID_REGEX
        + "}")
    @ApiOperation(value = "删除定时任务用id", notes = "")
    public ResponseEntity<CustomMessage<String>> deleteJob(@PathVariable String id) {
        service.deleteJobById(id);
        return ResultBuilder.buildOk("删除定时任务成功");
    }
}
