package com.c503.ca.cadv.enhance.websocket.config;

import com.c503.ca.cadv.common.config.ExecutorConfig;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.redis.connection.RedisConnectionFactory;
import org.springframework.data.redis.listener.PatternTopic;
import org.springframework.data.redis.listener.RedisMessageListenerContainer;
import org.springframework.data.redis.listener.adapter.MessageListenerAdapter;
import org.springframework.data.redis.serializer.Jackson2JsonRedisSerializer;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;


@Configuration
public class RedisMessageListenerConfig {
	
	@Autowired
	private RedisReceiver redisReceiver;
	
	@Bean
	public RedisMessageListenerContainer getRedisMessageListenerContainer(RedisConnectionFactory redisConnectionFactory) {
		RedisMessageListenerContainer redisMessageListenerContainer = new RedisMessageListenerContainer();
		redisMessageListenerContainer.setConnectionFactory(redisConnectionFactory);
		ThreadPoolTaskExecutor ex = new ExecutorConfig().nodeAsyncServiceExcutor("RedisMessageListenerConfig", 20);
		redisMessageListenerContainer.setTaskExecutor(ex);
		redisMessageListenerContainer.addMessageListener(messageListenerQueueAdapter(), new PatternTopic("/queue"));
		redisMessageListenerContainer.addMessageListener(messageListenerTopicAdapter(), new PatternTopic("/topic"));
		redisMessageListenerContainer.addMessageListener(messageListenerUserAdapter(), new PatternTopic("/user"));
		return redisMessageListenerContainer;
	}
	
	@Bean
	public MessageListenerAdapter messageListenerQueueAdapter() {
		MessageListenerAdapter messageListenerAdapter = new MessageListenerAdapter(redisReceiver, "sendQueueMsg");
		Jackson2JsonRedisSerializer<Object> jackson2JsonRedisSerializer = new Jackson2JsonRedisSerializer<>(
				Object.class);
		messageListenerAdapter.setSerializer(jackson2JsonRedisSerializer);
		return messageListenerAdapter;
	}
	
	@Bean
	public MessageListenerAdapter messageListenerTopicAdapter() {
		MessageListenerAdapter messageListenerAdapter = new MessageListenerAdapter(redisReceiver, "sendTopicMsg");
		Jackson2JsonRedisSerializer<Object> jackson2JsonRedisSerializer = new Jackson2JsonRedisSerializer<>(
				Object.class);
		messageListenerAdapter.setSerializer(jackson2JsonRedisSerializer);
		return messageListenerAdapter;
	}
	
	@Bean
	public MessageListenerAdapter messageListenerUserAdapter() {
		MessageListenerAdapter messageListenerAdapter = new MessageListenerAdapter(redisReceiver, "sendUserMsg");
		Jackson2JsonRedisSerializer<Object> jackson2JsonRedisSerializer = new Jackson2JsonRedisSerializer<>(
				Object.class);
		messageListenerAdapter.setSerializer(jackson2JsonRedisSerializer);
		return messageListenerAdapter;
	}
}
