package com.c503.ca.cadv.enhance.websocket.domain;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonValue;
import com.google.common.collect.Maps;
import lombok.Getter;

import java.util.Map;

/**
 * WebSocket 通用消息类型枚举
 *
 * @author liaow
 */
@Getter
public enum CommonMessageType implements IMessageType {

    /**
     * 连接消息推送
     */
    BROADCAST_MSG("BROADCAST_MSG", "系统广播消息"),

    /**
     * 文字通知 推送
     */
    INFO_MSG("INFO_MSG", "通用提示消息"),
    WARN_MSG("WARN_MSG", "通用警告消息"),
    ERROR_MSG("ERROR_MSG", "通用错误消息"),
    MATTERS_SIZE("MATTERS_SIZE", "待办事件数量"),

    IMPORT_CMD_PROCESS("IMPORT_CMD_PROCESS", "指令进度"),
    IMPORT_CMD_FINISHED("IMPORT_CMD_FINISHED", "指令文件导入完成"),
    IMPORT_FFCMD_PROCESS("IMPORT_FFCMD_PROCESS", "指令进度"),
    IMPORT_FFCMD_FINISHED("IMPORT_FFCMD_FINISHED", "指令文件导入完成"),

    SYNCHRONIZE_PROCESS("SYNCHRONIZE_PROCESS", "组织、岗位、用户同步进度"),
    SYNCHRONIZE_FINISHED("SYNCHRONIZE_FINISHED", "组织、岗位、用户同步进度完成"),

    ACCOUNT_REPLACE("ACCOUNT_REPLACE", "账号顶替消息"),

    TASK_DEL_MSG("TASK_DEL_MSG","流程删除消息通知"),
    /**
     * 结点控制消息推送
     */
    NODE_BAN("NODE_BAN", "结点封禁"),
    NODE_OUTLINE("NODE_OUTLINE", "结点下线"),
    NODE_LOCK_SCREEN("NODE_LOCK_SCREEN", "结点锁屏"),
    NODE_UNLOCK_SCREEN("NODE_UNLOCK_SCREEN", "节点锁屏解除"),


    /**
     * 企鹅日志服务
     */
    QE_IMSRCLOG("QE_IMSRCLOG", "遥控记录日志电文"),
    QE_CLDGRSLT("QE_CLDGRSLT", "生数结果"),
    /**
     * 遥控记录日志电文
     ***/
    QE_IMSRCFIL2DIS("QE_IMSRCFIL2DIS", "文件送监视通知 RCDV、RCDR"),
    /**
     * 文件送监视通知 RCDV、RCDR
     **/

    ZB_MSG_FRESH("ZB_MSG_FRESH", "主备机信息更新"),

    RPLNK("RPLNK", "RPLNK"),
    RCR("RCR", "RCR"),

    GENERATE_BEGIN("GENERATE_BEGIN", "注入文件数据生成开始"),
    GENERATE_PROGRESS("GENERATE_PROGRESS", "注入文件数据生成进度"),
    GENERATE_FAILED("GENERATE_FAILED", "注入文件数据生成失败"),
    GENERATE_FINISHED("GENERATE_FINISHED", "注入文件数据生成完成"),

    SEND_FM_PROGRESS("SEND_FM_PROGRESS", "发送电文进度推送"),
    SEND_FM_FINISHED("SEND_FM_FINISHED", "发送电文完成"),

    ;


    private static final Map<String, CommonMessageType> enumMap = Maps.newHashMap();
    private final String desc;

    @JsonValue  // jackson 序列化时使用
    private final String value;


    CommonMessageType(String value, String desc) {
        this.value = value;
        this.desc = desc;
    }

    @JsonCreator(mode = JsonCreator.Mode.DELEGATING) //jackson 映射枚举对象
    public static CommonMessageType forValue(String value) {
        CommonMessageType state = enumMap.get(value);
        if (state == null) {
            for (CommonMessageType enumObj : values()) {
                enumMap.put(enumObj.getValue(), enumObj);
            }
            state = enumMap.get(value);
        }
        return state;
    }

    public String getDesc() {
        return desc;
    }

    @Override
    public String toString() {
        return this.getValue();
    }

}
