package com.c503.ca.cadv.enhance.websocket.domain;

import com.c503.ca.cadv.common.constant.SecurityConstants;
import com.c503.ca.cadv.common.util.SecurityUtils;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;

/**
 * Websocket 消息报文格式
 *
 * @author liaow
 */
@Data
@AllArgsConstructor
@NoArgsConstructor
public class WebSocketMessage {

    /**
     * 消息接收者
     */
    private String userId;

    /**
     * topicType
     */
    private String topicType;
    /**
     * 消息类型
     */
    private String type;

    /**
     * 消息体/正文
     */
    private Object body;

    /**
     * 消息创建者
     */
    private String creator = defaultCreator();

    private String defaultCreator() {
        String currentUser = SecurityUtils.getUsername();
        return currentUser == null ? SecurityConstants.ADMIN_LOGIN_ID : currentUser;
    }

    public WebSocketMessage(String type, IMessageBody body) {
        this.type = type;
        this.body = body;
    }

    /**
     * 利用条件创造WebSocketMessage对象
     */
    public static WebSocketMessage of(String type, String text) {
        return new WebSocketMessage(type, new TextMessageBody(text));
    }

    /**
     * 利用条件创造WebSocketMessage对象
     */
    public static <T> WebSocketMessage of(String type, T data) {
        return new WebSocketMessage(type, new DataMessageBody<T>(data));
    }
}
