package com.c503.ca.cadv.enhance.websocket.web;

import com.c503.ca.cadv.common.util.domain.CustomMessage;
import com.c503.ca.cadv.common.util.domain.ResultBuilder;
import com.c503.ca.cadv.enhance.websocket.domain.CommonMessageType;
import com.c503.ca.cadv.enhance.websocket.domain.WebSocketMessage;
import com.c503.ca.cadv.enhance.websocket.util.WebSocketUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.NonNull;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * websocket 测试接口
 *
 * @author liaow
 */
@RestController
@RequestMapping("/ws")
@Api(tags = "websocket_controller")
public class WebsocketController {

    @GetMapping(value = "/test/broadcast")
    @ApiOperation(value = "测试websocket-广播消息推送")
    public ResponseEntity<CustomMessage<String>> sendBroadcast(@NonNull String message) {
        WebSocketUtils.sendBroadcast(message);
        return ResultBuilder.buildOk("消息推送完成");
    }

    @GetMapping(value = "/test/topic")
    @ApiOperation(value = "测试websocket-主题消息推送", notes = "messageType(枚举value): webSocketMessageType")
    public ResponseEntity<CustomMessage<String>> sendTopic(@NonNull String topic, @NonNull CommonMessageType messageType, @NonNull String message) {
        WebSocketUtils.sendTopic(topic, WebSocketMessage.of(messageType.getValue(), message));
        return ResultBuilder.buildOk("消息推送完成");
    }

    @GetMapping(value = "/test/user")
    @ApiOperation(value = "测试websocket-用户消息推送", notes = "messageType(枚举value): webSocketMessageType")
    public ResponseEntity<CustomMessage<String>> sendToUser(@NonNull String user, @NonNull CommonMessageType messageType, @NonNull String message) {
        WebSocketUtils.sendToUser(user, WebSocketMessage.of(messageType.getValue(), message));
        return ResultBuilder.buildOk("消息推送完成");
    }
}
