package com.c503.ca.cadv.module.check.domain;

import com.c503.ca.cadv.common.util.CollectionsUtils;
import com.google.common.collect.Lists;
import com.thoughtworks.xstream.annotations.XStreamAlias;
import com.thoughtworks.xstream.annotations.XStreamImplicit;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;

import java.util.List;

/**
 * 反算文件xml对象
 */
@Data
@XStreamAlias("cmlRoot")
@Slf4j
public class RsltFileXDO {

    public static final List<String> sCmdCode = Lists.newArrayList("GNCF1", "GNCF2", "GNCF3");

    /**
     * 文件名
     */
    @XStreamAlias("szFileName")
    private String szFileName;

    /**
     * 文件名
     */
    @XStreamAlias("stLoadData")
    @XStreamImplicit
    private List<StLoadDataXDO> stLoadDataList;

    /**
     * 格式化对象（主要针对含s型的反算文件）
     */
    public static RsltFileXDO format(RsltFileXDO rsltFileXDO) {
        RsltFileXDO formatXDO = new RsltFileXDO();
        try {
            if (rsltFileXDO == null) return null;
            formatXDO.setSzFileName(rsltFileXDO.getSzFileName());
            if (CollectionsUtils.isNotEmpty(rsltFileXDO.getStLoadDataList())) {
                List<StLoadDataXDO> formatDataXDOList = Lists.newArrayList();
                formatXDO.setStLoadDataList(formatDataXDOList);
                //遍历注入数据
                for (StLoadDataXDO stLoadDataXDO : rsltFileXDO.getStLoadDataList()) {
                    StLoadDataXDO formatDataXDO = new StLoadDataXDO();
                    formatDataXDO.setUcLevNoList(stLoadDataXDO.getUcLevNoList());
                    formatDataXDO.setUiCmdNumList(stLoadDataXDO.getUiCmdNumList());
                    formatDataXDOList.add(formatDataXDO);
                    formatDataXDO.setSzData(stLoadDataXDO.getSzData());

                    if (CollectionsUtils.isNotEmpty(stLoadDataXDO.getStCmdInfoList())) {
                        //全部第一层指令集合
                        List<StCmdInfoXDO> formatCmdInfoList = Lists.newArrayList();
                        //按帧分组集合
                        List<List<StCmdInfoXDO>> cmdInfoGroupList = Lists.newArrayList();
                        formatDataXDO.setStCmdInfoList(formatCmdInfoList);
                        formatDataXDO.setStCmdInfoGroupList(cmdInfoGroupList);

                        //无指令数集合时，全部归于一帧
                        if (CollectionsUtils.isEmpty(stLoadDataXDO.getUiCmdNumList())) {
                            formatCmdInfoList.addAll(stLoadDataXDO.getStCmdInfoList());
                        } else {
                            //反算文件内全部指令
                            List<StCmdInfoXDO> stCmdInfoXDOList = stLoadDataXDO.getStCmdInfoList();
                            //已分组的指令下标
                            int cmdIndex = 0;

                            //待分组指令数
                            int cmdNum = Integer.parseInt(stLoadDataXDO.getUiCmdNumList().get(0));
                            //已分组的指令下标偏移
                            cmdIndex += cmdNum;
                            //同帧组集合
                            List<StCmdInfoXDO> cmdInfoXDOGroup = Lists.newArrayList(stCmdInfoXDOList.subList(0, cmdIndex));
                            cmdInfoGroupList.add(cmdInfoXDOGroup);
                            formatCmdInfoList.addAll(cmdInfoXDOGroup);

                            //levNo[0]在以上代码，所以从1开始
                          for (int i = 1; i < stLoadDataXDO.getUcLevNoList().size(); i++) {
                                String levNo = stLoadDataXDO.getUcLevNoList().get(i);
                              //如果ucLevNo为1，建立新的分组
                                if ("1".equals(levNo)){
                                    //新分组的指令数目
                                    cmdNum = Integer.parseInt(stLoadDataXDO.getUiCmdNumList().get(i));
                                    //建立新分组
                                    cmdInfoXDOGroup = Lists.newArrayList(stCmdInfoXDOList.subList(cmdIndex, cmdIndex + cmdNum));
                                    //已分组的指令下标偏移
                                    cmdIndex += cmdNum;
                                    cmdInfoGroupList.add(cmdInfoXDOGroup);
                                    formatCmdInfoList.addAll(cmdInfoXDOGroup);
                                    //如果ucLevNo为2，指令集合并入上一分组最后一个指令的最后一个参数内
                                } else if ("2".equals(levNo)) {
                                    //该S号指令批次的指令数目
                                    cmdNum = Integer.parseInt(stLoadDataXDO.getUiCmdNumList().get(i));
                                    //S号指令集合
                                    List<StCmdInfoXDO> innerStCmdInfoXDOList = Lists.newArrayList(stCmdInfoXDOList.subList(cmdIndex, cmdIndex + cmdNum));
                                    //已分组的指令下标偏移
                                    cmdIndex += cmdNum;
                                    //上一分组最后的一个指令的参数集合
                                    List<StCmdPara> stCmdParaListOfLastGroup = cmdInfoXDOGroup.get(cmdInfoXDOGroup.size() - 1).getStCmdParaList();
                                    //S号指令集合放入最后一参数中
                                    stCmdParaListOfLastGroup.get(stCmdParaListOfLastGroup.size() - 1).setStCmdInfoXDOList(innerStCmdInfoXDOList);
                                }else{
                                    throw new RuntimeException("无法识别ucLevNo:" + levNo);
                                }

                            }
                        }
                    }
                }
            }
        } catch (Exception e) {
            log.error("格式化对象（主要针对含s型的反算文件）失败:{}", e.toString());
            e.printStackTrace();
            return null;
        }
        return formatXDO;
    }
}
