package com.c503.ca.cadv.module.check.util;

import com.c503.ca.cadv.common.config.SystemProperties;
import com.c503.ca.cadv.common.util.BeanConvertUtils;
import com.c503.ca.cadv.common.util.CollectionsUtils;

import com.c503.ca.cadv.common.util.IdGen;
import com.c503.ca.cadv.common.util.JacksonUtils;
import com.c503.ca.cadv.common.util.exception.RuntimeErrorException;
import com.c503.ca.cadv.common.util.xml.XmlUtils;
import com.c503.ca.cadv.enhance.ftp.FtpUtils;
import com.c503.ca.cadv.module.check.domain.*;
import com.c503.ca.cadv.module.check.domain.orin.racpa.RacpaXDO;
import com.c503.ca.cadv.module.check.domain.orin.racpa.StruRacpaData;
import com.c503.ca.cadv.module.check.domain.orin.rpgnc.RpgncXDO;
import com.c503.ca.cadv.module.check.domain.orin.rpgnc.StrRpgncBody;
import com.c503.ca.cadv.module.check.domain.orin.rpppc.RpppcXDO;
import com.c503.ca.cadv.module.check.domain.orin.rpppc.StrRpppcBody;
import com.c503.ca.cadv.module.check.domain.orin.rprlytrk.RprlytrkXDO;
import com.c503.ca.cadv.module.check.domain.orin.rprlytrk.StrRlystartrkBody;
import com.c503.ca.cadv.module.check.domain.orin.rsbeod.RsbeodXDO;
import com.c503.ca.cadv.module.check.domain.orin.rsbeod.StruRsbeoData;
import com.c503.ca.cadv.module.check.domain.orin.rsbeod.StruRsbeod;
import com.c503.ca.cadv.module.cmp.domain.vo.FillConfigVO;
import com.c503.ca.cadv.module.cmp.util.CmpHelper;
import com.c503.ca.cadv.module.cmp.util.RevCalCompConfigUtils;
import com.c503.ca.cadv.module.ge.domain.enums.ParamSignType;
import com.c503.ca.cadv.module.ge.domain.vo.CmdVO;
import com.c503.ca.cadv.module.ge.util.CmdHelper;
import com.c503.ca.cadv.module.ge.util.CmdParamUtil;
import com.c503.ca.cadv.module.info.domain.Catalog;
import com.c503.ca.cadv.module.info.domain.Target;
import com.c503.ca.cadv.module.info.domain.Task;
import com.c503.ca.cadv.module.info.domain.enums.CatalogType;
import com.c503.ca.cadv.module.info.domain.enums.TargetType;
import com.c503.ca.cadv.module.info.domain.vo.TargetVO;
import com.c503.ca.cadv.module.info.domain.vo.TargetZbVO;
import com.c503.ca.cadv.module.info.domain.vo.TaskVO;
import com.c503.ca.cadv.module.info.util.InfoHelper;
import com.c503.ca.cadv.module.plan.domain.CmdInst;
import com.c503.ca.cadv.module.plan.domain.CmdParamInst;
import com.c503.ca.cadv.module.plan.domain.vo.CmdInstVO;
import com.c503.ca.cadv.module.plan.domain.vo.CmdParamInstVO;
import com.c503.ca.cadv.module.plan.domain.vo.ConfigFileVO;
import com.c503.ca.cadv.module.plan.domain.vo.FileDataVO;
import com.c503.ca.cadv.module.plan.util.CmdConvertUtils;
import com.c503.ca.cadv.module.plan.util.CompUtils;
import com.c503.ca.cadv.module.plan.util.PlanHelper;
import com.google.common.collect.Maps;
import org.apache.commons.compress.utils.Lists;
import org.apache.commons.lang.StringUtils;

import java.io.File;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Author hyx
 * @Date 2024/6/14 11:02
 */
public class CheckUtil {

    public static CheckResult check(String targetCode, String filename) {

        Target target = InfoHelper.getTargetService().getByTargetCode(targetCode);
        if (target == null) {
            throw new RuntimeErrorException("目标不存在");
        }
        String targetId = target.getId();

        String taskId = target.getTaskId();
        Task task = InfoHelper.getTaskService().findOne(taskId);
        if (task == null) {
            throw new RuntimeException("任务不存在");
        }

        TargetZbVO targetZb = InfoHelper.getTargetZbService().findByTargetId(targetId);
        if (targetZb == null) {
            throw new RuntimeErrorException("目标主备机未配置");
        }

        RsltFileXDO rsltFileXDO = null;
        String ftpId = null;
        if (StringUtils.isNotEmpty(targetZb.getMainId())) {
            ftpId = targetZb.getMainId();
            rsltFileXDO = getRsltFile(ftpId, taskId, targetId, CatalogType.RSLT, filename);
            if ((rsltFileXDO == null || CollectionsUtils.isEmpty(rsltFileXDO.getStLoadDataList())) && StringUtils.isNotEmpty(targetZb.getStandbyId())) {
                ftpId = targetZb.getStandbyId();
                rsltFileXDO = getRsltFile(ftpId, taskId, targetId, CatalogType.RSLT, filename);
            }
        }
        if (rsltFileXDO == null || CollectionsUtils.isEmpty(rsltFileXDO.getStLoadDataList())) {
            throw new RuntimeErrorException("反算文件数据获取失败");
        }
        CompResult<List<CmdInstVO>, List<StCmdInfoXDO>> compResult = revCalComp(rsltFileXDO, ftpId, taskId, targetCode, filename);
        List<CmdInstVO> cmdInstVOList = compResult.getObj();
        List<StCmdInfoXDO> stCmdInfoXDOList = compResult.getCompObj();
        List<CmdInstVO> removeCmd = new ArrayList<>();
        List<StCmdInfoXDO> removeStCmdInfoXDOList = new ArrayList<>();
        for (CmdInstVO cmdInstVO : cmdInstVOList) {
            List<CmdParamInstVO> cmdParamInstList = cmdInstVO.getCmdParamInstList();
            if (CollectionsUtils.isNotEmpty(cmdParamInstList)) {
                List<CmdParamInstVO> newCmdParamInstList = new ArrayList<>();
                for (CmdParamInstVO cmdParamInstVO : cmdParamInstList) {
                    if (cmdParamInstVO.getCompResult() != CompResult.COMP_EQUAL) {
                        newCmdParamInstList.add(cmdParamInstVO);
                    }
                }
                if (newCmdParamInstList.size() > 0) {
                    cmdInstVO.setCmdParamInstList(newCmdParamInstList);
                } else {
                    removeCmd.add(cmdInstVO);
                }
            } else {
                removeCmd.add(cmdInstVO);
            }
        }
        if (removeCmd.size() > 0) {
            cmdInstVOList.removeAll(removeCmd);
        }
        for (StCmdInfoXDO stCmdInfoXDO : stCmdInfoXDOList) {
            List<StCmdPara> stCmdParaList = stCmdInfoXDO.getStCmdParaList();
            if (CollectionsUtils.isNotEmpty(stCmdParaList)) {
                List<StCmdPara> newStCmdParaList = new ArrayList<>();
                for (StCmdPara stCmdPara : stCmdParaList) {
                    if (stCmdPara.getCompResult() != CompResult.COMP_EQUAL) {
                        newStCmdParaList.add(stCmdPara);
                    }
                }
                if (newStCmdParaList.size() > 0) {
                    stCmdInfoXDO.setStCmdParaList(stCmdParaList);
                } else {
                    removeStCmdInfoXDOList.add(stCmdInfoXDO);
                }
            } else {
                removeStCmdInfoXDOList.add(stCmdInfoXDO);
            }
        }
        if (removeStCmdInfoXDOList.size() > 0) {
            stCmdInfoXDOList.removeAll(removeStCmdInfoXDOList);
        }
        CheckResult checkResult = new CheckResult();
        if (cmdInstVOList.size() > 0 && stCmdInfoXDOList.size() > 0) {
            checkResult.setCompResult(compResult);
            checkResult.setResult(false);
        } else {
            checkResult.setResult(true);
        }
        return checkResult;
    }

    public static RsltFileXDO getRsltFile(String ftpId, String taskId, String targetId, CatalogType catalogType, String filename) {
        String rsltServerPath = getRsltServerPath(taskId, targetId, catalogType);

        // 反算文件名
        filename = filename + ".rslt.xml";

        // 获取反算文件数据结构
        RsltFileXDO rsltFileXDO = getRsltFile(ftpId, rsltServerPath, filename);
        return rsltFileXDO;
    }

    public static RsltFileXDO getRsltFile(String ftpId, String serverPath, String fileFullName) {
        RsltFileXDO rsltFileXDO = getFileXDO(ftpId, serverPath, fileFullName, RsltFileXDO.class);
        return rsltFileXDO;
    }

    public static String getRsltServerPath(String taskId, String targetId, CatalogType catalogType) {
        String serverPath = getServerPath(taskId, targetId, catalogType, null);
        // 反算文件路径
        String rsltServerPath = serverPath + "rslt/";
        if (CatalogType.PUBDC.equals(catalogType)) {
            rsltServerPath = getServerPath(taskId, targetId, CatalogType.DC, null) + "remote/rslt/";
        }
        return rsltServerPath;
    }

    public static String getServerPath(String taskId, String targetId, CatalogType catalogType, String checkedTargetId) {
        TaskVO taskVO = InfoHelper.getTaskService().findOneVO(taskId);
        if (taskVO == null) {
            throw new RuntimeErrorException("任务不存在");
        }
        TargetVO targetVO = InfoHelper.getTargetService().findOneVO(targetId);
        if (targetVO == null) {
            throw new RuntimeErrorException("目标不存在");
        }
        Catalog catalog = InfoHelper.getCatalogService().findOneByCatalogType(catalogType);
        if (catalog == null) {
            throw new RuntimeErrorException("目录不存在，请配置");
        }
        if (!catalog.getPath().endsWith("/")) {
            catalog.setPath(catalog.getPath() + "/");
        }
        String envPath = System.getenv("BTAS3");
        String path;
        if (catalog.getCatalogType() == CatalogType.PUBDC && StringUtils.isNotEmpty(checkedTargetId)) {
            TargetVO checkTargetVO = InfoHelper.getTargetService().findOneVO(checkedTargetId);
            path = envPath + "/" + taskVO.getTaskNo().toLowerCase() + "/" + checkTargetVO.getTargetNo().toLowerCase() + catalog.getPath() + targetVO.getTargetNo().toLowerCase() + "/";
        } else {
            path = envPath + "/" + taskVO.getTaskNo().toLowerCase() + "/" + targetVO.getTargetNo().toLowerCase() + catalog.getPath();
        }
        if (!path.endsWith("/")) {
            path += "/";
        }
        return path;
    }


    public static  <T> T getFileXDO(String ftpId, String serverPath, String fileFullName, Class<T> clazz) {
        File cacheFile = null;
        try {
            String filePath = serverPath + "/" + fileFullName;
            if (serverPath.endsWith("/")) {
                filePath = serverPath + fileFullName;
            }
            String cacheFilePath = SystemProperties.getInstance().getFileUploadPath() + "/" + IdGen.uuid() + "_" + fileFullName;
            cacheFile = new File(cacheFilePath);
            if (!cacheFile.getParentFile().exists()) {
                cacheFile.getParentFile().mkdirs();
            }
            cacheFile.createNewFile();
            boolean success = FtpUtils.retrieve(ftpId, filePath, cacheFile);
            if (!success) {
                cacheFile.delete();
                throw new RuntimeErrorException("[" + filePath + "]文件不存在");
            }
            if (cacheFile.length() == 0) {
                throw new RuntimeErrorException("[" + filePath + "]文件内容为空");
            }
            Object obj = XmlUtils.fromIgnoreXML(cacheFile, clazz);
            cacheFile.delete();
            return JacksonUtils.getMapper().convertValue(obj, clazz);
        } catch (Exception e) {
            if (cacheFile != null) {
                cacheFile.delete();
            }
            if (e instanceof RuntimeErrorException) {
                throw new RuntimeErrorException(e.getMessage());
            }

            return null;
        }
    }

    public static CompResult<List<CmdInstVO>, List<StCmdInfoXDO>> revCalComp(RsltFileXDO rsltFileXDO, String ftpId, String taskId, String targetId, String filename) {
        // 获取注入配置文件的指令及指令参数
        ConfigFileVO configFileVO = new ConfigFileVO();
        FileDataVO fileDataVO = PlanHelper.getFileDataService().findLastCreatedFileRecord(filename, targetId);
        if (fileDataVO == null) {
            throw new RuntimeException("数据库无[ " + filename +  "]注入数据");
        } else {
            PlanHelper.getFileDataService().setCmdParamFileValue(fileDataVO, targetId);

            List<FileDataVO> fileDataVOList = Lists.newArrayList();
            fileDataVOList.add(fileDataVO);
            configFileVO.setFileDataList(fileDataVOList);
        }

        List<String> szDataList = rsltFileXDO.getStLoadDataList().get(0).getSzData();
        if (null == szDataList) {
            szDataList = Lists.newArrayList();
        }
        //反算文件第一层指令集合
        List<StCmdInfoXDO> stCmdInfoXDOList = rsltFileXDO.getStLoadDataList().get(0).getStCmdInfoList();
        if (null == stCmdInfoXDOList) {
            stCmdInfoXDOList = Lists.newArrayList();
        }

        //cfg第一层指令集合
        List<CmdInstVO> cmdInstVOList = null;
        for (FileDataVO fileData : configFileVO.getFileDataList()) {
            if (rsltFileXDO.getSzFileName().equals(fileData.getFile())) {
                cmdInstVOList = fileData.getCmdInstList();
                if (CollectionsUtils.isNotEmpty(cmdInstVOList)) {
                    for (CmdInstVO cmdInstVO : cmdInstVOList) {
                        if ("zc".equals(fileData.getOutType())) {
                            cmdInstVO.setIsDc(0);
                        } else if ("d".equals(fileData.getOutType())) {
                            cmdInstVO.setIsDc(1);
                        } else {
                            cmdInstVO.setIsDc(2);
                        }
                    }
                }
                break;
            }
        }
        if (cmdInstVOList == null) {
            throw new RuntimeException("无该反算文件名");
        }
        //处理特殊指令
        while (checkAndAddSpecialCmd(cmdInstVOList, ftpId, taskId, targetId, stCmdInfoXDOList));

        // s类型参数的指令移至外层
        moveCmdOfS(cmdInstVOList);
        // B W类型参数处理
        checkAndHandlerBW(cmdInstVOList, stCmdInfoXDOList, szDataList, rsltFileXDO.getStLoadDataList().get(0).getUiCmdNumList());
        // 比对
        compare(cmdInstVOList, stCmdInfoXDOList, ftpId, taskId, targetId);

        // 比对结果对象
        CompResult<List<CmdInstVO>, List<StCmdInfoXDO>> compResult = new CompResult<>();
        compResult.setObj(cmdInstVOList);
        compResult.setCompObj(stCmdInfoXDOList);

        if (CollectionsUtils.isEmpty(cmdInstVOList)) {
            throw new RuntimeErrorException("数据库无[" + filename + "]历史填表指令数据，无法进行比较");
        }

        return compResult;
    }

    public static void compare(List<CmdInstVO> cmdInstVOList, List<StCmdInfoXDO> stCmdInfoXDOList, String ftpId, String taskId, String targetId) {
        int i;
        // 按顺序逐个比对指令
        for (i = 0; i < cmdInstVOList.size() && i < stCmdInfoXDOList.size(); i++) {
            CmdInstVO cmdInstVO = cmdInstVOList.get(i);
            StCmdInfoXDO stCmdInfoXDO = stCmdInfoXDOList.get(i);

            List<CmdParamInstVO> cmdParamInstVOList = cmdInstVO.getCmdParamInstList();
            List<StCmdPara> stCmdParaList = stCmdInfoXDO.getStCmdParaList();
            if (CollectionsUtils.isEmpty(cmdInstVOList) || CollectionsUtils.isEmpty(stCmdParaList)) {
                continue;
            }
            int j;
            // 按顺序逐个比对指令参数
            for (j = 0; j < cmdParamInstVOList.size() && j < stCmdParaList.size(); j++) {
                CmdParamInstVO cmdParamInstVO = cmdParamInstVOList.get(j);
                StCmdPara stCmdPara = stCmdParaList.get(j);
                cmdParamInstVO.setCmdCode(cmdInstVO.getCmdCode());
                // 指令参数比对
                cmdParamComp(cmdParamInstVO, stCmdPara, ftpId, taskId, targetId);
                // 取值范围校验
                // 参数比对结果为相等时
                if (cmdParamInstVO.getCompResult() != null && CompResult.COMP_EQUAL == cmdParamInstVO.getCompResult()) {
                    if (StringUtils.isEmpty(cmdParamInstVO.getValueRange())) {
                        // 取值范围为空，判断长度
                        checkRangeWithLength(cmdParamInstVO);
                    } else {
                        // 按取值范围校验
                        checkValueRange(cmdParamInstVO);
                    }
                }
            }
            if (!cmdInstVO.getCmdCode().equals(stCmdInfoXDO.getSzZlCode())) {
                for (int k = 0; k < cmdParamInstVOList.size(); k++) {
                    cmdParamInstVOList.get(k).setParamValue("无比对对象");
                    cmdParamInstVOList.get(k).setCompValue("无比对对象");
                    cmdParamInstVOList.get(k).setCompResult(CompResult.COMP_NO_OBJ);
                }
                for (int k = 0; k < stCmdParaList.size(); k++) {
                    stCmdParaList.get(k).setCompValue("无比对对象");
                    stCmdParaList.get(k).setCompResult(CompResult.COMP_NO_OBJ);
                }
            } else {
                // 逐个比较后，cfg指令参数数目超出
                while (j < cmdParamInstVOList.size()) {
                    cmdParamInstVOList.get(j).setParamValue("无比对对象");
                    cmdParamInstVOList.get(j).setCompValue("无比对对象");
                    cmdParamInstVOList.get(j).setCompResult(CompResult.COMP_NO_OBJ);
                    j++;
                }
                // 逐个比较后，反算指令参数数目超出
                while (j < stCmdParaList.size()) {
                    stCmdParaList.get(j).setCompValue("无比对对象");
                    stCmdParaList.get(j).setCompResult(CompResult.COMP_NO_OBJ);
                    j++;
                }
            }
        }
        // 逐个比较后，cfg指令数目超出
        while (i < cmdInstVOList.size()) {
            List<CmdParamInstVO> cmdParamInstVOList = cmdInstVOList.get(i).getCmdParamInstList();
            for (CmdParamInstVO cmdParamInstVO : cmdParamInstVOList) {
                cmdParamInstVO.setParamValue("无比对对象");
                cmdParamInstVO.setCompValue("无比对对象");
                cmdParamInstVO.setCompResult(CompResult.COMP_NO_OBJ);
            }
            i++;
        }
        // 逐个比较后，反算指令数目超出
        while (i < stCmdInfoXDOList.size()) {
            List<StCmdPara> stCmdParaList = stCmdInfoXDOList.get(i).getStCmdParaList();
            for (StCmdPara stCmdPara : stCmdParaList) {
                stCmdPara.setCompValue("无比对对象");
                stCmdPara.setCompResult(CompResult.COMP_NO_OBJ);
            }
            i++;
        }

    }

    public static void checkValueRange(CmdParamInstVO cmdParamInstVO) {
        try {
            BigDecimal paramValue = toBigDecimal(cmdParamInstVO.getParamValue());
            String[] valueRanges = cmdParamInstVO.getValueRange().split("\\|");
            for (String range : valueRanges) {
                if (range.charAt(0) == '=') {
                    BigDecimal r = toBigDecimal(range.substring(1));
                    if (paramValue.compareTo(r) == 0) {
                        return;
                    }
                }
                if (range.contains("~")) {
                    BigDecimal min = toBigDecimal(range.substring(0, range.indexOf('~')));
                    BigDecimal max = toBigDecimal(range.substring(range.indexOf('~') + 1));
                    if (paramValue.compareTo(min) >= 0 && paramValue.compareTo(max) <= 0) {
                        return;
                    }
                }
            }
            cmdParamInstVO.setCompResult(CompResult.OUT_RANGE);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void checkRangeWithLength(CmdParamInstVO cmdParamInstVO) {
        try {
            String paramValue = cmdParamInstVO.getParamValue();
            BigDecimal pv = toBigDecimal(paramValue);
            if (cmdParamInstVO.getType().equals(CmdParamUtil.getType(CmdParamUtil.SIGNED))) {
                BigDecimal minLength = BigDecimal.valueOf((-1) * Math.pow(2, cmdParamInstVO.getLen() - 1));
                BigDecimal maxLength = BigDecimal.valueOf(Math.pow(2, cmdParamInstVO.getLen() - 1));
                if (!(pv.compareTo(minLength) >= 0 && pv.compareTo(maxLength) <= 0)) {
                    cmdParamInstVO.setCompResult(CompResult.OUT_RANGE);
                }
            }
            if (cmdParamInstVO.getType().equals(CmdParamUtil.getType(CmdParamUtil.UNSIGNED))) {
                BigDecimal maxLength = BigDecimal.valueOf(Math.pow(2, cmdParamInstVO.getLen()));
                if (!(pv.compareTo(BigDecimal.valueOf(0)) >= 0 && pv.compareTo(maxLength) <= 0)) {
                    cmdParamInstVO.setCompResult(CompResult.OUT_RANGE);
                }

            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static BigDecimal toBigDecimal(String value) throws Exception {
        BigDecimal pv;
        if (value.charAt(value.length() - 1) == 'H') {
            value = value.substring(0, value.length() - 1);
            pv = new BigDecimal(Long.valueOf(value, 16));
        } else {
            pv = new BigDecimal(value);
        }
        return pv;
    }

    /**
     * 指令参数比较
     */
    public static void cmdParamComp(CmdParamInstVO cmdParamInstVO,
                              StCmdPara stCmdPara,
                              String ftpId, String taskId, String targetId) {
        // 单个参数标识
        switch (cmdParamInstVO.getSingleParamSign()) {
            case A:
                // 无操作
                break;
            case M:
            case F:
            case W:
            case B:
                compFileValue(cmdParamInstVO, stCmdPara);
                break;
            case C:
                // 比较填表值（不考虑参数类型）
                compC(cmdParamInstVO, stCmdPara);
                break;
            case D:
                // 比较标称值
                compD(cmdParamInstVO, stCmdPara);
                break;
            case S:
                // 第二层指令比对
                compare(cmdParamInstVO.getCmdInstList(), stCmdPara.getStCmdInfoXDOList(), ftpId, taskId, targetId);
                break;
            case T:
            case V:
                // 比较填表值（考虑参数类型）
                compT(cmdParamInstVO, stCmdPara);
                break;
            default:
                break;
        }
    }

    public static boolean compFileValue(CmdParamInstVO cmdParamInstVO, StCmdPara stCmdPara) {
        if (cmdParamInstVO.getFileValue() == null) {
            cmdParamInstVO.setCompValue("文件值获取失败");
            cmdParamInstVO.setCompResult(CompResult.COMP_NO_FILE_VALUE);
            return false;
        }
        cmdParamInstVO.setParamValue(cmdParamInstVO.getFileValue());
        if(cmdParamInstVO.getParamValue().equals(stCmdPara.getSzCalcValue())){
            stCmdPara.setCompValue(stCmdPara.getSzCalcValue());
            cmdParamInstVO.setCompValue(cmdParamInstVO.getParamValue());
            cmdParamInstVO.setCompResult(CompResult.COMP_EQUAL);
            stCmdPara.setCompResult(CompResult.COMP_EQUAL);
            return true;
        }

        return CompUtils.cmdParamValAndRsltFileVal(cmdParamInstVO, stCmdPara);
    }

    public static boolean compC(CmdParamInstVO cmdParamInstVO, StCmdPara stCmdPara) {
        cmdParamInstVO.setParamValue(cmdParamInstVO.getValue());
        cmdParamInstVO.setCompValue(cmdParamInstVO.getParamValue());
        stCmdPara.setCompValue(stCmdPara.getSzCalcValue());
        boolean equal = cmdParamInstVO.getCompValue().equals(stCmdPara.getCompValue());
        if (!equal) {
            equal = RevCalCompConfigUtils.check(cmdParamInstVO.getCompValue(), stCmdPara.getCompValue(), cmdParamInstVO.getObjectCode(), cmdParamInstVO);
        }
        cmdParamInstVO.setCompResult(equal ? CompResult.COMP_EQUAL : CompResult.COMP_NO_EQUAL);
        stCmdPara.setCompResult(equal ? CompResult.COMP_EQUAL : CompResult.COMP_NO_EQUAL);
        return equal;
    }

    public static boolean compD(CmdParamInstVO cmdParamInstVO, StCmdPara stCmdPara) {
        cmdParamInstVO.setParamValue(cmdParamInstVO.getFix());
        if(StringUtils.isNotEmpty(cmdParamInstVO.getParamValue()) && cmdParamInstVO.getParamValue().equals(stCmdPara.getSzCalcValue())){
            stCmdPara.setCompValue(stCmdPara.getSzCalcValue());
            cmdParamInstVO.setCompValue(cmdParamInstVO.getParamValue());
            cmdParamInstVO.setCompResult(CompResult.COMP_EQUAL);
            stCmdPara.setCompResult(CompResult.COMP_EQUAL);
            return true;
        }
        return CompUtils.cmdParamValAndRsltFileVal(cmdParamInstVO, stCmdPara);
    }

    public static boolean compT(CmdParamInstVO cmdParamInstVO, StCmdPara stCmdPara) {
        cmdParamInstVO.setParamValue(cmdParamInstVO.getValue());
        if(StringUtils.isNotEmpty(cmdParamInstVO.getParamValue()) && cmdParamInstVO.getParamValue().equals(stCmdPara.getSzCalcValue())){
            stCmdPara.setCompValue(stCmdPara.getSzCalcValue());
            cmdParamInstVO.setCompValue(cmdParamInstVO.getParamValue());
            cmdParamInstVO.setCompResult(CompResult.COMP_EQUAL);
            stCmdPara.setCompResult(CompResult.COMP_EQUAL);
            return true;
        }
        return CompUtils.cmdParamValAndRsltFileVal(cmdParamInstVO, stCmdPara);
    }

    public static void moveCmdOfS(List<CmdInstVO> cmdInstVOList) {
        if (CollectionsUtils.isNotEmpty(cmdInstVOList)) {
            Map<Integer, List<CmdInstVO>> CmdInstVOMapToMove = Maps.newHashMap();
            // 下标偏移量
            int indexOffset = 0;
            for (int i = 0; i < cmdInstVOList.size(); i++) {
                List<CmdParamInstVO> cmdParamInstVOList = cmdInstVOList.get(i).getCmdParamInstList();
                if (CollectionsUtils.isNotEmpty(cmdParamInstVOList)) {
                    for (int j = 0; j < cmdParamInstVOList.size(); j++) {
                        if (ParamSignType.S.equals(cmdParamInstVOList.get(j).getSingleParamSign())) {
                            if (CollectionsUtils.isNotEmpty(cmdParamInstVOList.get(j).getCmdInstList())) {
                                if (CollectionsUtils.isNotEmpty(CmdInstVOMapToMove.get(i))) {
                                    CmdInstVOMapToMove.get(i).addAll(cmdParamInstVOList.get(j).getCmdInstList());
                                } else {
                                    CmdInstVOMapToMove.put(i, cmdParamInstVOList.get(j).getCmdInstList());
                                }
                                cmdParamInstVOList.get(j).setSingleParamSign(ParamSignType.A);
                                cmdParamInstVOList.get(j).setCmdInstList(null);
                            }
                        }
                    }
                }
            }

            if (!CmdInstVOMapToMove.isEmpty()) {
                for (Map.Entry<Integer, List<CmdInstVO>> entry : CmdInstVOMapToMove.entrySet()) {
                    Integer index = entry.getKey();
                    List<CmdInstVO> cmdInstVOListOfS = entry.getValue();
                    for (int i = 0; i < cmdInstVOListOfS.size(); i++) {
                        cmdInstVOList.add(index + indexOffset + 1 + i, cmdInstVOListOfS.get(i));
                    }
                    indexOffset += cmdInstVOListOfS.size();
                }
            }
        }
    }

    public static void checkAndHandlerBW(List<CmdInstVO> cmdInstVOList, List<StCmdInfoXDO> stCmdInfoXDOList, List<String> szDataList, List<String> cmdNumList) {

        //反算指令的下标偏移量
        int indexOffset = 0;
        for (int i = 0; i < cmdInstVOList.size(); i++) {
            CmdInstVO cmdInstVO = cmdInstVOList.get(i);
            StCmdInfoXDO stCmdInfoXD = stCmdInfoXDOList.get(i);
            if (CollectionsUtils.isNotEmpty(cmdInstVO.getCmdParamInstList())) {
                //该指令的全部b类型参数
                List<CmdParamInstVO> allCmaParamOfBList = com.google.common.collect.Lists.newArrayList();
                for (CmdParamInstVO cmdParamInstVO : cmdInstVO.getCmdParamInstList()) {
                    if (ParamSignType.B.equals(cmdParamInstVO.getSingleParamSign()) || ParamSignType.W.equals(cmdParamInstVO.getSingleParamSign())) {
                        //判断反算是否正常的BW类型
                        if (!isNormalBW(cmdInstVO, stCmdInfoXD)) {
                            String value = cmdParamInstVO.getFileValue();
                            if(StringUtils.isNotEmpty(value) && value.toUpperCase().endsWith("H")){
                                value = value.substring(0, value.length() - 1);
                                cmdParamInstVO.setFileValue(value);
                            }
                            allCmaParamOfBList.add(cmdParamInstVO);
                        }
                    }
                }

                //含有b类型参数时
                if (allCmaParamOfBList.size() > 0) {
                    //1.清空填表指令内的非B类型参数
                    cmdInstVO.setCmdParamInstList(allCmaParamOfBList);
                    //填表数据内一条指令生成一遍szData
                    //对应的szData在szDataList的i号位置,该szData反算出来的指令数目uiCmdNumList的i号位置
                    //对应反算指令stCmdInfoXDOList[uiCmdNumList(0) + ...  + uiCmdNumList(i - 1), uiCmdNumList(0) + ...  + uiCmdNumList(i) - 1]

                    //2.删除该段反算指令,并填充构建的数据块指令
                    int cmdNumIndex = 0;
                    //该反算指令的开始下标
                    int cmdInfoStartIndex = 0;
                    while (cmdNumIndex < i) {
                        cmdInfoStartIndex += Integer.parseInt(cmdNumList.get(cmdNumIndex));
                        cmdNumIndex++;
                    }

                    //待删除项目
                    int delNum = 0;
                    if (CollectionsUtils.isNotEmpty(cmdNumList) && StringUtils.isNotEmpty(cmdNumList.get(i))) {
                        delNum = Integer.parseInt(cmdNumList.get(i));
                    }
                    //已删除项目
                    int hasDelNum = 0;
                    while (hasDelNum < delNum) {
                        //一直删除该位置的反算指令,直至删除数目达到该段反算指令的数目
                        stCmdInfoXDOList.remove(cmdInfoStartIndex - indexOffset);
                        hasDelNum++;
                    }
                    //组建反算指令
                    StCmdInfoXDO stCmdInfoXDO = new StCmdInfoXDO();
                    stCmdInfoXDO.setSzZlCode(cmdInstVO.getCmdCode());
                    stCmdInfoXDO.setSzZlName("数据块");
                    stCmdInfoXDO.setStCmdParaList(com.google.common.collect.Lists.newArrayList());
                    int cmdParamNum = cmdInstVO.getCmdParamInstList().size();
                    int temp = 0;
                    while (temp < cmdParamNum) {
                        StCmdPara stCmdPara = new StCmdPara();
                        stCmdPara.setSzValExpln("数据块");
                        stCmdPara.setSzCalcValue(szDataList.get(i));
                        stCmdPara.setCompValue(szDataList.get(i));
                        stCmdInfoXDO.getStCmdParaList().add(stCmdPara);
                        temp++;
                    }

                    //添加至反算指令集合,位置为该反算指令的开始下标
                    stCmdInfoXDOList.add(cmdInfoStartIndex - indexOffset, stCmdInfoXDO);
                    //反算指令集合被修改,下次循环使用的还是旧下标,所以记录下标偏移量
                    indexOffset = indexOffset + hasDelNum - 1;
                }
            }
        }
    }

    public static boolean isNormalBW(CmdInstVO cmdInstVO, StCmdInfoXDO stCmdInfoXDO) {
        //该指令的全部b类型参数
        boolean containBW = false;
        for (CmdParamInstVO cmdParamInstVO : cmdInstVO.getCmdParamInstList()) {
            if (ParamSignType.B.equals(cmdParamInstVO.getSingleParamSign()) || ParamSignType.W.equals(cmdParamInstVO.getSingleParamSign())) {
                containBW = true;
            }
        }
        if (containBW && cmdInstVO.getCmdCode().equals(stCmdInfoXDO.getSzZlCode())) {
            return true;
        }

        return false;
    }


    private static boolean checkAndAddSpecialCmd(List<CmdInstVO> cmdInstVOList, String ftpId, String taskId, String targetId, List<StCmdInfoXDO> stCmdInfoXDOList) {
        boolean existSpecialCmd = false;
        int indexToReplace = -1;
        List<CmdInstVO> cmdInstVOListToAdd = Lists.newArrayList();
        for (int i = 0; i < cmdInstVOList.size(); i++) {
            CmdInstVO cmdInstVO = cmdInstVOList.get(i);
            if (StringUtils.isNotEmpty(cmdInstVO.getCmdCode()) && cmdInstVO.getCmdCode().startsWith("RPPPC")) {
                Target target = InfoHelper.getTargetService().findOne(targetId);
                if (target.getTargetType().equals(TargetType.MAN_AIRSHIP.getValue()) || target.getTargetType().equals(TargetType.GOODS_AIRSHIP.getValue())) {
                    //非三舱
                    cmdInstVOListToAdd = handleRpppcCmd(cmdInstVO, ftpId, taskId, targetId);
                    indexToReplace = i;
                    existSpecialCmd = true;
                    break;
                } else {
                    //三舱
                    if ("RPPPC_C".equals(cmdInstVO.getCmdCode()) || "RPPPC_DC".equals(cmdInstVO.getCmdCode())) {
                        //串行
                        cmdInstVOListToAdd = handleRpppcCmdCHorGT(cmdInstVO, ftpId, taskId, targetId, "c");
                        indexToReplace = i;
                        existSpecialCmd = true;
                        break;
                    } else if ("RPPPC_G".equals(cmdInstVO.getCmdCode())) {
                        //高通
                        cmdInstVOListToAdd = handleRpppcCmdCHorGT(cmdInstVO, ftpId, taskId, targetId, "g");
                        indexToReplace = i;
                        existSpecialCmd = true;
                        break;
                    }
                }
            }

            if ("RACPA".equals(cmdInstVO.getCmdCode())) {
                cmdInstVOListToAdd = handleRacpaCmd(cmdInstVO, ftpId, taskId, targetId);
                indexToReplace = i;
                existSpecialCmd = true;
                break;
            }
            if ("RSBEOD".equals(cmdInstVO.getCmdCode())) {
                cmdInstVOListToAdd = handleRsbeodCmd(cmdInstVO, ftpId, taskId, targetId);
                indexToReplace = i;
                existSpecialCmd = true;
                break;
            }
            if (StringUtils.isNotEmpty(cmdInstVO.getCmdCode()) && cmdInstVO.getCmdCode().startsWith("RPRLYTRK")) {
                cmdInstVOListToAdd = handleRprlytrkCmd(cmdInstVO, ftpId, taskId, targetId);
                indexToReplace = i;
                existSpecialCmd = true;
                break;
            }
            if ("RPGNC".equals(cmdInstVO.getCmdCode())) {
                Target target = InfoHelper.getTargetService().findOne(targetId);
                if (!target.getTargetType().equals(TargetType.GOODS_AIRSHIP.getValue())) {
                    int gncfIndex = i + 1;
                    CmdInstVO GNCF = null;
                    for (; gncfIndex < cmdInstVOList.size(); gncfIndex ++) {
                        if ("GNCF1".equals(cmdInstVOList.get(gncfIndex).getCmdCode()) || "GNCF2".equals(cmdInstVOList.get(gncfIndex).getCmdCode())) {
                            GNCF = cmdInstVOList.get(gncfIndex);
                            break;
                        }
                    }
                    if (GNCF != null) {
                        handleRpgncCmd(cmdInstVO, GNCF, ftpId, taskId, targetId, "GNCF");
                        indexToReplace = i;
                        existSpecialCmd = true;
                        break;
                    }
                } else {
                    int gnc02Index = i + 1;
                    CmdInstVO GNC02 = null;
                    for (; gnc02Index < cmdInstVOList.size(); gnc02Index++) {
                        if ("GNC02".equals(cmdInstVOList.get(gnc02Index).getCmdCode())) {
                            GNC02 = cmdInstVOList.get(gnc02Index);
                            break;
                        }
                    }
                    if (GNC02 != null) {
                        handleRpgncCmd(cmdInstVO, GNC02, ftpId, taskId, targetId, "GNC02");
                        indexToReplace = i;
                        existSpecialCmd = true;
                        break;
                    }
                }
            }
            // 新增第2圈返回GNC参数设置比对，替换临时做的第二圈反算比对
            if ("SECOND".equals(cmdInstVO.getCmdCode())) {
                cmdInstVOListToAdd = handleSECONDCmd(cmdInstVO, targetId, stCmdInfoXDOList, i);
                indexToReplace = i;
                existSpecialCmd = true;
                break;
            }
        }

        // 替换并添加指令
        if (existSpecialCmd && indexToReplace != -1) {
            cmdInstVOList.remove(indexToReplace);
            for (int i = 0; i < cmdInstVOListToAdd.size(); i++) {
                // 在删除位置添加
                cmdInstVOList.add(indexToReplace + i, cmdInstVOListToAdd.get(i));
            }
        }

        return existSpecialCmd;
    }



    private static List<CmdInstVO> handleRpppcCmd(CmdInstVO cmdInstVO, String ftpId, String taskId, String targetId) {
        List<CmdInstVO> cmdInstVOListToAdd = Lists.newArrayList();

        String rpppcFileName = cmdInstVO.getCmdParamInstList().get(0).getValue();
        ParamSignType singleParamSign = cmdInstVO.getCmdParamInstList().get(0).getSingleParamSign();
        if (singleParamSign == ParamSignType.D || singleParamSign == ParamSignType.M) {
            rpppcFileName = cmdInstVO.getCmdParamInstList().get(0).getParaPath();
        }

        String beginTime = cmdInstVO.getCmdParamInstList().get(1).getValue();
        String endTime = cmdInstVO.getCmdParamInstList().get(2).getValue();

        //段标
        String segmentMark = cmdInstVO.getCmdParamInstList().get(3).getValue();
        //生存模式
        String generateMode = cmdInstVO.getCmdParamInstList().get(4).getValue();
        if (StringUtils.isEmpty(generateMode) && ParamSignType.D.equals(cmdInstVO.getCmdParamInstList().get(4).getSingleParamSign()) && StringUtils.isNotEmpty(cmdInstVO.getCmdParamInstList().get(4).getFix())) {
            generateMode = cmdInstVO.getCmdParamInstList().get(4).getFix();
        }

        Target target = InfoHelper.getTargetService().getById(targetId);

        FillConfigVO fillConfigVO = CmpHelper.getFillConfigService().findWithInjTypeAndGenMode(cmdInstVO.getCmdCode(), generateMode, target.getTargetCode(), cmdInstVO.getIsDc());
        if (fillConfigVO == null) {
            throw new RuntimeException("特殊比对[RPPPC]的生成模式[" + generateMode + "]未配置");
        }

        //获取rpppc文件数据对象
        String orinServerPath = getServerPath(taskId, targetId, CatalogType.ORIN, null);
        RpppcXDO rpppcXDO = getFileXDO(ftpId, orinServerPath, rpppcFileName, RpppcXDO.class);
        if (rpppcXDO == null) {
            throw new RuntimeException(rpppcFileName + "获取失败");
        }
        if ("DQ".equals(segmentMark)) {
            segmentMark = "YX";
        }
        //根据开始时间和结束时间过滤rpppc指令参数信息
        String finalPhase = StringUtils.isEmpty(segmentMark) ? "YX" : segmentMark;
        List<StrRpppcBody> filterStrRpppcBodyList = rpppcXDO.getStrRpppcBodyList().stream()
                .filter(strRpppcBody -> finalPhase.equals(strRpppcBody.getCPhase()) && ((beginTime.compareTo(strRpppcBody.getCEventTimeSpan()) < 0
                && endTime.compareTo(strRpppcBody.getCEventTimeSpan()) > 0)
                || strRpppcBody.getCEventTimeSpan().equals(beginTime)
                || strRpppcBody.getCEventTimeSpan().equals(endTime))).collect(Collectors.toList());
        int rpppcBodyListSize = filterStrRpppcBodyList.size();

        //指令模板
        CmdVO cmdVOOfDel = CmdHelper.getCmdService().getCmdWithParamByTargetIdAndCode(targetId, fillConfigVO.getDelCmdCode());
        CmdVO cmdVOOfAdd = CmdHelper.getCmdService().getCmdWithParamByTargetIdAndCode(targetId, fillConfigVO.getAddCmdCode());
        if (cmdVOOfDel == null) {
            throw new RuntimeException("无" + fillConfigVO.getAddCmdCode() + "指令，比对失败");
        }
        if (cmdVOOfAdd == null) {
            throw new RuntimeException("无" + fillConfigVO.getAddCmdCode() + "指令，比对失败");
        }

        //每帧上限
        int upperLimit = fillConfigVO.getUppperLimit();
        //时间偏移量
        double timeOffset = fillConfigVO.getTimeOffset();
        int result = 0;
        if ("1".equals(generateMode)) {
            //边增边删
            //DMS3指令单元数
            String cmdUnitNum = "2";
            //[DMS5+DMS3(以21个rpppc指令参数信息为一组) * 若干]
            int index = 0;
            while (index + upperLimit <= rpppcBodyListSize) {
                StrRpppcBody comp = null;
                while (index + upperLimit <= rpppcBodyListSize) {
                    comp = filterStrRpppcBodyList.get(index + upperLimit);
                }
                result = addCmdInstVOByRpppc(cmdInstVOListToAdd, filterStrRpppcBodyList.subList(index, index + upperLimit), cmdVOOfAdd, cmdVOOfDel, cmdUnitNum, segmentMark, timeOffset, comp, 1);
                index = index + upperLimit - result;
            }
            //加入剩余不足21条的StrRpppcBody
            addCmdInstVOByRpppc(cmdInstVOListToAdd, filterStrRpppcBodyList.subList(index, rpppcBodyListSize), cmdVOOfAdd, cmdVOOfDel, cmdUnitNum, segmentMark, timeOffset, null, null);
            //第一条DMS5的开始时间为用户填写的开始时间
            cmdInstVOListToAdd.get(0).getCmdParamInstList().get(1).setValue(beginTime);
            //最后一条DMS5的结束时间为用户填写的结束时间
            cmdInstVOListToAdd.get(cmdInstVOListToAdd.size() - 2).getCmdParamInstList().get(2).setValue(endTime);
            for (int i = 2; i < cmdInstVOListToAdd.size(); i = i + 2) {
                cmdInstVOListToAdd.get(i).getCmdParamInstList().get(1).setValue(cmdInstVOListToAdd.get(i - 2).getCmdParamInstList().get(2).getValue());
            }
        } else if ("2".equals(generateMode)) {
            //先删后增
            //DMS3指令单元数
            String cmdUnitNum = "1";
            //只有一个DMS5，其开始时间 = 用户填写的开始时间，结束时间 = 用户填写的结束时间，其余为DMS3
            //第一条DMS3的上限少一
            int firstUpperLimit = upperLimit - 1;
            //第一条DMS3的指令单元数为2
            String firstCmdUnitNum = "2";
            if (firstUpperLimit >= rpppcBodyListSize) {
                addCmdInstVOByRpppc(cmdInstVOListToAdd, filterStrRpppcBodyList, cmdVOOfAdd, cmdVOOfDel, firstCmdUnitNum, segmentMark, timeOffset, null, null);
            } else {
                result = addCmdInstVOByRpppc(cmdInstVOListToAdd, filterStrRpppcBodyList.subList(0, firstUpperLimit), cmdVOOfAdd, cmdVOOfDel, firstCmdUnitNum, segmentMark, timeOffset, filterStrRpppcBodyList.get(firstUpperLimit), 1);
                int index = firstUpperLimit - result;
                while (index + upperLimit <= rpppcBodyListSize) {
                    StrRpppcBody comp = null;
                    if (index + upperLimit < rpppcBodyListSize) {
                        comp = filterStrRpppcBodyList.get(index + upperLimit - 1);
                    }
                    result = addCmdInstVOByRpppc(cmdInstVOListToAdd, filterStrRpppcBodyList.subList(index, index + upperLimit), cmdVOOfAdd, null, cmdUnitNum, segmentMark, timeOffset, comp, 2);
                    index = index + upperLimit - result;
                }
                //加入剩余不足22条的StrRpppcBody
                addCmdInstVOByRpppc(cmdInstVOListToAdd, filterStrRpppcBodyList.subList(index, rpppcBodyListSize), cmdVOOfAdd, null, cmdUnitNum, segmentMark, timeOffset, null, null);
            }
            //第一条为DMS5，开始时间为用户填写的开始时间，结束时间为用户填写的结束时间
            cmdInstVOListToAdd.get(0).getCmdParamInstList().get(1).setValue(beginTime);
            cmdInstVOListToAdd.get(0).getCmdParamInstList().get(2).setValue(endTime);
        } else if ("3".equals(generateMode)) {
            //仅增不减
            //DMS3指令单元数
            String cmdUnitNum = "1";
            //只有DMS3
            int index = 0;
            while (index + upperLimit <= rpppcBodyListSize) {
                StrRpppcBody comp = null;
                if (index + upperLimit < rpppcBodyListSize) {
                    comp = filterStrRpppcBodyList.get(index + upperLimit - 1);
                }
                result = addCmdInstVOByRpppc(cmdInstVOListToAdd, filterStrRpppcBodyList.subList(index, index + upperLimit), cmdVOOfAdd, null, cmdUnitNum, segmentMark, timeOffset, comp, 2);
                index = index + upperLimit - result;
            }
            //加入剩余不足22条的StrRpppcBody
            addCmdInstVOByRpppc(cmdInstVOListToAdd, filterStrRpppcBodyList.subList(index, filterStrRpppcBodyList.size()), cmdVOOfAdd, null, cmdUnitNum, segmentMark, timeOffset, null, null);
        }
        return cmdInstVOListToAdd;
    }

    private static int addCmdInstVOByRpppc(List<CmdInstVO> cmdInstVOListToAdd, List<StrRpppcBody> subRpppcList, CmdVO cmdVOOfAdd, CmdVO cmdVOOfDel, String cmdUnitNum, String segmentMark, double timeOffset, StrRpppcBody comp, Integer reduceIndex) {
        if (CollectionsUtils.isEmpty(subRpppcList)) {
            return -1;
        }
        //DMS3
        CmdInstVO cmdInstVOOfAdd = BeanConvertUtils.map(cmdVOOfAdd, CmdInstVO.class);
        List<CmdParamInstVO> cmdParamInstVOListOfAdd = JacksonUtils.deserializeList(JacksonUtils.toJsonString(cmdVOOfAdd.getCmdParamVOList()), CmdParamInstVO.class);

        //注入序号 取标称值
        cmdParamInstVOListOfAdd.get(0).setSingleParamSign(ParamSignType.V);
        cmdParamInstVOListOfAdd.get(0).setValue(cmdParamInstVOListOfAdd.get(0).getFix());
        //指令单元数
        cmdParamInstVOListOfAdd.get(1).setSingleParamSign(ParamSignType.V);
        cmdParamInstVOListOfAdd.get(1).setValue(cmdUnitNum);
        //数据长度
        cmdParamInstVOListOfAdd.get(2).setSingleParamSign(ParamSignType.A);
        cmdParamInstVOListOfAdd.get(2).setValue(null);
        //段标
        cmdParamInstVOListOfAdd.get(3).setSingleParamSign(ParamSignType.V);
        cmdParamInstVOListOfAdd.get(3).setValue(segmentMark);
        //指令时间码
        cmdParamInstVOListOfAdd.get(4).setSingleParamSign(ParamSignType.V);
        cmdParamInstVOListOfAdd.get(4).setValue(subRpppcList.get(0).getCEventTimeSpan());
        //指令名称
        cmdParamInstVOListOfAdd.get(5).setSingleParamSign(ParamSignType.V);
        cmdParamInstVOListOfAdd.get(5).setType(CmdParamUtil.getType(CmdParamUtil.DATABUF));
        cmdParamInstVOListOfAdd.get(5).setValue(subRpppcList.get(0).getCEventCode() + " " + subRpppcList.get(0).getCEventName());
        int result = 0;
        int size = subRpppcList.size();

        if (comp != null && subRpppcList.get(size - reduceIndex).getCEventTimeSpan().equals(comp.getCEventTimeSpan())) {
            result = reduceIndex;
            size = size - reduceIndex;
        }

        for (int i = 0; i < size; i++) {
            //指令时间码
            CmdParamInstVO cmdParamInstVO = BeanConvertUtils.map(cmdParamInstVOListOfAdd.get(4), CmdParamInstVO.class);
            cmdParamInstVO.setValue(subRpppcList.get(i).getCEventTimeSpan());
            cmdParamInstVOListOfAdd.add(cmdParamInstVO);
            //指令名称
            CmdParamInstVO cmdParamInstVO1 = BeanConvertUtils.map(cmdParamInstVOListOfAdd.get(5), CmdParamInstVO.class);
            cmdParamInstVO1.setValue(subRpppcList.get(i).getCEventCode() + " " + subRpppcList.get(i).getCEventName());
            cmdParamInstVOListOfAdd.add(cmdParamInstVO1);
        }
        if (cmdVOOfDel != null) {
            //DMS5
            CmdInstVO cmdInstVOOfDel = BeanConvertUtils.map(cmdVOOfDel, CmdInstVO.class);
            List<CmdParamInstVO> cmdParamInstVOListOfDel = JacksonUtils.deserializeList(JacksonUtils.toJsonString(cmdVOOfDel.getCmdParamVOList()), CmdParamInstVO.class);
            cmdInstVOOfDel.setCmdParamInstList(cmdParamInstVOListOfDel);

            //去除注入序号
            cmdParamInstVOListOfDel.remove(0);
            //去除指令单元数
            cmdParamInstVOListOfDel.remove(0);
            //段标
            cmdParamInstVOListOfDel.get(0).setSingleParamSign(ParamSignType.V);
            cmdParamInstVOListOfDel.get(0).setValue(segmentMark);
            //开始时间为上一条DMS的最后一个指令时间码 + 时间偏移量
            cmdParamInstVOListOfDel.get(1).setValue(cmdParamInstVOListOfAdd.get(4).getValue());
            //结束时间
            cmdParamInstVOListOfDel.get(2).setSingleParamSign(ParamSignType.V);
            //DMS5后面的DMS3的最后一个指令时间码 + 时间偏移量
            String rltTime = BigDecimalToRLLTimeStr(RlTTimeStrToBigDecimal(cmdParamInstVOListOfAdd.get(cmdParamInstVOListOfAdd.size() - 2).getValue()).add(BigDecimal.valueOf(timeOffset)));
            cmdParamInstVOListOfDel.get(2).setValue(rltTime);

            cmdInstVOListToAdd.add(cmdInstVOOfDel);
        }
        cmdInstVOListToAdd.add(cmdInstVOOfAdd);
        return result;
    }

    public static BigDecimal RlTTimeStrToBigDecimal(String rltTime) {
        if (StringUtils.isNotEmpty(rltTime)) {
            if (rltTime.contains("-") && rltTime.contains("T")) {
                return RlTTimeStrToBigDecimal2(rltTime);
            } else if (rltTime.contains("T")) {
                return RlTTimeStrToBigDecimal1(rltTime);
            }
        }
        return null;
    }

    public static BigDecimal RlTTimeStrToBigDecimal1(String rltTime) {
        String day = rltTime.substring(0, rltTime.indexOf('T'));
        String hour = rltTime.substring(rltTime.indexOf('T') + 1, rltTime.indexOf(':'));
        String min = rltTime.substring(rltTime.indexOf(':') + 1, rltTime.lastIndexOf(':'));
        String s = rltTime.substring(rltTime.lastIndexOf(':') + 1, rltTime.indexOf('.'));
        String ms = rltTime.substring(rltTime.indexOf('.') + 1);
        String time = (Integer.parseInt(day) * 24 * 60 * 60 + Integer.parseInt(hour) * 60 * 60 + Integer.parseInt(min) * 60 + Integer.parseInt(s)) + "." + ms;
        return new BigDecimal(time);
    }

    public static BigDecimal RlTTimeStrToBigDecimal2(String rltTime) {
        String day = rltTime.substring(rltTime.lastIndexOf('-') + 1, rltTime.indexOf('T'));
        String hour = rltTime.substring(rltTime.indexOf('T') + 1, rltTime.indexOf(':'));
        String min = rltTime.substring(rltTime.indexOf(':') + 1, rltTime.lastIndexOf(':'));
        String s = rltTime.substring(rltTime.lastIndexOf(':') + 1, rltTime.indexOf('.'));
        String ms = rltTime.substring(rltTime.indexOf('.') + 1);
        String time = (Integer.parseInt(day) * 24 * 60 * 60 + Integer.parseInt(hour) * 60 * 60 + Integer.parseInt(min) * 60 + Integer.parseInt(s) + "." + ms);
        return new BigDecimal(time);
    }

    public static String BigDecimalToRLLTimeStr(BigDecimal bigDecimal) {
        int day = bigDecimal.intValue() / (60 * 60 * 24);
        int hour = (bigDecimal.intValue() - day * 24 * 60 * 60) / (60 * 60);
        int min = (bigDecimal.intValue() - day * 24 * 60 * 60 - hour * 60 * 60) / 60;
        int sec = (bigDecimal.intValue() - day * 24 * 60 * 60 - hour * 60 * 60 - min * 60);

        String d = Integer.toString(day);
        while (d.length() < 4) {
            d = "0" + d;
        }

        String h = Integer.toString(hour);
        while (h.length() < 2) {
            h = "0" + h;
        }

        String m = Integer.toString(min);
        while (m.length() < 2) {
            m = "0" + m;
        }

        String s = Integer.toString(sec);
        while (s.length() < 2) {
            s = "0" + s;
        }
        return d + "T" + h + ":" + m + ":" + s + bigDecimal.toPlainString().substring(bigDecimal.toPlainString().lastIndexOf('.'));
    }


    public static List<CmdInstVO> handleRpppcCmdCHorGT(CmdInstVO cmdInstVO, String ftpId, String taskId, String targetId, String type) {
        List<CmdInstVO> cmdInstVOListToAdd = Lists.newArrayList();

        String rpppcFileName = cmdInstVO.getCmdParamInstList().get(0).getValue();
        rpppcFileName = rpppcFileName == null ? "rpppc.xml" : rpppcFileName;

        String beginTime = cmdInstVO.getCmdParamInstList().get(1).getValue();
        String endTime = cmdInstVO.getCmdParamInstList().get(2).getValue();

        //生成模式
        String generateMode = cmdInstVO.getCmdParamInstList().get(4).getValue();
        if (StringUtils.isEmpty(generateMode) && ParamSignType.D.equals(cmdInstVO.getCmdParamInstList().get(4).getSingleParamSign()) && StringUtils.isNotEmpty(cmdInstVO.getCmdParamInstList().get(4).getFix())) {
            generateMode = cmdInstVO.getCmdParamInstList().get(4).getFix();
        }

        //获取rpppc文件数据对象
        String orinServerPath = getServerPath(taskId, targetId, CatalogType.ORIN, null);
        RpppcXDO rpppcXDO = getFileXDO(ftpId, orinServerPath, rpppcFileName, RpppcXDO.class);
        if (rpppcXDO == null) {
            throw new RuntimeException(rpppcFileName + "获取失败");
        }
        //根据开始时间和结束时间过滤rpppc指令参数信息
        List<StrRpppcBody> filterStrRpppcBodyList = rpppcXDO.getStrRpppcBodyList().stream()
                .filter(strRpppcBody ->  ((beginTime.compareTo(strRpppcBody.getCEventTimeSpan()) < 0
                        && endTime.compareTo(strRpppcBody.getCEventTimeSpan()) > 0)
                        || strRpppcBody.getCEventTimeSpan().equals(beginTime)
                        || strRpppcBody.getCEventTimeSpan().equals(endTime))).collect(Collectors.toList());
        int rpppcBodyListSize = filterStrRpppcBodyList.size();

        String delCode = "null", addCode = "null";
        //上限
        int upperLimit = 9;
        if ("c".equals(type)) {
            delCode = "TS2000+TS0004";
            addCode = "TS2000+TS0003";
            upperLimit = 9;
        } else if ("g".equals(type)) {
            delCode = "TS1000+TS0004";
            addCode = "TS1000+TS0003";
            upperLimit = 39;
        }
        //指令模板
        CmdVO cmdVOOfDel = CmdHelper.getCmdService().getCmdWithParamByTargetIdAndCode(targetId, delCode);
        CmdVO cmdVOOfAdd = CmdHelper.getCmdService().getCmdWithParamByTargetIdAndCode(targetId, addCode);

        //时间偏移量(ms)
        double timeOffset = 500;
        if ("1".equals(generateMode)) {
            //边删边增
            //[TS2000+TS0004 TS2000+TS0003 * 9] * 若干
            int index = 0;
            while (index + upperLimit <= rpppcBodyListSize) {
                addCmdInstVOByRpppcCHorGT(cmdInstVOListToAdd, filterStrRpppcBodyList.subList(index, index + upperLimit - 1), cmdVOOfAdd, cmdVOOfDel, timeOffset);
                index += upperLimit - 1;
            }
            //加入剩余不足9条的StrRpppcBody
            addCmdInstVOByRpppcCHorGT(cmdInstVOListToAdd, filterStrRpppcBodyList.subList(index, rpppcBodyListSize), cmdVOOfAdd, cmdVOOfDel, timeOffset);
            //第一条0004开始时间为用户填写的开始时间 - 0.1ms
            String rltTime = BigDecimalToRLLTimeStr(RlTTimeStrToBigDecimal(beginTime).add(BigDecimal.valueOf(-0.1)));
            cmdInstVOListToAdd.get(0).getCmdParamInstList().get(2).setValue(rltTime);
            //最后一条0004的结束时间为用户填写的结束时间 -0.1ms
            int lastIndex = rpppcBodyListSize / (upperLimit + 1) * (upperLimit + 1);
            String rltTime2 = BigDecimalToRLLTimeStr(RlTTimeStrToBigDecimal(endTime).add(BigDecimal.valueOf(-0.1)));
            cmdInstVOListToAdd.get(lastIndex).getCmdParamInstList().get(2).setValue(rltTime2);
        } else if ("2".equals(generateMode)) {
            //先删后增
            //只有一个0004
            addCmdInstVOByRpppcCHorGT(cmdInstVOListToAdd, filterStrRpppcBodyList, cmdVOOfAdd, cmdVOOfDel, timeOffset);

            //第一条为0004开始时间为用户填写的开始时间+时间偏移量，结束时间为用户填写的结束时间+时间偏移量
            String rltTime = BigDecimalToRLLTimeStr(RlTTimeStrToBigDecimal(beginTime).add(BigDecimal.valueOf(-0.1)));
            cmdInstVOListToAdd.get(0).getCmdParamInstList().get(1).setValue(rltTime);
            String rltTime2 = BigDecimalToRLLTimeStr(RlTTimeStrToBigDecimal(endTime).add(BigDecimal.valueOf(-0.1)));
            cmdInstVOListToAdd.get(0).getCmdParamInstList().get(2).setValue(rltTime2);
        } else if ("3".equals(generateMode)) {
            //仅增不减
            //全是0003
            addCmdInstVOByRpppcCHorGT(cmdInstVOListToAdd, filterStrRpppcBodyList, cmdVOOfAdd, null, timeOffset);
        }
        return cmdInstVOListToAdd;
    }

    public static void addCmdInstVOByRpppcCHorGT(List<CmdInstVO> cmdInstVOListToAdd, List<StrRpppcBody> subRpppcList, CmdVO cmdVOOfAdd, CmdVO cmdVOOfDel, double timeOffset) {
        List<CmdParamInstVO> cmdParamInstVOListOfDel = null;
        if (cmdVOOfDel != null) {
            //TS2000+TS0004
            CmdInstVO cmdInstVOOfDel = BeanConvertUtils.map(cmdVOOfDel, CmdInstVO.class);
            cmdParamInstVOListOfDel = JacksonUtils.deserializeList(JacksonUtils.toJsonString(cmdVOOfDel.getCmdParamVOList()), CmdParamInstVO.class);
            cmdInstVOOfDel.setCmdParamInstList(cmdParamInstVOListOfDel);

            //帧序列号
            cmdParamInstVOListOfDel.get(0).setSingleParamSign(ParamSignType.V);
            cmdParamInstVOListOfDel.get(0).setValue("1");
            //程序块标识
            cmdParamInstVOListOfDel.get(1).setSingleParamSign(ParamSignType.V);
            cmdParamInstVOListOfDel.get(1).setValue("03E5H");
            //起始时间
            cmdParamInstVOListOfDel.get(2).setSingleParamSign(ParamSignType.V);
            if (cmdInstVOListToAdd.size() > 0) {
                //起始时间为其前最后一条0003的指令时间码 + 时间偏移量
                CmdInstVO last0003 = cmdInstVOListToAdd.get(cmdInstVOListToAdd.size() - 1);
                String rltTime = BigDecimalToRLLTimeStr(RlTTimeStrToBigDecimal(last0003.getCmdParamInstList().get(last0003.getCmdParamInstList().size() - 2).getValue()).add(BigDecimal.valueOf(timeOffset)));
                cmdParamInstVOListOfDel.get(2).setValue(rltTime);
            }
            //结束时间
            cmdParamInstVOListOfDel.get(3).setSingleParamSign(ParamSignType.V);
            cmdInstVOListToAdd.add(cmdInstVOOfDel);
        }

        //TS2000+TS0003
        CmdInstVO cmdInstVOOfAdd = BeanConvertUtils.map(cmdVOOfAdd, CmdInstVO.class);
        List<CmdParamInstVO> cmdParamInstVOListOfAdd = JacksonUtils.deserializeList(JacksonUtils.toJsonString(cmdInstVOOfAdd.getCmdParamInstList()), CmdParamInstVO.class);
        cmdInstVOOfAdd.setCmdParamInstList(cmdParamInstVOListOfAdd);

        //帧序列号
        cmdParamInstVOListOfAdd.get(0).setSingleParamSign(ParamSignType.V);
        cmdParamInstVOListOfAdd.get(0).setValue("1");
        //程序块标识
        cmdParamInstVOListOfAdd.get(1).setSingleParamSign(ParamSignType.V);
        cmdParamInstVOListOfAdd.get(1).setValue("03E5H");
        //指令时间码
        cmdParamInstVOListOfAdd.get(2).setSingleParamSign(ParamSignType.V);
        cmdParamInstVOListOfAdd.get(2).setValue(subRpppcList.get(0).getCEventTimeSpan());
        //指令编码
        cmdParamInstVOListOfAdd.get(3).setSingleParamSign(ParamSignType.V);
        cmdParamInstVOListOfAdd.get(3).setType(CmdParamUtil.getType(CmdParamUtil.DATABUF));
        cmdParamInstVOListOfAdd.get(3).setValue(subRpppcList.get(0).getCEventCode() + " " + subRpppcList.get(0).getCEventName());

        cmdInstVOListToAdd.add(cmdInstVOOfAdd);

        for (int i = 1; i < subRpppcList.size(); i ++) {
            //TS2000+TS0003
            CmdInstVO cmdInstVOOfAddCopy = BeanConvertUtils.map(cmdInstVOOfAdd, CmdInstVO.class);
            List<CmdParamInstVO> cmdParamInstVOListOfAddCopy = JacksonUtils.deserializeList(JacksonUtils.toJsonString(cmdInstVOOfAdd.getCmdParamInstList()), CmdParamInstVO.class);
            cmdInstVOOfAddCopy.setCmdParamInstList(cmdParamInstVOListOfAddCopy);

            //指令时间码
            cmdParamInstVOListOfAddCopy.get(2).setSingleParamSign(ParamSignType.V);
            cmdParamInstVOListOfAddCopy.get(2).setValue(subRpppcList.get(i).getCEventTimeSpan());
            //指令编码
            cmdParamInstVOListOfAddCopy.get(3).setSingleParamSign(ParamSignType.V);
            cmdParamInstVOListOfAddCopy.get(3).setType(CmdParamUtil.getType(CmdParamUtil.DATABUF));
            cmdParamInstVOListOfAddCopy.get(3).setValue(subRpppcList.get(i).getCEventCode() + " " + subRpppcList.get(i).getCEventName());

            cmdInstVOListToAdd.add(cmdInstVOOfAddCopy);
        }

        //0004的参数存在时，给结束时间赋值
        if (cmdParamInstVOListOfDel != null) {
            //结束时间为其后最后一条0003的指令时间码 + 时间偏移量
            CmdInstVO last0003 = cmdInstVOListToAdd.get(cmdInstVOListToAdd.size() - 1);
            String rltTime = BigDecimalToRLLTimeStr(RlTTimeStrToBigDecimal(last0003.getCmdParamInstList().get(last0003.getCmdParamInstList().size() - 2).getValue()).add(BigDecimal.valueOf(timeOffset)));
            cmdParamInstVOListOfDel.get(3).setValue(rltTime);
        }
    }

    public static List<CmdInstVO> handleRacpaCmd(CmdInstVO cmdInstVO, String ftpId, String taskId, String targetId) {
        List<CmdInstVO> cmdInstVOListToAdd = Lists.newArrayList();

        String racpaFileName = cmdInstVO.getCmdParamInstList().get(0).getValue();
        ParamSignType paramSignType = cmdInstVO.getCmdParamInstList().get(0).getSingleParamSign();
        if (paramSignType == ParamSignType.D || paramSignType == ParamSignType.M) {
            racpaFileName = cmdInstVO.getCmdParamInstList().get(0).getParaPath();
        }

        //落区起始时间
        String beginTime = cmdInstVO.getCmdParamInstList().get(1).getValue();
        //落区结束时间
        String endTime = cmdInstVO.getCmdParamInstList().get(2).getValue();
        //生成模式
        String generateMode = cmdInstVO.getCmdParamInstList().get(3).getValue();
        if (StringUtils.isEmpty(generateMode) && ParamSignType.D.equals(cmdInstVO.getCmdParamInstList().get(3).getSingleParamSign()) && StringUtils.isNotEmpty(cmdInstVO.getCmdParamInstList().get(3).getFix())) {
            generateMode = cmdInstVO.getCmdParamInstList().get(3).getFix();
        }

        Target target = InfoHelper.getTargetService().findOne(targetId);

        String injType = "RACPA";
        FillConfigVO fillConfigVO = CmpHelper.getFillConfigService().findWithInjTypeAndGenMode(injType, generateMode, target.getTargetCode(), cmdInstVO.getIsDc());
        if (fillConfigVO == null) {
            throw new RuntimeException("特殊比对[" + injType + "]的生成模式[" + generateMode + "]未配置");
        }

        //获取racpa文件数据对象
        String orinServerPath = getServerPath(taskId, targetId, CatalogType.ORIN, null);
        RacpaXDO racpaXDO = getFileXDO(ftpId, orinServerPath, racpaFileName, RacpaXDO.class);
        if (racpaXDO == null) {
            throw new RuntimeException(racpaFileName + "获取失败");
        }
        //根据开始时间和结束时间过滤racpa指令参数信息
        List<StruRacpaData> filterStruRacpaDataList = racpaXDO.getStruRacpaDataList().stream()
                .filter(struRacpaData -> (beginTime.compareTo(struRacpaData.getEpochReturn()) < 0
                        && endTime.compareTo(struRacpaData.getEpochReturn()) > 0)
                        || struRacpaData.getEpochReturn().equals(beginTime)
                        || struRacpaData.getEpochReturn().equals(endTime)).collect(Collectors.toList());
        int racpaDataListSize = filterStruRacpaDataList.size();

        //指令模板
        CmdVO cmdVOOfDel = CmdHelper.getCmdService().getCmdWithParamByTargetIdAndCode(targetId, fillConfigVO.getDelCmdCode());
        CmdVO cmdVOOfAdd = CmdHelper.getCmdService().getCmdWithParamByTargetIdAndCode(targetId, fillConfigVO.getAddCmdCode());

        //每帧上限
        int upperLimit = fillConfigVO.getUppperLimit();
        //时间偏移量(ms)
        double timeOffset = fillConfigVO.getTimeOffset();
        int result = 0;
        if ("1".equals(generateMode)) {
            //边删边增
            //DMS10指令单元数
            String cmdUnitNum = "2";
            // [DMS12 + DMS10 (以13个racpa指令参数信息为一组)] * 若干
            int index = 0;
            while (index + upperLimit <= racpaDataListSize) {
                StruRacpaData comp = null;
                if (index + upperLimit < racpaDataListSize) {
                    comp = filterStruRacpaDataList.get(index + upperLimit);
                }
                result = addCmdInstVOByRacpa(cmdInstVOListToAdd, filterStruRacpaDataList.subList(index, index + upperLimit), cmdVOOfAdd, cmdVOOfDel, cmdUnitNum, timeOffset, comp, 1);
                index = index + upperLimit - result;
            }
            //加入剩余不足13条的StruRacpaData
            addCmdInstVOByRacpa(cmdInstVOListToAdd, filterStruRacpaDataList.subList(index, racpaDataListSize), cmdVOOfAdd, cmdVOOfDel, cmdUnitNum, timeOffset, null, null);
            //第一条DMS12开始时间为用户填写的开始时间
            cmdInstVOListToAdd.get(0).getCmdParamInstList().get(0).setValue(beginTime);
            //最后一条DMS12的结束时间为用户填写的结束时间
            cmdInstVOListToAdd.get(cmdInstVOListToAdd.size() - 2).getCmdParamInstList().get(1).setValue(endTime);
            for (int i = 2; i < cmdInstVOListToAdd.size(); i = i + 2) {
                cmdInstVOListToAdd.get(i).getCmdParamInstList().get(0).setValue(cmdInstVOListToAdd.get(i - 2).getCmdParamInstList().get(1).getValue());
            }
        } else if ("2".equals(generateMode)) {
            //先删后增
            //DMS10指令单元数
            String cmdUnitNum = "1";
            //只有一个DMS12 其开始时间 = 用户填写的开始时间 结束时间 = 用户填写的结束时间
            //第一条DMS10的上限少一
            int firstUpperLimit = upperLimit - 1;
            //第一条DMS3的指令单元数为2
            String firstCmdUnitNum = "2";
            if (firstUpperLimit > racpaDataListSize) {
                addCmdInstVOByRacpa(cmdInstVOListToAdd, filterStruRacpaDataList, cmdVOOfAdd, cmdVOOfDel, firstCmdUnitNum, timeOffset, null, null);
            } else {
                result = addCmdInstVOByRacpa(cmdInstVOListToAdd, filterStruRacpaDataList.subList(0, firstUpperLimit), cmdVOOfAdd, cmdVOOfDel, firstCmdUnitNum, timeOffset, filterStruRacpaDataList.get(firstUpperLimit), 1);
                int index = firstUpperLimit - result;
                while (index + upperLimit <= racpaDataListSize) {
                    StruRacpaData comp = null;
                    if (index + upperLimit <= racpaDataListSize) {
                        comp = filterStruRacpaDataList.get(index + upperLimit - 1);
                    }
                    result = addCmdInstVOByRacpa(cmdInstVOListToAdd, filterStruRacpaDataList.subList(index, index + upperLimit), cmdVOOfAdd, null, cmdUnitNum, timeOffset, comp, 2);
                    index = index + upperLimit - result;
                }
                //加入剩余不足14条的StrRacpaBody
                addCmdInstVOByRacpa(cmdInstVOListToAdd, filterStruRacpaDataList.subList(index, racpaDataListSize), cmdVOOfAdd, null, cmdUnitNum, timeOffset, null, null);
            }
            //第一条DMS12 开始时间为用户填写的开始时间 结束时间为用户填写的结束时间
            cmdInstVOListToAdd.get(0).getCmdParamInstList().get(0).setValue(beginTime);
            cmdInstVOListToAdd.get(0).getCmdParamInstList().get(1).setValue(endTime);
        } else if ("3".equals(generateMode)) {
            //仅增不减
            //DMS10指令单元数
            String cmdUnitNum = "1";
            int index = 0;
            while (index + upperLimit <= racpaDataListSize) {
                StruRacpaData comp = null;
                if (index + upperLimit < racpaDataListSize) {
                    comp = filterStruRacpaDataList.get(index + upperLimit - 1);
                }
                result = addCmdInstVOByRacpa(cmdInstVOListToAdd, filterStruRacpaDataList.subList(index, index + upperLimit), cmdVOOfAdd, null, cmdUnitNum, timeOffset, comp, 2);
                index = index + upperLimit - result;
            }
            //加入剩余不足14条的StrRacpaBody
            addCmdInstVOByRacpa(cmdInstVOListToAdd, filterStruRacpaDataList.subList(index, racpaDataListSize), cmdVOOfAdd, null, cmdUnitNum, timeOffset, null, null);
        }
        return cmdInstVOListToAdd;
    }

    public static int addCmdInstVOByRacpa(List<CmdInstVO> cmdInstVOListToAdd, List<StruRacpaData> subRacaoaList, CmdVO cmdVOOfAdd, CmdVO cmdVOOfDel, String cmdUnitNum, double timeOffset, StruRacpaData comp, Integer reduceIndex) {
        if (CollectionsUtils.isEmpty(subRacaoaList)) {
            return -1;
        }
        //DMS10
        CmdInstVO cmdInstVOOfAdd = BeanConvertUtils.map(cmdVOOfAdd, CmdInstVO.class);
        List<CmdParamInstVO> cmdParamInstVOListOfAdd = JacksonUtils.deserializeList(JacksonUtils.toJsonString(cmdVOOfAdd.getCmdParamVOList()), CmdParamInstVO.class);
        cmdInstVOOfAdd.setCmdParamInstList(cmdParamInstVOListOfAdd);

        //注入序号取标称值
        cmdParamInstVOListOfAdd.get(0).setSingleParamSign(ParamSignType.V);
        cmdParamInstVOListOfAdd.get(0).setValue(cmdParamInstVOListOfAdd.get(0).getFix());
        //指令单元数
        cmdParamInstVOListOfAdd.get(1).setSingleParamSign(ParamSignType.V);
        cmdParamInstVOListOfAdd.get(1).setValue(cmdUnitNum);
        //落区个数
        cmdParamInstVOListOfAdd.get(2).setSingleParamSign(ParamSignType.A);
        cmdParamInstVOListOfAdd.get(2).setValue(null);
        //落区返回启动时间
        cmdParamInstVOListOfAdd.get(3).setSingleParamSign(ParamSignType.V);
        cmdParamInstVOListOfAdd.get(3).setValue(subRacaoaList.get(0).getEpochReturn());
        //落区自动应急返回制动速度增量
        cmdParamInstVOListOfAdd.get(4).setSingleParamSign(ParamSignType.V);
        cmdParamInstVOListOfAdd.get(4).setValue(subRacaoaList.get(0).getDv());
        //落区1制动发动机关机时刻修正量
        cmdParamInstVOListOfAdd.get(5).setSingleParamSign(ParamSignType.V);
        cmdParamInstVOListOfAdd.get(5).setValue(subRacaoaList.get(0).getDtxz());
        //落区1推返分离时刻修正量
        cmdParamInstVOListOfAdd.get(6).setSingleParamSign(ParamSignType.V);
        cmdParamInstVOListOfAdd.get(6).setValue(subRacaoaList.get(0).getDtmk());
        //落区1落点位置纬度
        cmdParamInstVOListOfAdd.get(7).setSingleParamSign(ParamSignType.V);
        cmdParamInstVOListOfAdd.get(7).setValue(subRacaoaList.get(0).getLatiLandLocus());
        //落区1落点位置经度
        cmdParamInstVOListOfAdd.get(8).setSingleParamSign(ParamSignType.V);
        cmdParamInstVOListOfAdd.get(8).setValue(subRacaoaList.get(0).getLongLandLocus());

        int result = 0;
        int size = subRacaoaList.size();
        if (comp != null && subRacaoaList.get(size - reduceIndex).getEpochReturn().equals(comp.getEpochReturn())) {
            result = reduceIndex;
            size = size - reduceIndex;
        }

        for (int i = 1; i < size; i++) {
            //落区返回启动时间
            CmdParamInstVO cmdParamInstVO = BeanConvertUtils.map(cmdParamInstVOListOfAdd.get(3), CmdParamInstVO.class);
            cmdParamInstVO.setValue(subRacaoaList.get(i).getEpochReturn());
            cmdParamInstVOListOfAdd.add(cmdParamInstVO);
            //落区自动应急返回制动速度增量
            CmdParamInstVO cmdParamInstVO1 = BeanConvertUtils.map(cmdParamInstVOListOfAdd.get(4), CmdParamInstVO.class);
            cmdParamInstVO1.setValue(subRacaoaList.get(i).getDv());
            cmdParamInstVOListOfAdd.add(cmdParamInstVO1);
            //落区1制动发动机关机时刻修正量
            CmdParamInstVO cmdParamInstVO2 = BeanConvertUtils.map(cmdParamInstVOListOfAdd.get(5), CmdParamInstVO.class);
            cmdParamInstVO2.setValue(subRacaoaList.get(i).getDtxz());
            cmdParamInstVOListOfAdd.add(cmdParamInstVO2);
            //落区1推返分离时刻修正量
            CmdParamInstVO cmdParamInstVO3 = BeanConvertUtils.map(cmdParamInstVOListOfAdd.get(6), CmdParamInstVO.class);
            cmdParamInstVO3.setValue(subRacaoaList.get(i).getDtmk());
            cmdParamInstVOListOfAdd.add(cmdParamInstVO3);
            //落区1落点位置纬度
            CmdParamInstVO cmdParamInstVO4 = BeanConvertUtils.map(cmdParamInstVOListOfAdd.get(7), CmdParamInstVO.class);
            cmdParamInstVO4.setValue(subRacaoaList.get(i).getLatiLandLocus());
            cmdParamInstVOListOfAdd.add(cmdParamInstVO4);
            //落区1落点位置经度
            CmdParamInstVO cmdParamInstVO5 = BeanConvertUtils.map(cmdParamInstVOListOfAdd.get(8), CmdParamInstVO.class);
            cmdParamInstVO5.setValue(subRacaoaList.get(i).getLongLandLocus());
            cmdParamInstVOListOfAdd.add(cmdParamInstVO5);
        }
        if (cmdVOOfDel != null) {
            //DMS12
            CmdInstVO cmdInstVOOfDel = BeanConvertUtils.map(cmdVOOfDel, CmdInstVO.class);
            List<CmdParamInstVO> cmdParamInstVOListOfDel = JacksonUtils.deserializeList(JacksonUtils.toJsonString(cmdInstVOOfDel.getCmdParamInstList()), CmdParamInstVO.class);
            cmdInstVOOfDel.setCmdParamInstList(cmdParamInstVOListOfDel);

            //去除注入序号和指令单元数
            cmdParamInstVOListOfDel.remove(0);
            cmdParamInstVOListOfDel.remove(0);

            //上一条DMS10的最后一个落区返回启动时间 + 时间偏移量
            cmdParamInstVOListOfDel.get(0).setSingleParamSign(ParamSignType.V);
            cmdParamInstVOListOfDel.get(0).setValue(cmdParamInstVOListOfAdd.get(3).getValue());
            //结束时间
            cmdParamInstVOListOfDel.get(1).setSingleParamSign(ParamSignType.V);
            //下一条DMS10的最后一个落区返回启动时间 + 时间偏移量
            String rltTime = BigDecimalToRLLTimeStr(RlTTimeStrToBigDecimal(cmdParamInstVOListOfAdd.get(cmdParamInstVOListOfAdd.size() - 6).getValue()).add(BigDecimal.valueOf(timeOffset)));
            cmdParamInstVOListOfDel.get(1).setValue(rltTime);

            cmdInstVOListToAdd.add(cmdInstVOOfDel);
        }
        cmdInstVOListToAdd.add(cmdInstVOOfAdd);
        return result;
    }

    public static List<CmdInstVO> handleRsbeodCmd(CmdInstVO cmdInstVO, String ftpId, String taskId, String targetId) {
        List<CmdInstVO> cmdInstVOListToAdd = Lists.newArrayList();

        String rsbeodFileName = cmdInstVO.getCmdParamInstList().get(0).getValue();
        ParamSignType singleParamSign = cmdInstVO.getCmdParamInstList().get(0).getSingleParamSign();
        if (singleParamSign == ParamSignType.D || singleParamSign == ParamSignType.M) {
            rsbeodFileName = cmdInstVO.getCmdParamInstList().get(0).getParaPath();
        }

        //中继指向起始时间
        String beginTime = cmdInstVO.getCmdParamInstList().get(1).getValue();
        //中继指向结束时间
        String endTime = cmdInstVO.getCmdParamInstList().get(2).getValue();
        //测站代号
        String observeName = cmdInstVO.getCmdParamInstList().get(3).getValue();

        Target target = InfoHelper.getTargetService().findOne(targetId);

        String injType = "RSBEOD";
        FillConfigVO fillConfigVO = CmpHelper.getFillConfigService().findWithInjType(injType, target.getTargetCode(), cmdInstVO.getIsDc());
        if (fillConfigVO == null) {
            throw new RuntimeException("特殊比对[" + injType + "]未配置");
        }

        //获取rsbeod文件数据对象
        String orinServerPath = getServerPath(taskId, targetId, CatalogType.ORIN, null);
        RsbeodXDO rsbeodXDO = getFileXDO(ftpId, orinServerPath, rsbeodFileName, RsbeodXDO.class);
        if (rsbeodXDO == null) {
            throw new RuntimeException(rsbeodFileName + "获取失败");
        }

        //根据测站代号和起始时间获取测站
        List<StruRsbeoData> struRsbeoDataList = null;
        String startTime = null;
        for (StruRsbeod struRsbeod : rsbeodXDO.getStruRsbeodList()) {
            if (((StringUtils.isEmpty(observeName) && StringUtils.isEmpty(struRsbeod.getObserveName())) || observeName.equals(struRsbeod.getObserveName())) && beginTime.equals(struRsbeod.getStartTime())) {
                startTime = struRsbeod.getStartTime();
                struRsbeoDataList = struRsbeod.getStruRsbeoDataList();
                break;
            }
        }
        if (CollectionsUtils.isEmpty(struRsbeoDataList)) {
            throw new RuntimeException(rsbeodFileName + "无测站[" + observeName + "],开始时间[" + beginTime + "]的数据");
        }

        //根据开始时间和结束时间过滤struRsbeodData信息
        List<StruRsbeoData> filterStruRsbeoDataList = struRsbeoDataList.stream()
                .filter(struRsbeoData -> (beginTime.compareTo(struRsbeoData.getTEpoch()) < 0
                        && endTime.compareTo(struRsbeoData.getTEpoch()) > 0)
                        || struRsbeoData.getTEpoch().equals(beginTime)
                        || struRsbeoData.getTEpoch().equals(endTime)).collect(Collectors.toList());
        int struRsbeodDataListSize = filterStruRsbeoDataList.size();

        //指令模板
        CmdVO cmdVOOfAdd = CmdHelper.getCmdService().getCmdWithParamByTargetIdAndCode(targetId, fillConfigVO.getAddCmdCode());

        //每帧上限
        int upperLimit = fillConfigVO.getUppperLimit();
        int index = 0;
        while (index + upperLimit <= struRsbeodDataListSize) {
            addCmdInstVOByRsbeod(cmdInstVOListToAdd, filterStruRsbeoDataList.subList(index, index + upperLimit), cmdVOOfAdd, startTime);
            index += upperLimit;
        }
        addCmdInstVOByRsbeod(cmdInstVOListToAdd, filterStruRsbeoDataList.subList(index, filterStruRsbeoDataList.size()), cmdVOOfAdd, startTime);
        return cmdInstVOListToAdd;
    }

    public static void addCmdInstVOByRsbeod(List<CmdInstVO> cmdInstVOListToAdd, List<StruRsbeoData> subList, CmdVO cmdVOOfAdd, String startTime) {
        //DMS10
        CmdInstVO cmdInstVOOfAdd = BeanConvertUtils.map(cmdVOOfAdd, CmdInstVO.class);
        List<CmdParamInstVO> cmdParamInstVOListOfAdd = JacksonUtils.deserializeList(JacksonUtils.toJsonString(cmdVOOfAdd.getCmdParamVOList()), CmdParamInstVO.class);
        cmdInstVOOfAdd.setCmdParamInstList(cmdParamInstVOListOfAdd);

        //注入序号 取标称值
        cmdParamInstVOListOfAdd.get(0).setSingleParamSign(ParamSignType.V);
        cmdParamInstVOListOfAdd.get(0).setValue(cmdParamInstVOListOfAdd.get(0).getFix());
        //指令单元数
        cmdParamInstVOListOfAdd.get(1).setSingleParamSign(ParamSignType.A);
        cmdParamInstVOListOfAdd.get(1).setValue(null);
        //包总数
        cmdParamInstVOListOfAdd.get(2).setSingleParamSign(ParamSignType.A);
        cmdParamInstVOListOfAdd.get(2).setValue(null);
        //包号A
        cmdParamInstVOListOfAdd.get(3).setSingleParamSign(ParamSignType.A);
        cmdParamInstVOListOfAdd.get(3).setValue(null);
        //起始时刻（北京时）
        cmdParamInstVOListOfAdd.get(4).setSingleParamSign(ParamSignType.V);
        cmdParamInstVOListOfAdd.get(4).setValue(startTime);

        int i = 5, size = cmdParamInstVOListOfAdd.size();
        for (; i < size + 1 && (i / 2) < subList.size() + 2;) {
            CmdParamInstVO cmdParamInstVO = cmdParamInstVOListOfAdd.get(i);
            cmdParamInstVO.setSingleParamSign(ParamSignType.V);
            cmdParamInstVO.setValue(subList.get(i / 2 - 2).getDObjectA());

            CmdParamInstVO cmdParamInstVO1 = cmdParamInstVOListOfAdd.get(i + 1);
            cmdParamInstVO1.setSingleParamSign(ParamSignType.V);
            cmdParamInstVO1.setValue(subList.get(i / 2 - 2).getDObjectB());

            i += 2;
        }
        while (i < cmdParamInstVOListOfAdd.size()) {
            cmdParamInstVOListOfAdd.remove(i);
        }
        cmdInstVOListToAdd.add(cmdInstVOOfAdd);
    }

    public static List<CmdInstVO> handleRprlytrkCmd(CmdInstVO cmdInstVO, String ftpId, String taskId, String targetId) {
        List<CmdInstVO> cmdInstVOListToAdd = Lists.newArrayList();

        String rprlytrkFileName = cmdInstVO.getCmdParamInstList().get(0).getValue();
        ParamSignType singleParamSign = cmdInstVO.getCmdParamInstList().get(0).getSingleParamSign();
        if (singleParamSign == ParamSignType.D || singleParamSign == ParamSignType.M) {
            rprlytrkFileName = cmdInstVO.getCmdParamInstList().get(0).getParaPath();
        }

        //起始时间
        String beginTime = cmdInstVO.getCmdParamInstList().get(1).getValue();
        //结束时间
        String endTime = cmdInstVO.getCmdParamInstList().get(2).getValue();

        Target target = InfoHelper.getTargetService().findOne(targetId);

        FillConfigVO fillConfigVO = CmpHelper.getFillConfigService().findWithInjType(cmdInstVO.getCmdCode(), target.getTargetCode(), cmdInstVO.getIsDc());
        if (fillConfigVO == null) {
            throw new RuntimeException("特殊比对[" + cmdInstVO.getCmdCode() + "]未配置");
        }

        //获取RORLYTRK文件数据对象
        String orinServerPath = getServerPath(taskId, targetId, CatalogType.ORIN, null);
        RprlytrkXDO rprlytrkXDO = getFileXDO(ftpId, orinServerPath, rprlytrkFileName, RprlytrkXDO.class);
        if (rprlytrkXDO == null) {
            throw new RuntimeException(rprlytrkFileName + "获取失败");
        }

        List<StrRlystartrkBody> filterList = Lists.newArrayList();

        //根据测站代号和起始时间获取测站
        List<StrRlystartrkBody> strRlystartrkBodyList = rprlytrkXDO.getStrRlystartrkBodyList();
        if (CollectionsUtils.isNotEmpty(strRlystartrkBodyList)) {
            for (StrRlystartrkBody strRlystartrkBody : strRlystartrkBodyList) {
                if (StringUtils.isNotEmpty(strRlystartrkBody.getCKSAFre()) && beginTime.compareTo(strRlystartrkBody.getCTimeIn()) <=0 && endTime.compareTo(strRlystartrkBody.getCTimeIn()) >= 0) {
                    filterList.add(strRlystartrkBody);
                }
            }
        }

        if (CollectionsUtils.isEmpty(filterList)) {
            throw new RuntimeException(rprlytrkFileName + "无在[" + beginTime + "," + endTime + "]时间范围内的数据");
        }

        int size = filterList.size();

        //指令模板
        CmdVO cmdVOOfAdd = CmdHelper.getCmdService().getCmdWithParamByTargetIdAndCode(targetId, fillConfigVO.getAddCmdCode());

        int upperLimit = fillConfigVO.getUppperLimit();
        int index = 0;
        while (index + upperLimit <= size) {
            addCmdInstVOByRprlytrk(cmdInstVOListToAdd, filterList.subList(index, index + upperLimit), cmdVOOfAdd);
            index += upperLimit;
        }
        addCmdInstVOByRprlytrk(cmdInstVOListToAdd, filterList.subList(index, size), cmdVOOfAdd);
        return cmdInstVOListToAdd;
    }

    public static void addCmdInstVOByRprlytrk(List<CmdInstVO> cmdInstVOListToAdd, List<StrRlystartrkBody> subList, CmdVO cmdVOOfAdd) {
        CmdInstVO cmdInstVOOfAdd = BeanConvertUtils.map(cmdVOOfAdd, CmdInstVO.class);
        List<CmdParamInstVO> cmdParamInstVOListOfAdd = JacksonUtils.deserializeList(JacksonUtils.toJsonString(cmdVOOfAdd.getCmdParamVOList()), CmdParamInstVO.class);
        cmdInstVOOfAdd.setCmdParamInstList(cmdParamInstVOListOfAdd);

        //帧序列号
        cmdParamInstVOListOfAdd.get(0).setSingleParamSign(ParamSignType.A);
        cmdParamInstVOListOfAdd.get(0).setValue(null);
        //参数有效标识
        cmdParamInstVOListOfAdd.get(1).setSingleParamSign(ParamSignType.A);
        cmdParamInstVOListOfAdd.get(1).setValue(null);
        //当前包号
        cmdParamInstVOListOfAdd.get(2).setSingleParamSign(ParamSignType.A);
        cmdParamInstVOListOfAdd.get(2).setValue(null);
        //总包数
        cmdParamInstVOListOfAdd.get(3).setSingleParamSign(ParamSignType.A);
        cmdParamInstVOListOfAdd.get(3).setValue(null);

        int i = 4, size = cmdParamInstVOListOfAdd.size();
        for (; i < size + 1 && (i / 4) < subList.size() + 1;) {
            //第i组起始时刻（北京时）
            CmdParamInstVO cmdParamInstVO = cmdParamInstVOListOfAdd.get(i);
            cmdParamInstVO.setSingleParamSign(ParamSignType.V);
            cmdParamInstVO.setValue(subList.get(i / 4 - 1).getCTimeIn());
            //跟踪中继星号
            CmdParamInstVO cmdParamInstVO1 = cmdParamInstVOListOfAdd.get(i + 1);
            cmdParamInstVO1.setSingleParamSign(ParamSignType.V);
            cmdParamInstVO1.setValue(subList.get(i / 4 - 1).getCRLYCode());
            //第i组任务状态
            CmdParamInstVO cmdParamInstVO2 = cmdParamInstVOListOfAdd.get(i + 2);
            cmdParamInstVO2.setSingleParamSign(ParamSignType.V);
            cmdParamInstVO2.setValue(subList.get(i / 4 - 1).getCKSAFre());
            //第i组结束时刻（北京时）
            CmdParamInstVO cmdParamInstVO3 = cmdParamInstVOListOfAdd.get(i + 3);
            cmdParamInstVO3.setSingleParamSign(ParamSignType.V);
            cmdParamInstVO3.setValue(subList.get(i / 4 - 1).getCTimeOut());

            i += 4;
        }
        while (i < cmdParamInstVOListOfAdd.size()) {
            cmdParamInstVOListOfAdd.remove(i);
        }
        cmdInstVOListToAdd.add(cmdInstVOOfAdd);
    }



    public static void handleRpgncCmd(CmdInstVO rpgnc, CmdInstVO cmdInstVO, String ftpId, String taskId, String targetId, String type) {
        String rpgncFileName = rpgnc.getCmdParamInstList().get(0).getValue();
        ParamSignType singleParamSign = rpgnc.getCmdParamInstList().get(0).getSingleParamSign();
        if (singleParamSign == ParamSignType.D || singleParamSign == ParamSignType.M) {
            rpgncFileName = rpgnc.getCmdParamInstList().get(0).getParaPath();
        }

        // G指令计划起始时间
        String beginTime = rpgnc.getCmdParamInstList().get(1).getValue();
        // G指令计划结束时间
        String endTime = rpgnc.getCmdParamInstList().get(2).getValue();

        // 获取rpgnc文件数据对象
        String orinServerPath = getServerPath(taskId, targetId, CatalogType.ORIN, null);
        RpgncXDO rpgncXDO = getFileXDO(ftpId, orinServerPath, rpgncFileName, RpgncXDO.class);
        if (rpgncXDO == null) {
            throw new RuntimeErrorException(rpgncFileName + "获取失败");
        }

        List<StrRpgncBody> filterStrRpngcBodyList = Lists.newArrayList();

        List<StrRpgncBody> strRpgncBodyList = rpgncXDO.getStrRpgncBodyList();
        if (CollectionsUtils.isNotEmpty(strRpgncBodyList)) {
            for (StrRpgncBody strRpgncBody : strRpgncBodyList) {
                if (beginTime.compareTo(strRpgncBody.getCEventTimeSpan()) <= 0
                        && endTime.compareTo(strRpgncBody.getCEventTimeSpan()) >= 0) {
                    filterStrRpngcBodyList.add(strRpgncBody);
                }
            }
        }

        if (CollectionsUtils.isEmpty(filterStrRpngcBodyList)) {
            throw new RuntimeErrorException(rpgncFileName + "无在[" + beginTime + "," + endTime + "]时间范围内的数据");
        }

        if ("GNCF".equals(type)) {
            addInnerCmdInstVOByRpgnc(cmdInstVO, filterStrRpngcBodyList, targetId);
        }
        if ("GNC02".equals(type)) {
            fillCmdParamByRpgnc(cmdInstVO, filterStrRpngcBodyList);
        }
    }

    public static void addInnerCmdInstVOByRpgnc(CmdInstVO GNCF, List<StrRpgncBody> filterStrRpgncBodyList, String targetId) {

        CmdParamInstVO cmdParamInstVO = GNCF.getCmdParamInstList().get(1);
        cmdParamInstVO.setSingleParamSign(ParamSignType.S);

        if (null == cmdParamInstVO.getCmdInstList()) {
            List<CmdInstVO> cmdInstVOList = com.google.common.collect.Lists.newArrayList();
            cmdParamInstVO.setCmdInstList(cmdInstVOList);
        }

        List<CmdInstVO> cmdInstVOList = com.google.common.collect.Lists.newArrayList();

        for (StrRpgncBody strRpgncBody : filterStrRpgncBodyList) {
            CmdVO cmdVO = CmdHelper.getCmdService().getCmdWithParamByTargetIdAndCode(targetId, strRpgncBody.getCEventCode());
            CmdInstVO innerCmdInstVO = BeanConvertUtils.map(cmdVO, CmdInstVO.class);
            List<CmdParamInstVO> innerCmdParamInstVOList = JacksonUtils.deserializeList(JacksonUtils.toJsonString(cmdVO.getCmdParamVOList()), CmdParamInstVO.class);
            innerCmdParamInstVOList.get(0).setSingleParamSign(ParamSignType.V);
            innerCmdParamInstVOList.get(0).setValue(strRpgncBody.getCEventTimeSpan());
            innerCmdInstVO.setCmdParamInstList(innerCmdParamInstVOList);
            cmdInstVOList.add(innerCmdInstVO);
        }
        cmdParamInstVO.setCmdInstList(cmdInstVOList);
    }

    public static void fillCmdParamByRpgnc(CmdInstVO GNC02, List<StrRpgncBody> filterStrRpgncBodyList) {
        List<CmdParamInstVO> cmdParamInstVOList = GNC02.getCmdParamInstList();

        //全部填上标称值
        cmdParamInstVOList.forEach(cmdParamInstVO -> {
            if (StringUtils.isEmpty(cmdParamInstVO.getValue())) {
                cmdParamInstVO.setSingleParamSign(ParamSignType.V);
                cmdParamInstVO.setValue(cmdParamInstVO.getFix());
            }
        });

        //第26条参数为截取文件的指令个数
        cmdParamInstVOList.get(25).setSingleParamSign(ParamSignType.V);
        cmdParamInstVOList.get(25).setValue(String.valueOf(filterStrRpgncBodyList.size()));

        //从45条参数开始填充指令时间
        int paramIndex = 44;
        for (StrRpgncBody strRpgncBody : filterStrRpgncBodyList) {
            if (paramIndex < cmdParamInstVOList.size()) {
                cmdParamInstVOList.get(paramIndex).setSingleParamSign(ParamSignType.V);
                cmdParamInstVOList.get(paramIndex).setValue(strRpgncBody.getCEventTimeSpan());
                paramIndex++;

                cmdParamInstVOList.get(paramIndex).setSingleParamSign(ParamSignType.V);
                cmdParamInstVOList.get(paramIndex).setValue(strRpgncBody.getCEventCode() + " " + strRpgncBody.getCEventName());
                paramIndex++;
            }
        }
    }

    /**
     * 处理第二圈反算比对特殊比对功能，指令参数基本固定不变
     * GF154,GF155,GF156,GF157,GF158,GF159,GF160,GD006,GF759,GF709,GF710,G114,G123,G116,G117,G119,G120,G14
     *
     * @param cmdInstVO
     * @param targetId
     */
    public static List<CmdInstVO> handleSECONDCmd(CmdInstVO cmdInstVO, String targetId, List<StCmdInfoXDO> stCmdInfoXDOList, int i) {
        List<CmdInstVO> cmdInstVOS = com.google.common.collect.Lists.newArrayList();
        CmdVO GNCF1 = CmdHelper.getCmdService().getCmdWithParamByTargetIdAndCode(targetId, "GNCF1");
        CmdInstVO gncf1CmdInstVO = CmdConvertUtils.convertCmdVoToCmdInstVo(GNCF1);
        gncf1CmdInstVO.getCmdParamInstList().get(0).setSingleParamSign(ParamSignType.S);
        cmdInstVOS.add(gncf1CmdInstVO);
        List<CmdInstVO> sCmdInstVos = com.google.common.collect.Lists.newArrayList();
        CmdVO GI017 = CmdHelper.getCmdService().getCmdWithParamByTargetIdAndCode(targetId, "GI017");
        CmdInstVO gi017 = CmdConvertUtils.convertCmdVoToCmdInstVo(GI017);
        gi017.getCmdParamInstList().get(0).setSingleParamSign(ParamSignType.C);
        gi017.getCmdParamInstList().get(0).setValue("66666666H(不允许)");
        gi017.setSort(0);
        sCmdInstVos.add(gi017);
        CmdVO GI002 = CmdHelper.getCmdService().getCmdWithParamByTargetIdAndCode(targetId, "GI002");
        CmdInstVO gi002 = CmdConvertUtils.convertCmdVoToCmdInstVo(GI002);
        gi002.getCmdParamInstList().get(0).setSingleParamSign(ParamSignType.C);
        gi002.getCmdParamInstList().get(0).setValue("ffffffffH(不校)");
        gi002.setSort(1);
        sCmdInstVos.add(gi002);
        CmdVO GI029 = CmdHelper.getCmdService().getCmdWithParamByTargetIdAndCode(targetId, "GI029");
        CmdInstVO gi029 = CmdConvertUtils.convertCmdVoToCmdInstVo(GI029);
        gi029.getCmdParamInstList().get(0).setSingleParamSign(ParamSignType.C);
        gi029.getCmdParamInstList().get(0).setValue("66666666H(19根数)");
        gi029.setSort(2);
        sCmdInstVos.add(gi029);
        CmdVO GI020 = CmdHelper.getCmdService().getCmdWithParamByTargetIdAndCode(targetId, "GI020");
        CmdInstVO gi020 = CmdConvertUtils.convertCmdVoToCmdInstVo(GI020);
        gi020.getCmdParamInstList().get(0).setSingleParamSign(ParamSignType.C);
        gi020.getCmdParamInstList().get(0).setValue("66666666H(不允许)");
        gi020.setSort(3);
        sCmdInstVos.add(gi020);
        List<CmdInstVO> secCmdInstVos = com.google.common.collect.Lists.newArrayList();
        List<CmdParamInstVO> secondCmdParamInsts = cmdInstVO.getCmdParamInstList();
        if(stCmdInfoXDOList.size() < secondCmdParamInsts.size() + i + 5){
            throw new RuntimeErrorException("反算文件格式错误！");
        }
        for (int index = 0; index < secondCmdParamInsts.size(); index++) {

            String cmdCode = stCmdInfoXDOList.get(index + i + 5).getSzZlCode();
            secCmdInstVos.add(CmdConvertUtils.convertCmdVoToCmdInstVo(CmdHelper.getCmdService().getCmdWithParamByTargetIdAndCode(targetId, cmdCode)));
        }

        for (int index = 0; index < secondCmdParamInsts.size(); index++) {
            CmdParamInstVO paramInstVO = secondCmdParamInsts.get(index);
            CmdInstVO cmdInstVO1 = secCmdInstVos.get(index);
            cmdInstVO1.setSort(index + 4);
            if (cmdInstVO1.getCmdParamInstList() != null) {
                cmdInstVO1.getCmdParamInstList().get(0).setValueRange(paramInstVO.getValueRange());
                cmdInstVO1.getCmdParamInstList().get(0).setSingleParamSign(paramInstVO.getSingleParamSign());
                cmdInstVO1.getCmdParamInstList().get(0).setFix(paramInstVO.getFix());
                cmdInstVO1.getCmdParamInstList().get(0).setValue(paramInstVO.getValue());
                cmdInstVO1.getCmdParamInstList().get(0).setFileValue(paramInstVO.getFileValue());
            }
            sCmdInstVos.add(cmdInstVO1);
        }

        gncf1CmdInstVO.getCmdParamInstList().get(0).setCmdInstList(sCmdInstVos);
        List<CmdInst> sCmdInsts = BeanConvertUtils.mapAsList(sCmdInstVos, CmdInst.class);
        String sValue = CmdParamInst.formatValueOfS(sCmdInsts);
        gncf1CmdInstVO.getCmdParamInstList().get(0).setValue(sValue);
        return cmdInstVOS;
    }
}
