package com.c503.ca.cadv.module.cluster.core.monitor;


import io.netty.bootstrap.Bootstrap;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.EventLoopGroup;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.socket.nio.NioSocketChannel;
import io.netty.handler.logging.LogLevel;
import io.netty.handler.logging.LoggingHandler;

/**
 * @author Mr.Lee
 * @date 2021/06/30 09:34
 **/
public class HeartBeatClient {

    // 远端IP
    private String remortHost;

    // 远端端口
    private int remortPort;

    private ChannelFuture future;

    public HeartBeatClient(String remortHost,int remortPort){
        this.remortHost = remortHost;
        this.remortPort = remortPort;
    }

    /**
     * @Description:  以线程的方式，启动一个心跳客户端，用于发送心跳信息
     * @Author: Mr.Lee
     * @Date: 2021-6-30 15:12
     * @Param: 无
     * @Return: 无
     */
    public void start() {
        new Thread() {
            public void run() {
                EventLoopGroup group = new NioEventLoopGroup(1);
                Bootstrap b = new Bootstrap();
                b.group(group)
                        .channel(NioSocketChannel.class)
                        //设置日志
                        .handler(new LoggingHandler(LogLevel.INFO))
                        .handler(new ChannelInitializer<SocketChannel>() {
                            @Override
                            protected void initChannel(SocketChannel sc) throws Exception {
                                //Netty中使用Marshalling
                                sc.pipeline().addLast(MarshallingCodeCFactory.buildMarshallingDecoder());
                                sc.pipeline().addLast(MarshallingCodeCFactory.buildMarshallingEncoder());
                                sc.pipeline().addLast(new HeartBeatHandler4Client());
                            }
                        });

                try {
                    future = b.connect(remortHost, remortPort).sync();
                    future.channel().closeFuture().sync();
                } catch (Exception e) {
                    // 如果链接创建失败，说明服务端不在线
                    ClusterMonitor.handleDeadNode(remortHost,remortPort);
                }finally {
                    group.shutdownGracefully();
                }
            }
        }.start();
    }

    /**
     * @Description: 客户端向服务端发送消息
     * @Author: Mr.Lee
     * @Date: 2021-7-7 21:25
     * @Param: message
     * @Return:
     */
    public void sendMessage(Object message){
        if(null != future){
            future.channel().writeAndFlush(message);
        }
    }

}
