package com.c503.ca.cadv.module.cluster.core.monitor;

import com.c503.ca.cadv.common.util.spring.SpringContextHolder;
import com.c503.ca.cadv.module.cluster.loadbalance.process.BusinessProcessor;
import com.c503.ca.cadv.module.cluster.core.model.BusinessMsg;
import com.c503.ca.cadv.module.cluster.core.model.Constants;
import com.c503.ca.cadv.module.cluster.core.model.NodeStatus;

import io.netty.channel.ChannelFutureListener;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.ChannelInboundHandlerAdapter;
import io.netty.handler.timeout.IdleStateEvent;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.net.InetSocketAddress;

/**
 * ServerHeartBeatHandler
 *
 * @author Mr.Lee
 * @date 2021/06/30 09:10
 **/
public class HeartBeatHandler4Server extends ChannelInboundHandlerAdapter {

    private static final Logger LOG = LoggerFactory.getLogger(HeartBeatHandler4Server.class);

    /*Server端需要对IP进行判断*/
    private boolean auth(ChannelHandlerContext ctx, Object msg){
        LOG.info("[集群监控服务] 服务端收到鉴权码：" + msg);
        if(null!=msg && msg.equals(Constants.AUTH_KEY)){
            ctx.writeAndFlush(Constants.SUCCESS_KEY);
            return true;
        } else {
            //短连接
            ctx.writeAndFlush("server authentication fail!")
                    .addListener(ChannelFutureListener.CLOSE);
            return false;
        }
    }

    @Override
    public void channelRead(ChannelHandlerContext ctx, Object msg) {
        if(msg instanceof String){
            auth(ctx, msg);
        } else if (msg instanceof NodeStatus) {
            NodeStatus nodeStatus = (NodeStatus) msg;//因为使用了Marshalling序列化框架，所以可以直接转换为NodeStatu对象

          //  LOG.debug("[集群监控服务] 收到Master节点心跳: node "+  nodeStatus.getNode().getIp() +" Status:" + nodeStatus.getStatus());

            if (!((NodeStatus) msg).getNode().getRole().equals("slave")) {
             //   LOG.debug("[集群监控服务] 收到Master节点心跳: node "+  nodeStatus.getNode().getIp() +" Status:" + nodeStatus.getStatus());
            }

            ctx.writeAndFlush("HeartBeat-Client：NodeStatus has received!");

        } else if(msg instanceof BusinessMsg){
            // 如果是收到客户端分发过来的组播（电文）消息，则说明负载均衡结论是需要当前节点来处理该组播（电文）消息
            // 调用组播（电文）消息处理方法
            BusinessProcessor processor = SpringContextHolder.getBean(BusinessProcessor.class);
            processor.process((BusinessMsg)msg);

        } else{
            ctx.writeAndFlush("connect failure!").addListener(ChannelFutureListener.CLOSE);
        }
    }

    /**
     * 用户事件被触发，触发某个事件之后就会被调用
     * 它会调研ChannelHandlerContext的fireUserEventTriggered(Object)方法
     * 来将事件转发给ChannelPipeline管道的下一个ChannelInboundHandler对象
     * @param ctx 上下文对象
     * @param evt 事件对象
     */
    @Override
    public void userEventTriggered(ChannelHandlerContext ctx, Object evt) {
        //如果这个事件本身是一个IdleStateEvent类型　表示他是一个空闲状态事件
        if(evt instanceof IdleStateEvent){
            //强制类型转换一下
           // IdleStateEvent event= (IdleStateEvent)evt;
			/*
			 * String eventType = null; switch (event.state()){ case READER_IDLE: eventType
			 * = "读空闲"; break; case WRITER_IDLE: // eventType = "写空闲"; break; case ALL_IDLE:
			 * // eventType = "读写空闲"; break; default: break; }
			 */
            InetSocketAddress address = (InetSocketAddress) ctx.channel().remoteAddress();
            ClusterMonitor.handleDeadNode(address.getHostName(),address.getPort());
            ctx.close();
        }
    }

	@Override
	public void channelInactive(ChannelHandlerContext ctx) throws Exception {
		super.channelInactive(ctx);
	}

}
