package com.c503.ca.cadv.module.cluster.core.monitor;


import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.ChannelFuture;
import io.netty.channel.ChannelInitializer;
import io.netty.channel.ChannelOption;
import io.netty.channel.EventLoopGroup;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import io.netty.handler.logging.LogLevel;
import io.netty.handler.logging.LoggingHandler;

/**
 * @author Mr.Lee
 * @date 2021/06/30 09:34
 **/
public class HeartBeatServer {

	 private static final Logger LOG = LoggerFactory.getLogger(HeartBeatServer.class);

    // 本地监听端口
    private int port;

    public HeartBeatServer(int port){
        this.port = port;
    }

    /**
     * @Description: 以线程的方式，启动一个心跳服务端，用于接收心跳消息
     * @Author: Mr.Lee
     * @Date: 2021-6-30 15:15
     * @Param: 无
     * @Return: 无
     */
    public void start() {
        new Thread() {
            public void run() {
                EventLoopGroup pGroup = new NioEventLoopGroup(1);
                EventLoopGroup cGroup = new NioEventLoopGroup(1);

                ServerBootstrap b = new ServerBootstrap();
                b.group(pGroup, cGroup)
                        .channel(NioServerSocketChannel.class)
                        //保持长连接
                        .childOption(ChannelOption.SO_KEEPALIVE, true)
                        .option(ChannelOption.SO_BACKLOG, 1024)
                        //设置日志
                        .handler(new LoggingHandler(LogLevel.INFO))
                        .childHandler(new ChannelInitializer<SocketChannel>() {
                            protected void initChannel(SocketChannel sc) throws Exception {
                                //Netty中使用Marshalling
                                sc.pipeline().addLast(MarshallingCodeCFactory.buildMarshallingDecoder());
                                sc.pipeline().addLast(MarshallingCodeCFactory.buildMarshallingEncoder());
                                 /*
                                    IdleStateHandler 对空闲检测提供的handler/处理器   检测的是服务器端
                                    说明：当某个Channel在一定的时间，间隔之内没有进行读、写或读和写的操作时
                                    就会出发IdleStateEvent事件

                                    IdleStateHandler(读的时间,写的时间,读写的时间)　默认秒
                                        读写的时间　读和写空闲触发一个就会触发这个事件
                                        例：IdleStateHandler(10,10,10)  如果在10秒之内都没有进行读操作，就会触发一个事件，后面两个一样
                                    IdleStateHandler(读的时间,写的时间,读和写的时间,时间单位<不指定默认是秒>)
                                 */
                                /*sc.pipeline().addLast(new IdleStateHandler(5,5,5, TimeUnit.SECONDS));*/
                                sc.pipeline().addLast(new HeartBeatHandler4Server());
                            }
                        });

                ChannelFuture future  = null;
                try {
                    future  = b.bind(port).sync();
                    future .channel().closeFuture().sync();
                    if(future.isSuccess()){
                    	LOG.debug("集群心跳检测服务：服务端启动成功！");
                    }
                } catch (InterruptedException e) {
                	LOG.debug("集群心跳检测服务：服务端启动失败,错误信息[{}]",e.getMessage());
                }
                pGroup.shutdownGracefully();
                cGroup.shutdownGracefully();

            }
        }.start();
    }

}
