package com.c503.ca.cadv.module.cluster.loadbalance.algorithm;

import org.springframework.stereotype.Service;

import com.c503.ca.cadv.module.cluster.core.model.Node;

import java.util.List;

/**
 * 负载均衡算法：轮询调度法
 *
 * 轮询算法按顺序把每个新的连接请求分配给下一个服务器，最终把所有请求平分给所有的服务器。
 * 优点：绝对公平
 * 缺点：无法根据服务器性能去分配，无法合理利用服务器资源。
 *
 * @author Mr.Lee
 * @date 2021/06/21 17:48
 **/

@Service
public class RoundRobin implements ICalculatStrategy{

    // 记录当前已经负载均衡的请求数量
    public static Integer pos = 0;

    @Override
    public Node calculate(List<Node> nodesList) {
        Node target = null;
        // 定义一个循环的值，如果大于set就从0开始
        synchronized (pos){
            if (pos >= nodesList.size()){
                pos=0;
            }
            target = nodesList.get(pos);
            //轮询+1
            pos ++;
        }
        return target;
    }
}
