package com.c503.ca.cadv.module.cluster.loadbalance.process;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.c503.ca.cadv.module.cluster.core.model.BusinessMsg;
import com.c503.ca.cadv.module.cluster.core.monitor.ClusterMonitor;

/**
 * 组播（电文）业务负载均衡处理器，处理步骤如下：
 *
 * 1.判断当前节点是否是Leader，如果当前节点不是Leader：不做任何处理；
 *
 * 2.当前节点是Leader，则执行负载均衡算法，根据负载结果判断后续操作；
 *
 * 3.如果负载到本节点：调用组播（电文）业务处理接口
 *
 * 4.如果负载到集群中其他节点：调用组播（电文）业务处理接口
 *
 *
 *
 * @author Mr.Lee
 * @date 2021/07/05 10:59
 **/

@Service
public class LoadBalanceProcessor {

	private static final Logger LOG = LoggerFactory.getLogger(LoadBalanceProcessor.class);



	@Autowired
	private BusinessProcessor businessProcessor;

	public void process(BusinessMsg businessMsg){
		// 判断当前节点是否Leader
		if(ClusterMonitor.isLocalLeader()){
			// 如果目标处理节点是本节点，直接调用本地处理接口
			LOG.info("本节点是Master节点，电文由本节点处理");
			businessProcessor.process(businessMsg);
		}else{
			// 本节点不是Leader,不进行任何操作
			LOG.info("本节点不是Master节点，不进行任何操作");
			return ;
		}

	}



}
