/*
* Copyright &copy; 2020-2021  c503-htphy All rights reserved.
*/
package com.c503.ca.cadv.module.cmp.service;


import com.c503.ca.cadv.common.mvc.QueryDetail;
import com.c503.ca.cadv.common.mvc.domain.IdAutoEntity;
import com.c503.ca.cadv.common.mvc.service.DataVOService;
import com.c503.ca.cadv.module.cmp.domain.FillConfig;
import com.c503.ca.cadv.module.cmp.domain.vo.FillConfigVO;
import com.c503.ca.cadv.module.cmp.repository.FillConfigRepository;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

@Service
@Transactional(rollbackFor = Exception.class)
public class FillConfigService extends DataVOService<FillConfigRepository, FillConfig, FillConfigVO> {



    @Override
    public boolean saveOrUpdate(FillConfigVO entity) {
        QueryDetail<FillConfig> query = createQuery();
        if (StringUtils.isEmpty(entity.getGenerateMode())) {
            entity.setGenerateMode(null);
            query.lambda().isNull(FillConfig::getGenerateMode);
        } else {
            query.lambda().eq(FillConfig::getGenerateMode, entity.getGenerateMode());
        }
        query.lambda().eq(FillConfig::getInjType, entity.getInjType())
                .eq(FillConfig::getIsDc, entity.getIsDc())
                .eq(FillConfig::getTargetCode, entity.getTargetCode());
        if (StringUtils.isNotEmpty(entity.getId())) {
            query.lambda().ne(IdAutoEntity::getId, entity.getId());
        }
        if (count(query) > 0) {
            throw new RuntimeException("保存失败，存在目标码，注入数据，生成模式，是否代传都相同的数据");
        }
        return super.saveOrUpdate(entity);
    }

    public FillConfigVO findWithInjTypeAndGenMode(String injType, String genMode, String targetCode, Integer isDc){
        return copyBeanToVO(findOne(createQuery().lambda()
                .eq(FillConfig::getInjType,injType)
                .eq(FillConfig::getGenerateMode,genMode)
                .eq(FillConfig::getIsDc, isDc)
                .eq(FillConfig::getTargetCode, targetCode)));
    }
    public  FillConfigVO findWithInjType(String injType,String targetCode, Integer isDc){
        return  copyBeanToVO(findOne(createQuery().lambda()
                .eq(FillConfig::getInjType,injType)
                .eq(FillConfig::getIsDc, isDc)
                .eq(FillConfig::getTargetCode, targetCode)));
    }


}
