package com.c503.ca.cadv.module.cmp.service;

import com.c503.ca.cadv.common.mvc.QueryDetail;
import com.c503.ca.cadv.common.mvc.domain.IdAutoEntity;
import com.c503.ca.cadv.common.mvc.service.DataVOService;
import com.c503.ca.cadv.module.cmp.domain.RevCalCmdParam;
import com.c503.ca.cadv.module.cmp.domain.vo.RevCalCmdParamVO;
import com.c503.ca.cadv.module.cmp.repository.RevCalCmdParamRepository;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.Serializable;
import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

@Service
@Transactional(rollbackFor = Exception.class)
public class RevCalCmdParamService extends DataVOService<RevCalCmdParamRepository, RevCalCmdParam, RevCalCmdParamVO> {

    @Override
    public boolean removeByIds(Collection<? extends Serializable> idList) {
        return super.physicalRemoveByIds(idList);
    }

    @Override
    public boolean saveOrUpdate(RevCalCmdParamVO entity) {
        QueryDetail<RevCalCmdParam> query = createQuery();
        query.lambda().eq(RevCalCmdParam::getCmdCode, entity.getCmdCode())
                .eq(RevCalCmdParam::getDataNo, entity.getDataNo())
                .eq(RevCalCmdParam::getTargetCode, entity.getTargetCode())
                .eq(RevCalCmdParam::getConfigId, entity.getConfigId());
        if (StringUtils.isNotEmpty(entity.getId())) {
            query.lambda().ne(IdAutoEntity::getId, entity.getId());
        }
        if (count(query) > 0) {
            throw new RuntimeException("保存失败，该指令参数已经绑定过该配置不可重复绑定");
        }
        return super.saveOrUpdate(entity);
    }

    public List<RevCalCmdParam> findByCmdParam(String cmdCode, Integer dataNo, String objectCode) {
        QueryDetail<RevCalCmdParam> query = createQuery();
        query.lambda().eq(RevCalCmdParam::getCmdCode, cmdCode)
                .eq(RevCalCmdParam::getDataNo, dataNo)
                .eq(RevCalCmdParam::getTargetCode, objectCode);
        return findAll(query);
    }

    public List<RevCalCmdParam> findByConfigIds(Collection<? extends Serializable> idList) {
        QueryDetail<RevCalCmdParam> query = createQuery();
        query.lambda().in(RevCalCmdParam::getConfigId, idList);
        return findAll(query);
    }

    public List<RevCalCmdParam> findByConfigId(String configId) {
        QueryDetail<RevCalCmdParam> query = createQuery();
        query.lambda().eq(RevCalCmdParam::getConfigId, configId);
        return findAll(query);
    }

    public List<RevCalCmdParam> findByTargetCode(String targetCode) {
        QueryDetail<RevCalCmdParam> query = createQuery();
        query.lambda().eq(RevCalCmdParam::getTargetCode, targetCode);
        return findAll(query);
    }

    public boolean physicalRemoveByTargetCode(String targetCode) {
        List<RevCalCmdParam> list = findByTargetCode(targetCode);
        if (CollectionUtils.isEmpty(list)) {
            return true;
        }
        return physicalRemoveByIds(list.stream().map(IdAutoEntity::getId).collect(Collectors.toList()));
    }

    public void bindBatch(List<RevCalCmdParamVO> list) {
        for (RevCalCmdParamVO revCalCmdParamVO : list) {
            try {
                saveOrUpdate(revCalCmdParamVO);
            } catch (Exception e) {
                throw new RuntimeException("指令代号[" + revCalCmdParamVO.getCmdCode() + "]参数序号[" + revCalCmdParamVO.getDataNo() + "]绑定失败," + e.getMessage());
            }
        }
    }

    public void delByCmdCode(String cmdCode, Integer dataNo,String targetCode) {
        QueryDetail<RevCalCmdParam> query = createQuery();
        query.lambda().eq(RevCalCmdParam::getTargetCode, targetCode)
                .eq(RevCalCmdParam::getCmdCode, cmdCode);
        if (dataNo != null) {
            query.lambda().eq(RevCalCmdParam::getDataNo, dataNo);
        }
        List<RevCalCmdParam> revCalCmdParamList = findAll(query);
        if (CollectionUtils.isNotEmpty(revCalCmdParamList)) {
            physicalRemoveByIds(revCalCmdParamList.stream().map(IdAutoEntity::getId).collect(Collectors.toList()));
        }
    }
}
