package com.c503.ca.cadv.module.cmp.service;

import com.c503.ca.cadv.common.mvc.QueryDetail;
import com.c503.ca.cadv.common.mvc.domain.IdAutoEntity;
import com.c503.ca.cadv.common.mvc.service.DataVOService;
import com.c503.ca.cadv.common.util.BeanConvertUtils;
import com.c503.ca.cadv.common.util.xml.XmlUtils;
import com.c503.ca.cadv.module.cmp.domain.RevCalCmdParam;
import com.c503.ca.cadv.module.cmp.domain.RevCalConfig;
import com.c503.ca.cadv.module.cmp.domain.vo.RevCalConfigVO;
import com.c503.ca.cadv.module.cmp.repository.RevCalConfigRepository;
import com.c503.ca.cadv.module.cmp.xml.RevCalCmdParamXml;
import com.c503.ca.cadv.module.cmp.xml.RevCalConfigXml;
import com.c503.ca.cadv.module.cmp.xml.RevCalConfigXmlList;
import com.c503.ca.cadv.module.ge.domain.CmdParamType;
import com.c503.ca.cadv.module.info.service.TargetService;
import com.c503.ca.cadv.module.sys.domain.vo.DictVO;
import com.c503.ca.cadv.module.sys.util.DictUtils;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.io.File;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

@Service
@Transactional(rollbackFor = Exception.class)
public class RevCalConfigService extends DataVOService<RevCalConfigRepository, RevCalConfig, RevCalConfigVO> {

    @Resource
    private RevCalCmdParamService revCalCmdParamService;

    @Resource
    private TargetService targetService;

    @Override
    public boolean saveOrUpdate(RevCalConfigVO entity) {
        Integer type = entity.getType();
        if (type == 0) {
            entity.setPrecision("");
            entity.setAllowableError("");
        } else if (type == 1) {
            entity.setSourceValue("");
            entity.setTargetValue("");
            entity.setAllowableError("");
        } else if (type == 2 || type == 3) {
            entity.setSourceValue("");
            entity.setTargetValue("");
            entity.setPrecision("");
        } else if (type == 4) {
            entity.setSourceValue("");
            entity.setTargetValue("");
            entity.setPrecision("");
            entity.setAllowableError("");
        }
        return super.saveOrUpdate(entity);
    }

    @Override
    public boolean removeByIds(Collection<? extends Serializable> idList) {
        List<String> revCalCmdParamIdList = revCalCmdParamService.findByConfigIds(idList).stream().map(IdAutoEntity::getId).collect(Collectors.toList());
        revCalCmdParamService.physicalRemoveByIds(revCalCmdParamIdList);
        return physicalRemoveByIds(idList);
    }

    public RevCalConfigXmlList generateXml(String targetCode) {
        RevCalConfigXmlList xmlList = new RevCalConfigXmlList();
        List<RevCalConfigXml> list = new ArrayList<>();
        xmlList.setRevCalConfigXmlList(list);
        QueryDetail<RevCalConfig> query = createQuery();
        query.lambda().eq(RevCalConfig::getTargetCode, targetCode);
        List<RevCalConfig> revCalConfigList = findAll(query);
        if (CollectionUtils.isEmpty(revCalConfigList)) {
            return null;
        }
        for (RevCalConfig revCalConfig : revCalConfigList) {
            RevCalConfigXml revCalConfigXml = BeanConvertUtils.map(revCalConfig, RevCalConfigXml.class);
            list.add(revCalConfigXml);
            List<RevCalCmdParam> revCalCmdParamList = revCalCmdParamService.findByConfigId(revCalConfig.getId());
            if (CollectionUtils.isNotEmpty(revCalCmdParamList)) {
                revCalConfigXml.setRevCalCmdParamXmlList(BeanConvertUtils.mapAsList(revCalCmdParamList, RevCalCmdParamXml.class));
            }
        }

        return xmlList;
    }

    public List<RevCalConfig> findByTargetCode(String targetCode) {
        QueryDetail<RevCalConfig> query = createQuery();
        query.lambda().eq(RevCalConfig::getTargetCode, targetCode);
        return findAll(query);
    }

    public void importFile(File cacheFile, String objectCode) {
        if (cacheFile.exists()) {
            try {
                RevCalConfigXmlList xmlList = null;
                try {
                    xmlList = (RevCalConfigXmlList) XmlUtils.fromIgnoreXML(cacheFile, RevCalConfigXmlList.class);
                } catch (Exception e) {
                    log.error("xml转换失败");
                    throw new RuntimeException("xml转换失败");
                }
                if (xmlList == null) {
                    throw new RuntimeException("导入文件为空!");
                }
                List<RevCalConfigXml> revCalConfigXmlList = xmlList.getRevCalConfigXmlList();
                if (CollectionUtils.isNotEmpty(revCalConfigXmlList)) {
                    List<RevCalConfig> revCalConfigList = new ArrayList<>();
                    List<RevCalCmdParam> revCalCmdParamList = new ArrayList<>();

                    Set<String> targetCodeSet = revCalConfigXmlList.stream().map(RevCalConfigXml::getTargetCode).collect(Collectors.toSet());
                    for (RevCalConfigXml revCalConfigXml : revCalConfigXmlList) {
                        List<RevCalCmdParamXml> revCalCmdParamXmlList = revCalConfigXml.getRevCalCmdParamXmlList();

                        RevCalConfig revCalConfig = BeanConvertUtils.map(revCalConfigXml, RevCalConfig.class);
                        revCalConfigList.add(revCalConfig);
                        Integer type = revCalConfig.getType();
                        if (type == null) {
                            throw new RuntimeException("类型[type]不能为空!");
                        }
                        if (type != 0 && type != 1 && type != 2 && type != 3 && type != 4) {
                            throw new RuntimeException("类型[type]错误!");
                        }
                        if (revCalConfig.getTargetCode() == null) {
                            throw new RuntimeException("目标码[targetCode]不能为空!");
                        }
                        if(!revCalConfigXml.getTargetCode().equals(objectCode)){
                            throw new RuntimeException("类型[targetCode]不能匹配!");
                        }
                        if (type == 0 && (StringUtils.isEmpty(revCalConfig.getSourceValue()) || StringUtils.isEmpty(revCalConfig.getTargetValue()))) {
                            throw new RuntimeException("类型[type]为0时,原值[sourceValue]和匹配值[targetValue]不能为空!");
                        }
                        if (type == 1 && StringUtils.isEmpty(revCalConfig.getPrecision())) {
                            throw new RuntimeException("类型[type]为1时,精度[precision]不能为空!");
                        }
                        if ((type == 2 || type == 3) && StringUtils.isEmpty(revCalConfig.getAllowableError())) {
                            throw new RuntimeException("类型[type]为2/3时,误差值/有效位[allowableError]不能为空!");
                        }

                        if (CollectionUtils.isNotEmpty(revCalCmdParamXmlList)) {
                            targetCodeSet.addAll(revCalCmdParamXmlList.stream().map(RevCalCmdParamXml::getTargetCode).collect(Collectors.toSet()));
                            List<RevCalCmdParam> list = BeanConvertUtils.mapAsList(revCalCmdParamXmlList, RevCalCmdParam.class);
                            revCalCmdParamList.addAll(list);
                            revCalConfig.setRevCalCmdParamList(list);
                        }

                    }
                    if (targetCodeSet.size() != 1) {
                        throw new RuntimeException("导入文件只能存在一种目标码!");
                    }
                    for (String targetCode : targetCodeSet) {
                        boolean success = physicalRemoveByTargetCode(targetCode);
                        boolean success1 = revCalCmdParamService.physicalRemoveByTargetCode(targetCode);
                        if (!success || !success1) {
                            throw new RuntimeException("删除 "+ targetCode +"反算比对规则配置数据失败!");
                        }
                    }

                    for (RevCalConfig revCalConfig : revCalConfigList) {
                        save(revCalConfig);
                    }

                    for (RevCalConfig revCalConfig : revCalConfigList) {
                        List<RevCalCmdParam> list = revCalConfig.getRevCalCmdParamList();
                        if (CollectionUtils.isNotEmpty(list)) {
                            for (RevCalCmdParam revCalCmdParam : list) {
                                revCalCmdParam.setConfigId(revCalConfig.getId());
                            }
                        }
                    }

                    revCalCmdParamService.saveOrUpdateBatch(revCalCmdParamList);
                }
            } catch (Exception e) {
                throw new RuntimeException(e.getMessage());
            }
        }
    }

    private boolean physicalRemoveByTargetCode(String targetCode) {
        List<RevCalConfig> list = findByTargetCode(targetCode);
        if (CollectionUtils.isEmpty(list)) {
            return true;
        }
        return physicalRemoveByIds(list.stream().map(IdAutoEntity::getId).collect(Collectors.toList()));
    }

    public List<CmdParamType> getCmdParamTypeList() {
        List<CmdParamType> result = new ArrayList<>();
        List<DictVO> dictList = DictUtils.getDictList("dvc_cmd_param_type");
        for (DictVO dict : dictList) {
            CmdParamType cmdParamType = new CmdParamType();
            cmdParamType.setType(Integer.parseInt(dict.getVal()));
            cmdParamType.setName(dict.getName());
            result.add(cmdParamType);
        }
        return result;
    }
}
