package com.c503.ca.cadv.module.cmp.util;

import com.c503.ca.cadv.common.util.CollectionsUtils;
import com.c503.ca.cadv.common.util.spring.SpringContextHolder;
import com.c503.ca.cadv.module.cmp.domain.RevCalCmdParam;
import com.c503.ca.cadv.module.cmp.domain.RevCalConfig;
import com.c503.ca.cadv.module.cmp.service.RevCalCmdParamService;
import com.c503.ca.cadv.module.cmp.service.RevCalConfigService;
import com.c503.ca.cadv.module.plan.domain.vo.CmdParamInstVO;
import com.c503.ca.cadv.module.plan.util.CompUtils;
import org.assertj.core.util.Lists;

import java.math.BigDecimal;
import java.util.List;

public class RevCalCompConfigUtils {
    private static volatile RevCalConfigService revCalConfigService;

    public static RevCalConfigService getRevCalConfigService() {
        if (revCalConfigService == null) {
            synchronized (RevCalConfigService.class) {
                if (revCalConfigService == null) {
                    revCalConfigService = SpringContextHolder.getBean(RevCalConfigService.class);
                }
            }
        }
        return revCalConfigService;
    }

    private static volatile RevCalCmdParamService revCalCmdParamService;

    public static RevCalCmdParamService getRevCalCmdParamService() {
        if (revCalCmdParamService == null) {
            synchronized (RevCalCmdParamService.class) {
                if (revCalCmdParamService == null) {
                    revCalCmdParamService = SpringContextHolder.getBean(RevCalCmdParamService.class);
                }
            }
        }
        return revCalCmdParamService;
    }

    public static boolean check(String value1, String value2, String objectCode, CmdParamInstVO cmdParamInstVO) {
        List<RevCalCmdParam> revCalCmdParamList = getRevCalCmdParamService().findByCmdParam(cmdParamInstVO.getCmdCode(), cmdParamInstVO.getDataNo(), objectCode);
        if (CollectionsUtils.isEmpty(revCalCmdParamList)) {
            return false;
        }
        for (RevCalCmdParam revCalCmdParam : revCalCmdParamList) {
            if (checkOne(value1, value2, revCalCmdParam)) {
                return true;
            }
        }
        return false;
    }

    public static boolean checkOne(String value1, String value2, RevCalCmdParam revCalCmdParam) {
        RevCalConfig revCalConfig = getRevCalConfigService().findOne(revCalCmdParam.getConfigId());
        if (revCalConfig == null) {
            return false;
        }
        Integer type = revCalConfig.getType();
        if (type == 0) {
            String sourceValue = revCalConfig.getSourceValue();
            String targetValue = revCalConfig.getTargetValue();
            if (!sourceValue.equals(value1) && !sourceValue.equals(value2)) {
                return false;
            }
            List<String> targetValueList = Lists.newArrayList(targetValue.split(","));
            if ((sourceValue.equals(value1) && !targetValueList.contains(value2)) || (sourceValue.equals(value2) && !targetValueList.contains(value1))) {
                return false;
            }
        } else if (type == 1){
            try {
                if ("0".equals(revCalConfig.getPrecision())) {
                    return true;
                }
                BigDecimal bigDecimal1 = new BigDecimal(value1);
                BigDecimal bigDecimal2 = new BigDecimal(value2);
                BigDecimal precision = new BigDecimal(revCalConfig.getPrecision());
                bigDecimal1 = bigDecimal1.divide(precision).setScale(0, BigDecimal.ROUND_DOWN);
                bigDecimal2 = bigDecimal2.divide(precision).setScale(0, BigDecimal.ROUND_DOWN);
                return bigDecimal1.compareTo(bigDecimal2) == 0 ? true : false;
            } catch (Exception e) {
                return false;
            }
        } else if (type == 2) {
            try {
                BigDecimal bigDecimal1 = new BigDecimal(value1);
                BigDecimal bigDecimal2 = new BigDecimal(value2);
                return bigDecimal1.subtract(bigDecimal2).abs().subtract(new BigDecimal(revCalConfig.getAllowableError())).compareTo(new BigDecimal("0")) == 1 ? false : true;
            } catch (Exception e) {
                return false;
            }
        } else if (type == 3) {
            try {
                String allowableError = revCalConfig.getAllowableError();
                BigDecimal bigDecimal = new BigDecimal(allowableError);
                //bigDecimal = bigDecimal.setScale(0, RoundingMode.HALF_UP);
                BigDecimal bigDecimal1 = new BigDecimal(value1);
                BigDecimal bigDecimal2 = new BigDecimal(value2);
                bigDecimal1 = CompUtils.validDigital1(bigDecimal1, bigDecimal.intValue());
                bigDecimal2 = CompUtils.validDigital1(bigDecimal2, bigDecimal.intValue());
                return bigDecimal1.compareTo(bigDecimal2) == 0 ? true : false;
            } catch (Exception e) {
                return false;
            }
        } else if (type == 4) {
            return true;
        } else {
            return false;
        }
        return true;
    }
}
