/*
* Copyright &copy; 2020-2021  c503-htphy All rights reserved.
*/
package com.c503.ca.cadv.module.cmp.web;

import com.c503.ca.cadv.common.rest.base.DataVOResource;
import com.c503.ca.cadv.common.util.JacksonUtils;
import com.c503.ca.cadv.common.util.domain.CustomMessage;
import com.c503.ca.cadv.common.util.domain.PageModel;
import com.c503.ca.cadv.common.util.domain.ResultBuilder;
import com.c503.ca.cadv.module.cmp.domain.vo.FillConfigVO;
import com.c503.ca.cadv.module.cmp.service.FillConfigService;
import com.c503.ca.cadv.module.config.annotation.RepeatSubmit;
import com.c503.ca.cadv.module.log.util.MyLog;
import com.c503.ca.cadv.module.log.util.OperateType;
import com.fasterxml.jackson.databind.JsonNode;
import com.google.common.collect.Lists;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

/**
* <p>
* 比对信息填充配制表 控制器
* </p>
*
* @author jintch
*/
@RestController
@RequestMapping("/cmp/fill-config")
@Api(tags = "cmp_FillConfig")
public class FillConfigResource extends DataVOResource<FillConfigService, FillConfigVO> {
    public FillConfigResource(FillConfigService service) {
        super(service);
    }

    @GetMapping(value = "/")
    @ApiOperation(value = "获取比对信息填充配制表列表", notes = "根据分页参数，获取分页列表")
    public ResponseEntity<CustomMessage<JsonNode>> getPage(PageModel pm) {
        service.findVOPage(pm);
        JsonNode rs = JacksonUtils.toJsonNodeWithDefaultMapper(pm);
        return ResultBuilder.buildObject(rs);
    }

    /**
    * 保存 比对信息填充配制表
    */
    @PostMapping(value = "/", produces = MediaType.APPLICATION_JSON_VALUE)
    @ApiOperation(value = "保存比对信息填充配制表", notes = "")
    @MyLog(value = "保存比对信息填充配制", type = "id", model = "注入数据配置")
    @RepeatSubmit
    public ResponseEntity<CustomMessage<FillConfigVO>> save(@Valid @RequestBody FillConfigVO vo) {
        try {
            boolean success = service.saveOrUpdate(vo);
            if (success) {
                return ResultBuilder.buildOk(vo, "保存成功");
            } else {
                return ResultBuilder.buildFailed("保存失败");
            }
        } catch (Exception e) {
            return ResultBuilder.buildFailed(e.getMessage());
        }
    }

    /**
    * 批量删除
    */
    @PostMapping(value = "/del")
    @ApiOperation(value = "删除实体", notes = "批量删除，ID通过逗号分隔")
    @MyLog(value = "删除比对信息填充配制", type = OperateType.DELETE, model = "注入数据配置")
    public ResponseEntity<CustomMessage<String>> delete(@RequestBody String[] ids) {
        service.removeByIds(Lists.newArrayList(ids));
        return ResultBuilder.buildOk("删除成功");
    }

}
