package com.c503.ca.cadv.module.cmp.web;

import com.c503.ca.cadv.common.rest.base.DataVOResource;
import com.c503.ca.cadv.common.util.JacksonUtils;
import com.c503.ca.cadv.common.util.domain.CustomMessage;
import com.c503.ca.cadv.common.util.domain.PageModel;
import com.c503.ca.cadv.common.util.domain.ResultBuilder;
import com.c503.ca.cadv.module.cmp.domain.vo.RevCalCmdParamVO;
import com.c503.ca.cadv.module.cmp.service.RevCalCmdParamService;
import com.c503.ca.cadv.module.config.annotation.RepeatSubmit;
import com.c503.ca.cadv.module.ge.domain.CmdAndCmdParam;
import com.c503.ca.cadv.module.ge.service.CmdParamService;
import com.c503.ca.cadv.module.log.util.MyLog;
import com.c503.ca.cadv.module.log.util.OperateType;
import com.fasterxml.jackson.databind.JsonNode;
import com.google.common.collect.Lists;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("/cmp/rev-cal-cmd-param")
@Api(tags = "cmp_RevCalCmdParam")
public class RevCalCmdParamResource extends DataVOResource<RevCalCmdParamService, RevCalCmdParamVO> {
    public RevCalCmdParamResource(RevCalCmdParamService service) {
        super(service);
    }

    @Resource
    private CmdParamService cmdParamService;

    @GetMapping(value = "/")
    @ApiOperation(value = "获取反算比对特殊配置参数绑定列表", notes = "根据分页参数，获取分页列表")
    public ResponseEntity<CustomMessage<JsonNode>> getPage(PageModel pm) {
        service.findVOPage(pm);
        List<RevCalCmdParamVO> data = pm.getData();
        if (CollectionUtils.isNotEmpty(data)) {
            List<CmdAndCmdParam> cmdParamList = cmdParamService.findParamByObjCode(data.get(0).getTargetCode());
            if (CollectionUtils.isNotEmpty(cmdParamList)) {
                Map<String, CmdAndCmdParam> map = new HashMap<>();
                for (CmdAndCmdParam cmdParam : cmdParamList) {
                    map.put(cmdParam.getCmdCode() + "-" + cmdParam.getDataNo(), cmdParam);
                }
                for (RevCalCmdParamVO revCalCmdParamVO : data) {
                    CmdAndCmdParam cmdParam = map.get(revCalCmdParamVO.getCmdCode() + "-" + revCalCmdParamVO.getDataNo());
                    if (cmdParam != null) {
                        revCalCmdParamVO.setCmdParamName(cmdParam.getExplain());
                        revCalCmdParamVO.setType(cmdParam.getType());
                    }
                };
            }
        }
        JsonNode rs = JacksonUtils.toJsonNodeWithDefaultMapper(pm);
        return ResultBuilder.buildObject(rs);
    }

    /**
     * 保存 比对信息填充配制表
     */
    @PostMapping(value = "/", produces = MediaType.APPLICATION_JSON_VALUE)
    @ApiOperation(value = "保存", notes = "")
    @MyLog(value = "保存反算比对特殊配置参数绑定", type = "id", model = "注入数据配置")
    @RepeatSubmit
    public ResponseEntity<CustomMessage<RevCalCmdParamVO>> save(@Valid @RequestBody RevCalCmdParamVO vo) {
        try {
            boolean success = service.saveOrUpdate(vo);
            if (success) {
                return ResultBuilder.buildOk(vo, "保存成功");
            } else {
                return ResultBuilder.buildFailed("保存失败");
            }
        } catch (Exception e) {
            return ResultBuilder.buildFailed(e.getMessage());
        }
    }

    /**
     * 批量删除
     */
    @PostMapping(value = "/del")
    @ApiOperation(value = "删除实体", notes = "批量删除，ID通过逗号分隔")
    @MyLog(value = "删除反算比对特殊配置参数绑定", type = OperateType.DELETE, model = "注入数据配置")
    public ResponseEntity<CustomMessage<String>> delete(@RequestBody String[] ids) {
        service.removeByIds(Lists.newArrayList(ids));
        return ResultBuilder.buildOk("删除成功");
    }

    @PostMapping(value = "/bindBatch")
    @ApiOperation(value = "批量绑定", notes = "")
    @RepeatSubmit
    public ResponseEntity<CustomMessage<String>> bindBatch(@RequestBody List<RevCalCmdParamVO> list) {
        service.bindBatch(list);
        return ResultBuilder.buildOk("保存成功");
    }
}
