package com.c503.ca.cadv.module.cmp.web;

import com.c503.ca.cadv.common.config.SystemProperties;
import com.c503.ca.cadv.common.rest.base.DataVOResource;
import com.c503.ca.cadv.common.util.FileUtils;
import com.c503.ca.cadv.common.util.IdGen;
import com.c503.ca.cadv.common.util.JacksonUtils;
import com.c503.ca.cadv.common.util.domain.CustomMessage;
import com.c503.ca.cadv.common.util.domain.PageModel;
import com.c503.ca.cadv.common.util.domain.ResultBuilder;
import com.c503.ca.cadv.common.util.xml.XmlUtils;
import com.c503.ca.cadv.module.cmp.domain.vo.RevCalConfigVO;
import com.c503.ca.cadv.module.cmp.service.RevCalConfigService;
import com.c503.ca.cadv.module.cmp.xml.RevCalConfigXmlList;
import com.c503.ca.cadv.module.config.annotation.RepeatSubmit;
import com.c503.ca.cadv.module.ge.domain.CmdParamType;
import com.c503.ca.cadv.module.log.util.MyLog;
import com.c503.ca.cadv.module.log.util.OperateType;
import com.fasterxml.jackson.databind.JsonNode;
import com.google.common.collect.Lists;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.validation.Valid;
import java.io.File;
import java.util.List;


@RestController
@RequestMapping("/cmp/rev-cal")
@Api(tags = "cmp_RevCalConfig")
public class RevCalConfigResource extends DataVOResource<RevCalConfigService, RevCalConfigVO> {
    public RevCalConfigResource(RevCalConfigService service) {
        super(service);
    }

    @GetMapping(value = "/")
    @ApiOperation(value = "获取反算比对特殊配置列表", notes = "根据分页参数，获取分页列表")
    public ResponseEntity<CustomMessage<JsonNode>> getPage(PageModel pm) {
        service.findVOPage(pm);
        JsonNode rs = JacksonUtils.toJsonNodeWithDefaultMapper(pm);
        return ResultBuilder.buildObject(rs);
    }

    /**
    * 保存 比对信息填充配制表
    */
    @PostMapping(value = "/", produces = MediaType.APPLICATION_JSON_VALUE)
    @ApiOperation(value = "保存", notes = "")
    @MyLog(value = "保存反算比对特殊配置", type = "id", model = "注入数据配置")
    @RepeatSubmit
    public ResponseEntity<CustomMessage<RevCalConfigVO>> save(@Valid @RequestBody RevCalConfigVO vo) {
        try {
            boolean success = service.saveOrUpdate(vo);
            if (success) {
                return ResultBuilder.buildOk(vo, "保存成功");
            } else {
                return ResultBuilder.buildFailed("保存失败");
            }
        } catch (Exception e) {
            return ResultBuilder.buildFailed(e.getMessage());
        }
    }

    /**
    * 批量删除
    */
    @PostMapping(value = "/del")
    @ApiOperation(value = "删除实体", notes = "批量删除，ID通过逗号分隔")
    @MyLog(value = "删除反算比对特殊配置", type = OperateType.DELETE, model = "注入数据配置")
    public ResponseEntity<CustomMessage<String>> delete(@RequestBody String[] ids) {
        service.removeByIds(Lists.newArrayList(ids));
        return ResultBuilder.buildOk("删除成功");
    }

    @GetMapping("export")
    @ApiOperation(value = "导出反算比对规则配置")
    public void export(@RequestParam("targetCode") String targetCode) {
        RevCalConfigXmlList xmlList = service.generateXml(targetCode);
        if (xmlList == null) {
            throw new RuntimeException("该目标无反算比对规则配置");
        }
        String filePath = SystemProperties.getInstance().getFileDownloadPath() + "/" + IdGen.uuid() + "/";
        String fileName = targetCode + ".xml";
        FileUtils.createFile(filePath + fileName);
        FileUtils.writeFile(filePath + fileName, XmlUtils.toIgnoreFieldXML(xmlList));
        try {
            FileUtils.downloadFile(response(), fileName, filePath + fileName);
        } catch (Exception e) {
            log.error("下载失败：{}", e.getMessage());
        } finally {
            new File(filePath + fileName).delete();
            new File(filePath).delete();
        }
    }

    @PostMapping("import")
    @ApiOperation(value = "导入反算比对规则配置")
    public ResponseEntity<CustomMessage<String>> importFile(@RequestParam MultipartFile file, @RequestParam("targetCode") String targetCode) {

        String path = SystemProperties.getInstance().getFileUploadPath() + "/" + IdGen.uuid() + "_" + file.getOriginalFilename();
        File cacheFile = null;
        try {
            boolean success = FileUtils.createFile(path);
            if (!success) {
                throw new RuntimeException("本地缓存文件创建失败!");
            }
            cacheFile = new File(path);
            FileUtils.upload(file, cacheFile);
            service.importFile(cacheFile, targetCode);
        } catch (Exception e) {
            return ResultBuilder.buildFailed("导入失败!" + e);
        } finally {

        }
        return ResultBuilder.buildOk("导入成功!");
    }


    @GetMapping(value = "/getCmdParamType")
    @ApiOperation(value = "获取参数类型", notes = "")
    public ResponseEntity<CustomMessage<List<CmdParamType>>> getCmdParamTypeList() {
        List<CmdParamType> cmdParamTypeList =  service.getCmdParamTypeList();
        return ResultBuilder.buildDataOk(cmdParamTypeList);
    }
}
