package com.c503.ca.cadv.module.config.config.mybatisplus;

import com.baomidou.mybatisplus.annotation.DbType;
import com.baomidou.mybatisplus.core.MybatisConfiguration;
import com.baomidou.mybatisplus.core.metadata.TableInfoHelper;
import com.baomidou.mybatisplus.extension.plugins.MybatisPlusInterceptor;
import com.baomidou.mybatisplus.extension.plugins.inner.DataPermissionInterceptor;
import com.baomidou.mybatisplus.extension.plugins.inner.OptimisticLockerInnerInterceptor;
import com.baomidou.mybatisplus.extension.plugins.inner.PaginationInnerInterceptor;
import com.c503.ca.cadv.common.mvc.domain.BaseEntity;
import com.c503.ca.cadv.common.mvc.domain.DataEntity;
import com.c503.ca.cadv.common.mvc.domain.IdEntity;
import com.c503.ca.cadv.common.mvc.domain.TreeEntity;
import com.c503.ca.cadv.module.sys.util.UpdateInterceptor;
import org.apache.ibatis.builder.MapperBuilderAssistant;
import org.mybatis.spring.mapper.MapperScannerConfigurer;
import org.springframework.context.EnvironmentAware;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.core.env.Environment;

/**
 * Mybatis plugin 配置
 *
 * @author liaow
 */
@Configuration
public class MyBatisPlusConfiguration implements EnvironmentAware {

    private Environment environment;

    @Override
    public void setEnvironment(Environment environment) {
        this.environment = environment;

        //注入父类字段缓存，用于Lambda查询
        MapperBuilderAssistant builderAssistant = new MapperBuilderAssistant(new MybatisConfiguration(), "");
        TableInfoHelper.initTableInfo(builderAssistant, BaseEntity.class);
        TableInfoHelper.initTableInfo(builderAssistant, DataEntity.class);
        TableInfoHelper.initTableInfo(builderAssistant, IdEntity.class);
        TableInfoHelper.initTableInfo(builderAssistant, TreeEntity.class);
    }

    /**
     * 插件
     */
    @Bean
    public MybatisPlusInterceptor mybatisPlusInterceptor() {
        MybatisPlusInterceptor interceptor = new MybatisPlusInterceptor();
        interceptor.addInnerInterceptor(new PaginationInnerInterceptor(DbType.MYSQL));
        interceptor.addInnerInterceptor(new OptimisticLockerInnerInterceptor());
        return interceptor;
    }

    @Bean
    public MapperScannerConfigurer mapperScannerConfigurer() {
        MapperScannerConfigurer scannerConfigurer = new MapperScannerConfigurer();
        scannerConfigurer.setBasePackage("com.c503.ca");
        return scannerConfigurer;
    }

    @Bean
    public UpdateInterceptor updateInterceptor() {
        return new UpdateInterceptor();
    }

}
