package com.c503.ca.cadv.module.config.mvc.handler;

import com.baomidou.mybatisplus.core.handlers.MetaObjectHandler;
import com.c503.ca.cadv.common.constant.SecurityConstants;
import com.c503.ca.cadv.common.mvc.domain.DataEntity;
import com.c503.ca.cadv.common.util.SecurityUtils;
import com.c503.ca.cadv.module.config.domain.UserDTO;
import lombok.extern.slf4j.Slf4j;
import org.apache.ibatis.reflection.MetaObject;
import org.springframework.stereotype.Component;

import java.util.Date;
import java.util.Optional;

/**
 * 自动填充
 *
 * @author liaow
 */
@Component
@Slf4j
public class EntityMetaObjectHandler implements MetaObjectHandler {

    /**
     * 自动填充插入信息
     */
    @Override
    public void insertFill(MetaObject metaObject) {
        String auditorId;
        try {
            //创建人由用户登录帐号替换为用户名称
            String createdBy = "";
            if (metaObject.getGetterType(DataEntity.F_CREATEDBY) != null && metaObject.getValue(DataEntity.F_CREATEDBY) != null) {
                createdBy = metaObject.getValue(DataEntity.F_CREATEDBY).toString();
                if ("".equals(createdBy)) {
                    createdBy = SecurityConstants.ADMIN_ID;
                }
            } else {
                createdBy = SecurityConstants.ADMIN_ID;
            }
            auditorId =  Optional.ofNullable(SecurityUtils.getUsername()).orElse(createdBy);
        } catch (Exception e) {
            auditorId = null;
        }
        metaObject.setValue(DataEntity.F_CREATEDBY, auditorId);
        Date date = new Date();
        if (metaObject.getGetterType(DataEntity.F_CREATEDDATE) != null && metaObject.getValue(DataEntity.F_CREATEDDATE) != null) {
            date = (Date) metaObject.getValue(DataEntity.F_CREATEDDATE);
        }
        metaObject.setValue(DataEntity.F_CREATEDDATE, date);
        metaObject.setValue(DataEntity.F_LASTMODIFIEDBY, auditorId);
        metaObject.setValue(DataEntity.F_LASTMODIFIEDDATE, date);
    }

    /**
     * 自动填充更新信息
     */
    @Override
    public void updateFill(MetaObject metaObject) {
        Optional<UserDTO> userDTO;
        String auditorId;
        try {
            //创建人由用户登录帐号替换为用户名称
            if (metaObject.getValue(DataEntity.F_LASTMODIFIEDBY) != null && metaObject.getValue(DataEntity.F_LASTMODIFIEDBY) != null) {
                auditorId = metaObject.getValue(DataEntity.F_LASTMODIFIEDBY).toString();
            } else {
                auditorId = Optional.ofNullable(SecurityUtils.getUsername()).orElse(SecurityConstants.ADMIN_ID);
            }
        } catch (Exception e) {
            auditorId = null;
        }
        Date date = new Date();
        metaObject.setValue(DataEntity.F_LASTMODIFIEDBY, auditorId);
        metaObject.setValue(DataEntity.F_LASTMODIFIEDDATE, date);
    }
}
