package com.c503.ca.cadv.module.config.sharding.config;

import com.c503.ca.cadv.module.info.domain.Target;
import com.c503.ca.cadv.module.info.service.TargetService;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.shardingsphere.driver.jdbc.core.datasource.ShardingSphereDataSource;
import org.apache.shardingsphere.infra.config.algorithm.AlgorithmConfiguration;
import org.apache.shardingsphere.infra.config.rule.RuleConfiguration;
import org.apache.shardingsphere.infra.metadata.database.schema.decorator.model.ShardingSphereSchema;
import org.apache.shardingsphere.infra.metadata.database.schema.decorator.model.ShardingSphereTable;
import org.apache.shardingsphere.mode.manager.ContextManager;
import org.apache.shardingsphere.sharding.api.config.ShardingRuleConfiguration;
import org.apache.shardingsphere.sharding.api.config.rule.ShardingTableRuleConfiguration;
import org.apache.shardingsphere.sharding.rule.ShardingRule;
import org.apache.shardingsphere.singletable.rule.SingleTableRule;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.sql.DataSource;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

@Component
//@EnableScheduling
//@Order(1000)
public class RefreshTarget implements InitializingBean {

    private static Logger logger = LoggerFactory.getLogger(RefreshTarget.class);

    @Resource
    private DataSource dataSource;

    @Resource
    private TargetService targetService;

    public void actualTablesRefresh(List<String> targetCodeList) throws Exception {
        //sharding数据源
        ShardingSphereDataSource shardingDataSource = (ShardingSphereDataSource) dataSource;

        // 获取context信息
        ContextManager contextManager = getContextManager(shardingDataSource);
        //获取配置的分片信息
        Collection<RuleConfiguration> configurations = contextManager.getMetaDataContexts().getMetaData().getDatabases().get("logic_db").getRuleMetaData().getConfigurations();
        for (RuleConfiguration configuration : configurations) {
            if (configuration instanceof ShardingRuleConfiguration) {
                ShardingRuleConfiguration shardingRuleConfiguration = (ShardingRuleConfiguration) configuration;
                Map<String, AlgorithmConfiguration> shardingAlgorithms = shardingRuleConfiguration.getShardingAlgorithms();
                Collection<ShardingTableRuleConfiguration> addTableConfigurations = new LinkedList<>();

                //获取node列表配置
                for (ShardingTableRuleConfiguration shardingTableRuleConfiguration : shardingRuleConfiguration.getTables()) {
                    //处理node信息，建表，自动刷新node都在此方法处理
                    ShardingTableRuleConfiguration addTableConfiguration = createTableRule(shardingTableRuleConfiguration, shardingAlgorithms, targetCodeList);
                    addTableConfigurations.add(addTableConfiguration);
                }
                ((ShardingRuleConfiguration) configuration).setTables(addTableConfigurations);
            }
        }
        Map<String, ShardingSphereSchema> schemas = contextManager.getMetaDataContexts().getMetaData().getDatabases().get("logic_db").getSchemas();
        Map<String, ShardingSphereSchema> oldSchemas = new ConcurrentHashMap<>();
        oldSchemas.putAll(schemas);
        contextManager.alterRuleConfiguration("logic_db", configurations);

        SingleTableRule rule = contextManager.getMetaDataContexts().getMetaData().getDatabases().get("logic_db").getRuleMetaData().getSingleRule(SingleTableRule.class);
        Collection<String> tables = rule.getTables();
        ShardingRule singleRule = contextManager.getMetaDataContexts().getMetaData().getDatabases().get("logic_db").getRuleMetaData().getSingleRule(ShardingRule.class);
        Collection<String> ruleTables = singleRule.getTables();
        List<String> allTables = new ArrayList<>();
        allTables.addAll(tables);
        allTables.addAll(ruleTables);
        ShardingSphereSchema oldSchema = oldSchemas.get("logic_db");
        Map<String, ShardingSphereTable> oldTables = oldSchema.getTables();
        Map<String, ShardingSphereTable> newTables = new ConcurrentHashMap<>();

        for (Map.Entry<String, ShardingSphereTable> entry : oldTables.entrySet()) {
            if (allTables.contains(entry.getKey())) {
                newTables.put(entry.getKey(), entry.getValue());
            }
        }
        ShardingSphereSchema newSchema = new ShardingSphereSchema(newTables);
        Map<String, ShardingSphereSchema> newSchemas = new ConcurrentHashMap<>();
        newSchemas.put("logic_db", newSchema);
        contextManager.getMetaDataContexts().getMetaData().getDatabases().get("logic_db").putSchema("logic_db", newSchema);
        //将新数据添加进context中
        setContextManager(shardingDataSource, contextManager);
    }

    private ShardingTableRuleConfiguration createTableRule(ShardingTableRuleConfiguration shardingTableRuleConfiguration, Map<String, AlgorithmConfiguration> shardingAlgorithms,List<String> targetCodeList) {
        AlgorithmConfiguration shardingAlgorithm = shardingAlgorithms.get(shardingTableRuleConfiguration.getTableShardingStrategy().getShardingAlgorithmName());
        // 获取当前分表使用的自定义类的全路径
        String algorithmClassName = shardingAlgorithm.getProps().getProperty("algorithmClassName");
        if (StringUtils.isBlank(algorithmClassName)) {
            return shardingTableRuleConfiguration;
        }
        // 通过反射执行‘分表自定义类’的构建物理表方法
        String logicTable = shardingTableRuleConfiguration.getLogicTable();
        String actualDataNodes = "";
        try {
            //反射拿到自定分表类，node节点信息，建表都在此类中处理，反射调用
            Class<?> aClass = Class.forName(algorithmClassName);
            Object o = aClass.newInstance();
            //构建节点信息
            Method buildNodes = aClass.getMethod("buildNodes", String.class, List.class, String.class);
            // 执行得物理表的配置结果
            actualDataNodes = (String) buildNodes.invoke(o, logicTable, targetCodeList, "cadv");
            // 执行建表逻辑
            Method createTables = aClass.getMethod("createTables", ShardingSphereDataSource.class, String.class, List.class);
            createTables.invoke(o, dataSource, logicTable, targetCodeList);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        if (StringUtils.isBlank(actualDataNodes)) {
            return shardingTableRuleConfiguration;
        }
        ShardingTableRuleConfiguration addTableConfiguration = new ShardingTableRuleConfiguration(logicTable, actualDataNodes);
        addTableConfiguration.setTableShardingStrategy(shardingTableRuleConfiguration.getTableShardingStrategy());
        addTableConfiguration.setDatabaseShardingStrategy(shardingTableRuleConfiguration.getDatabaseShardingStrategy());
        addTableConfiguration.setKeyGenerateStrategy(shardingTableRuleConfiguration.getKeyGenerateStrategy());
        return addTableConfiguration;

    }

    private ContextManager getContextManager(ShardingSphereDataSource dataSource) {
        try {
            Field contextManagerField = dataSource.getClass().getDeclaredField("contextManager");
            contextManagerField.setAccessible(true);
            return (ContextManager) contextManagerField.get(dataSource);
        } catch (Exception e) {
            throw new RuntimeException("系统异常");
        }
    }

    private void setContextManager(ShardingSphereDataSource dataSource, ContextManager manager) {
        try {
            Field contextManagerField = dataSource.getClass().getDeclaredField("contextManager");
            contextManagerField.setAccessible(true);
            contextManagerField.set(dataSource, manager);
        } catch (Exception e) {
            throw new RuntimeException("系统异常");
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
       // refresh(null, null);
    }

    public void refresh(String targetCode, List<String> removeTargetCodes) throws Exception {
        List<Target> targetList = targetService.findAll();
        if (CollectionUtils.isNotEmpty(targetList)) {
            List<String> targetCodeList = targetList.stream().map(Target::getTargetCode).collect(Collectors.toList());

            if (StringUtils.isNotEmpty(targetCode) && !targetCodeList.contains(targetCode)) {
                targetCodeList.add(targetCode);
            }

            if (CollectionUtils.isNotEmpty(removeTargetCodes)) {
                targetCodeList.remove(removeTargetCodes);
            }

            actualTablesRefresh(targetCodeList);
        }
    }



}
