package com.c503.ca.cadv.module.config.sharding.config;

import com.c503.ca.cadv.module.info.domain.Target;
import com.c503.ca.cadv.module.info.service.TargetService;
import com.zaxxer.hikari.HikariConfig;
import com.zaxxer.hikari.HikariDataSource;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.curator.shaded.com.google.common.collect.Maps;
import org.apache.shardingsphere.driver.jdbc.core.datasource.ShardingSphereDataSource;
import org.apache.shardingsphere.infra.config.algorithm.AlgorithmConfiguration;
import org.apache.shardingsphere.infra.config.rule.RuleConfiguration;
import org.apache.shardingsphere.infra.datanode.DataNode;
import org.apache.shardingsphere.infra.metadata.database.resource.ShardingSphereResource;
import org.apache.shardingsphere.infra.metadata.database.schema.decorator.model.ShardingSphereSchema;
import org.apache.shardingsphere.infra.metadata.database.schema.decorator.model.ShardingSphereTable;
import org.apache.shardingsphere.mode.manager.ContextManager;
import org.apache.shardingsphere.sharding.api.config.ShardingRuleConfiguration;
import org.apache.shardingsphere.sharding.api.config.rule.ShardingTableRuleConfiguration;
import org.apache.shardingsphere.sharding.api.config.strategy.sharding.ShardingStrategyConfiguration;
import org.apache.shardingsphere.sharding.rule.ShardingRule;
import org.apache.shardingsphere.sharding.rule.TableRule;
import org.apache.shardingsphere.singletable.config.SingleTableRuleConfiguration;
import org.apache.shardingsphere.singletable.rule.SingleTableRule;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.core.annotation.Order;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import javax.sql.DataSource;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.*;
import java.util.concurrent.ConcurrentHashMap;
import java.util.stream.Collectors;

@Component
@EnableScheduling
@Order(1000)
public class ShardingDbRuleActualDataNodesRefreshJob implements InitializingBean {

    private static Logger logger = LoggerFactory.getLogger(ShardingDbRuleActualDataNodesRefreshJob.class);

    private final String logicDb = "logic_db";

    private final String defaultDataSourceName = "calp";

    @Resource
    private ShardingDbConfiguration dbConfiguration;

    @Resource
    private DataSource dataSource;

    @Resource
    private TargetService targetService;

    public void actualDatabaseRefresh(List<String> targetCodeList){
        //sharding数据源
        ShardingSphereDataSource shardingDataSource = (ShardingSphereDataSource) dataSource;

        // 获取context信息
        ContextManager contextManager = getContextManager(shardingDataSource);
        //获取配置的分片信息
        Collection<RuleConfiguration> configurations = contextManager.getMetaDataContexts().getMetaData().getDatabase(logicDb).getRuleMetaData().getConfigurations();
        for (RuleConfiguration configuration : configurations) {
            if (configuration instanceof ShardingRuleConfiguration) {
                ShardingRuleConfiguration shardingRuleConfiguration = (ShardingRuleConfiguration) configuration;
                Map<String, AlgorithmConfiguration> shardingAlgorithms = shardingRuleConfiguration.getShardingAlgorithms();
                Collection<ShardingTableRuleConfiguration> addDatabaseConfigurations = new LinkedList<>();
                //获取node列表配置
                for (ShardingTableRuleConfiguration shardingTableRuleConfiguration : shardingRuleConfiguration.getTables()) {
                    ShardingStrategyConfiguration databaseShardingStrategy = shardingTableRuleConfiguration.getDatabaseShardingStrategy();
                    ShardingTableRuleConfiguration addTableConfiguration = null;
                    //处理node信息，建表，自动刷新node都在此方法处理
                    if (databaseShardingStrategy != null) {
                        addTableConfiguration = createDatabaseRule(shardingTableRuleConfiguration, shardingAlgorithms, targetCodeList);
                    } else {
                        addTableConfiguration = createTableRule(shardingTableRuleConfiguration, shardingAlgorithms, targetCodeList);
                    }
                    addDatabaseConfigurations.add(addTableConfiguration);
                }
                ((ShardingRuleConfiguration) configuration).setTables(addDatabaseConfigurations);
            }
        }

        SingleTableRule rule = contextManager.getMetaDataContexts().getMetaData().getDatabase(logicDb).getRuleMetaData().getSingleRule(SingleTableRule.class);
        Collection<String> tables = rule.getTables();
        ShardingRule singleRule = contextManager.getMetaDataContexts().getMetaData().getDatabase(logicDb).getRuleMetaData().getSingleRule(ShardingRule.class);
        for (RuleConfiguration configuration : configurations) {
            if (configuration instanceof ShardingRuleConfiguration) {
                ShardingRuleConfiguration shardingRuleConfiguration = (ShardingRuleConfiguration) configuration;

                //获取node列表配置
                for (ShardingTableRuleConfiguration shardingTableRuleConfiguration : shardingRuleConfiguration.getTables()) {
                    this.updateActualDataNodes(singleRule.getTableRule(shardingTableRuleConfiguration.getLogicTable()),shardingTableRuleConfiguration.getActualDataNodes());
                }
            }
            if(configuration instanceof SingleTableRuleConfiguration){
                SingleTableRuleConfiguration shardingRuleConfiguration = (SingleTableRuleConfiguration) configuration;
                shardingRuleConfiguration.setDefaultDataSource(defaultDataSourceName);
            }
        }
        Map<String, ShardingSphereSchema> schemas = contextManager.getMetaDataContexts().getMetaData().getDatabase(logicDb).getSchemas();
        Map<String, ShardingSphereSchema> oldSchemas = new ConcurrentHashMap<>();
        oldSchemas.putAll(schemas);
        contextManager.alterRuleConfiguration(logicDb, configurations);
        Collection<String> ruleTables = singleRule.getTables();
        List<String> allTables = new ArrayList<>();
        allTables.addAll(tables);
        allTables.addAll(ruleTables);
        ShardingSphereSchema oldSchema = oldSchemas.get(logicDb);
        Map<String, ShardingSphereTable> oldTables = oldSchema.getTables();
        Map<String, ShardingSphereTable> newTables = new ConcurrentHashMap<>();

        for (Map.Entry<String, ShardingSphereTable> entry : oldTables.entrySet()) {
            if (allTables.contains(entry.getKey())) {
                newTables.put(entry.getKey(), entry.getValue());
            }
        }
        ShardingSphereSchema newSchema = new ShardingSphereSchema(newTables);
        Map<String, ShardingSphereSchema> newSchemas = new ConcurrentHashMap<>();
        newSchemas.put(logicDb, newSchema);
        contextManager.getMetaDataContexts().getMetaData().getDatabases().get(logicDb).putSchema(logicDb, newSchema);
        //更新数据源配置,创建数据源,并PUT到resources中
        for (String code : targetCodeList) {
            createDataSource(contextManager.getMetaDataContexts().getMetaData().getDatabases().get(logicDb).getResource(), code);
        }
        //将新数据添加进context中
        setContextManager(shardingDataSource, contextManager);
    }

    private void updateActualDataNodes(TableRule tableRule, String actualDataNodes) {
        try {
            List<String> actualNodes = Arrays.asList(actualDataNodes.split(","));
            Field actualDataNodesField = TableRule.class.getDeclaredField("actualDataNodes");
            Field modifersField = Field.class.getDeclaredField("modifiers");
            modifersField.setAccessible(true);
            modifersField.setInt(actualDataNodesField, actualDataNodesField.getModifiers() & ~Modifier.FINAL);
            List<DataNode> dataNodes = actualNodes.stream().map(DataNode::new).collect(Collectors.toList());

            actualDataNodesField.setAccessible(true);
            actualDataNodesField.set(tableRule, dataNodes);
            Map<DataNode, Integer> dataNodeIntegerMap = new HashMap<>();
            Set<String> actualTables = new HashSet<>();
            for (int i = 0; i < dataNodes.size(); i++) {
                dataNodeIntegerMap.put(dataNodes.get(i), i);
                actualTables.add(dataNodes.get(i).getTableName());
            }
            Map<String, List<DataNode>> dataSourceNodes = dataNodes.stream().collect(Collectors.groupingBy(DataNode::getDataSourceName));

            Field actualTablesField = TableRule.class.getDeclaredField("actualTables");
            actualTablesField.setAccessible(true);
            actualTablesField.set(tableRule, actualTables);

            Field dataNodeIndexMapField = TableRule.class.getDeclaredField("dataNodeIndexMap");
            dataNodeIndexMapField.setAccessible(true);
            dataNodeIndexMapField.set(tableRule, dataNodeIntegerMap);

            Map<String, Collection<String>> dataSourceToTableMap = Maps.newHashMap();
            dataSourceNodes.forEach((ds, node) -> {
                dataSourceToTableMap.put(ds, node.stream().map(DataNode::getTableName).collect(Collectors.toSet()));
            });

            Field dataSourceToTablesMapField = TableRule.class.getDeclaredField("dataSourceToTablesMap");
            dataSourceToTablesMapField.setAccessible(true);
            dataSourceToTablesMapField.set(tableRule, dataSourceToTableMap);
        } catch (NoSuchFieldException e) {
            logger.error(e.getMessage());
        } catch (IllegalAccessException e) {
            logger.error(e.getMessage());
        }
    }

    private ShardingTableRuleConfiguration createDatabaseRule(ShardingTableRuleConfiguration shardingTableRuleConfiguration, Map<String, AlgorithmConfiguration> shardingAlgorithms, List<String> targetCodeList) {
        AlgorithmConfiguration shardingAlgorithm = shardingAlgorithms.get(shardingTableRuleConfiguration.getDatabaseShardingStrategy().getShardingAlgorithmName());
        // 获取当前分表使用的自定义类的全路径
        String algorithmClassName = shardingAlgorithm.getProps().getProperty("algorithmClassName");
        if (StringUtils.isBlank(algorithmClassName)) {
            return shardingTableRuleConfiguration;
        }
        // 通过反射执行‘分表自定义类’的构建物理表方法
        String logicTable = shardingTableRuleConfiguration.getLogicTable();
        String oriActualDataNodes = shardingTableRuleConfiguration.getActualDataNodes();
        String databaseName = oriActualDataNodes.substring(0, oriActualDataNodes.indexOf("."));
        String actualDataNodes = "";
        try {
            //反射拿到自定分表类，node节点信息，建表都在此类中处理，反射调用
            Class<?> aClass = Class.forName(algorithmClassName);
            Object o = aClass.newInstance();
            //构建节点信息
            Method buildNodes = aClass.getMethod("buildNodes", String.class, List.class, String.class);
            // 执行得物理表的配置结果
            actualDataNodes = (String) buildNodes.invoke(o, logicTable, targetCodeList, databaseName);
            // 执行建表逻辑
            Method createTables = aClass.getMethod("createDataBases", ShardingSphereDataSource.class, String.class, String.class, List.class);
            createTables.invoke(o, dataSource, databaseName, logicTable, targetCodeList);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        if (StringUtils.isBlank(actualDataNodes)) {
            return shardingTableRuleConfiguration;
        }
        ShardingTableRuleConfiguration addTableConfiguration = new ShardingTableRuleConfiguration(logicTable, actualDataNodes);
        addTableConfiguration.setTableShardingStrategy(shardingTableRuleConfiguration.getTableShardingStrategy());
        addTableConfiguration.setDatabaseShardingStrategy(shardingTableRuleConfiguration.getDatabaseShardingStrategy());
        addTableConfiguration.setKeyGenerateStrategy(shardingTableRuleConfiguration.getKeyGenerateStrategy());
        return addTableConfiguration;

    }

    private ShardingTableRuleConfiguration createTableRule(ShardingTableRuleConfiguration shardingTableRuleConfiguration, Map<String, AlgorithmConfiguration> shardingAlgorithms,List<String> targetCodeList) {
        AlgorithmConfiguration shardingAlgorithm = shardingAlgorithms.get(shardingTableRuleConfiguration.getTableShardingStrategy().getShardingAlgorithmName());
        // 获取当前分表使用的自定义类的全路径
        String algorithmClassName = shardingAlgorithm.getProps().getProperty("algorithmClassName");
        if (StringUtils.isBlank(algorithmClassName)) {
            return shardingTableRuleConfiguration;
        }
        // 通过反射执行‘分表自定义类’的构建物理表方法
        String logicTable = shardingTableRuleConfiguration.getLogicTable();
        String oriActualDataNodes = shardingTableRuleConfiguration.getActualDataNodes();
        String databaseName = oriActualDataNodes.substring(0, oriActualDataNodes.indexOf("."));
        String actualDataNodes = "";
        try {
            //反射拿到自定分表类，node节点信息，建表都在此类中处理，反射调用
            Class<?> aClass = Class.forName(algorithmClassName);
            Object o = aClass.newInstance();
            //构建节点信息
            Method buildNodes = aClass.getMethod("buildNodes", String.class, List.class, String.class);
            // 执行得物理表的配置结果
            actualDataNodes = (String) buildNodes.invoke(o, logicTable, targetCodeList, databaseName);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        if (StringUtils.isBlank(actualDataNodes)) {
            return shardingTableRuleConfiguration;
        }
        ShardingTableRuleConfiguration addTableConfiguration = new ShardingTableRuleConfiguration(logicTable, actualDataNodes);
        addTableConfiguration.setTableShardingStrategy(shardingTableRuleConfiguration.getTableShardingStrategy());
        addTableConfiguration.setDatabaseShardingStrategy(shardingTableRuleConfiguration.getDatabaseShardingStrategy());
        addTableConfiguration.setKeyGenerateStrategy(shardingTableRuleConfiguration.getKeyGenerateStrategy());
        return addTableConfiguration;

    }

    private ContextManager getContextManager(ShardingSphereDataSource dataSource) {
        try {
            Field contextManagerField = dataSource.getClass().getDeclaredField("contextManager");
            contextManagerField.setAccessible(true);
            return (ContextManager) contextManagerField.get(dataSource);
        } catch (Exception e) {
            throw new RuntimeException("系统异常");
        }
    }

    private void setContextManager(ShardingSphereDataSource dataSource, ContextManager manager) {
        try {
            Field contextManagerField = dataSource.getClass().getDeclaredField("contextManager");
            contextManagerField.setAccessible(true);
            contextManagerField.set(dataSource, manager);
        } catch (Exception e) {
            throw new RuntimeException("系统异常");
        }
    }

    @Override
    public void afterPropertiesSet() throws Exception {
        refresh(null, null);
    }

    public void refresh(String targetCode, List<String> removeTargetCodes) throws Exception {
        List<Target> targetList = targetService.findAll();
        if (CollectionUtils.isNotEmpty(targetList)) {
            List<String> targetCodeList = targetList.stream().map(Target::getTargetCode).collect(Collectors.toList());

            if (StringUtils.isNotEmpty(targetCode) && !targetCodeList.contains(targetCode)) {
                targetCodeList.add(targetCode);
            }

            if (CollectionUtils.isNotEmpty(removeTargetCodes)) {
                targetCodeList.remove(removeTargetCodes);
            }

            actualDatabaseRefresh(targetCodeList);
        }
     }

    /**
     * 新建数据源
     */
    public DataSource createDataSource(ShardingSphereResource resource, String code) {
        // 获取配置文件中的数据库配置
        HikariConfig config = dbConfiguration.newHikariConfigInstance(
                dbConfiguration.getJdbcUrl().replace(defaultDataSourceName, defaultDataSourceName + "_" + code),
                dbConfiguration.getHikari().getPoolName() + "_" + code);
        // 创建数据源
        HikariDataSource ds = new HikariDataSource(config);
        // PUT到resources中
        resource.getDataSources().put(defaultDataSourceName + "_" + code, ds);
        return ds;
    }
}
