package com.c503.ca.cadv.module.config.sharding.table;

import org.apache.commons.lang3.StringUtils;
import org.apache.shardingsphere.driver.jdbc.core.datasource.ShardingSphereDataSource;
import org.apache.shardingsphere.sharding.api.sharding.standard.PreciseShardingValue;
import org.apache.shardingsphere.sharding.api.sharding.standard.RangeShardingValue;
import org.apache.shardingsphere.sharding.api.sharding.standard.StandardShardingAlgorithm;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.sql.Connection;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.*;

/**
 * @ClassName TableTargetCodeShardingAlgorithm
 * @Description 按目标分表策略
 * @Author guoxb
 * @Date 2022/11/29
 * @Version 1.0
 **/
public class TableTargetCodeShardingAlgorithm implements StandardShardingAlgorithm<String> {

    public static Logger logger = LoggerFactory.getLogger(TableTargetCodeShardingAlgorithm.class);

    @Override
    public Collection<String> doSharding(Collection<String> tableNames, RangeShardingValue<String> rangeShardingValue) {
        return tableNames;
    }

    @Override
    public String doSharding(Collection<String> tableNames, PreciseShardingValue<String> preciseShardingValue) {
        String logicTableName = preciseShardingValue.getLogicTableName();
        String targetCode = preciseShardingValue.getValue();
        String table = logicTableName.concat("_").concat(targetCode);

        return tableNames.stream()
                .filter(tableName -> tableName.equalsIgnoreCase(table)).findFirst()
                .orElse(table);
    }

    @Override
    public Properties getProps() {
        return null;
    }

    @Override
    public void init(Properties properties) {

    }

    /**
     * 构建可用表的nodes
     *
     * @param logicTableName 源表名
     * @param targetCodeList 目标码
     * @return 物理表的集合，用，号拼接
     */
    public String buildNodes(String logicTableName, List<String> targetCodeList, String dataSourceName) {
        List<String> tableNameList = new ArrayList<>();
        for (String targetCode : targetCodeList) {
            tableNameList.add(dataSourceName + "." + logicTableName + "_" + targetCode);
        }
        return StringUtils.join(tableNameList, ",");
    }

    /**
     * 建表
     *
     * @param dataSource     数据源
     * @param logicTableName 源表名
     * @param targetCodeList 目标码
     */
    public void createTables(ShardingSphereDataSource dataSource, String logicTableName, List<String> targetCodeList) {
        Connection connection = null;
        Statement statement = null;
        try {
            connection = dataSource.getConnection();
            statement = connection.createStatement();
            for (String targetCode : targetCodeList) {
                try {
                    statement.execute("create table IF NOT EXISTS `" + logicTableName + "_" + targetCode + "`" + " like `" + logicTableName + "`");
                } catch (SQLException e) {
                    logger.error("建表失败:" + logicTableName + "_" + targetCode);
                }
            }
        } catch (Exception e) {
            logger.error("建表异常:" + e);
        } finally {
            if (statement != null) {
                try {
                    statement.close();
                } catch (SQLException e) {
                    e.printStackTrace();
                }
            }
            if (connection != null) {
                try {
                    connection.close();
                } catch (SQLException e) {
                    e.printStackTrace();
                }
            }
        }
    }
}
