package com.c503.ca.cadv.module.fm.utils;

import java.util.List;

/**
 * 数据转换工具
 *
 * @author 金天成
 * @date 2021年8月24日14:35:09
 */
public class DataConvertUtils {

    private static final String HexString = "0123456789abcdef";
    private static final char[] HexCharArr = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'};

    /**
     * 字节数组转十六进制字符串
     *
     * @param btArr 字节数组
     * @return 十六进制字符串
     */
    public static String byteArrToHex(byte[] btArr) {
        char strArr[] = new char[btArr.length * 2];
        int i = 0;
        for (byte bt : btArr) {
            strArr[i++] = HexCharArr[bt >>> 4 & 0xf];
            strArr[i++] = HexCharArr[bt & 0xf];
        }
        return new String(strArr);
    }

    private static final String[] binaryArray =
            {"0000", "0001", "0010", "0011",
                    "0100", "0101", "0110", "0111",
                    "1000", "1001", "1010", "1011",
                    "1100", "1101", "1110", "1111"};

    /**
     * 转换为二进制字符串
     */
    public static String byteArrToBinaryStr(byte[] bArray) {
        String outStr = "";
        int pos = 0;
        for (byte b : bArray) {
            //高四位
            pos = (b & 0xF0) >> 4;
            outStr += binaryArray[pos];
            //低四位
            pos = b & 0x0F;
            outStr += binaryArray[pos];
        }
        return outStr;

    }

    /**
     * 十六进制字符串转字节数组
     *
     * @param hexStr 十六进制字符串
     * @return 字节数组
     */
    public static byte[] hexToByteArr(String hexStr) {
        char[] charArr = hexStr.toLowerCase().toCharArray();
        byte[] btArr = new byte[charArr.length / 2];
        int index = 0;
        for (int i = 0; i < charArr.length; i++) {
            int highBit = HexString.indexOf(charArr[i]);
            int lowBit = HexString.indexOf(charArr[++i]);
            btArr[index] = (byte) (highBit << 4 | lowBit);
            index++;
        }
        return btArr;
    }

    public static byte[] listToByteArr(List<Integer> list) {
        byte[] bytes = new byte[list.size()];
        for (int i = 0; i < list.size(); i++) {
            bytes[i] = list.get(i).byteValue();
        }
        return bytes;
    }

    /**
     * 十六进制字符串转二进制字符串
     *
     * @param hexString 十六进制字符串
     * @return 二进制字符串
     */
    public static String hexStrToBinaryStr(String hexString) {
        if (hexString == null || "".equals(hexString)) {
            return null;
        }
        StringBuilder builder = new StringBuilder();
        for (int i = 0; i < hexString.length(); i++) {
            String indexStr = hexString.charAt(i) + "";
            String binaryStr = Integer.toBinaryString(Integer.parseInt(indexStr, 16));
            while (binaryStr.length() < 4) {
                binaryStr = "0" + binaryStr;
            }
            builder.append(binaryStr);
        }
        return builder.toString();
    }

    /**
     * 十六进制转有符号整型
     * @param hexString 十六进制字符串
     * @return 有符号整型字符串
     */
    public static String hexStrToSignedIntegerStr(String hexString) {
        return String.valueOf(Integer.valueOf(hexString, 16).shortValue());
    }

    public static String hexStrToUnsignedIntegerStr(String hexString) {
        return String.valueOf(Integer.valueOf(hexString, 16));
    }

    public static String binaryStrToHexStr(String binaryStr) {
        while(binaryStr.length() % 4 != 0) {
            binaryStr = "0" + binaryStr;
        }
        StringBuilder builder = new StringBuilder();
        for(int i = 0; i < binaryStr.length(); i = i+ 4) {
            builder.append(Integer.toHexString(Integer.parseInt(binaryStr.substring(i, i + 4), 2)));
        }
        return builder.toString();
    }

}
