package com.c503.ca.cadv.module.fm.utils;

import com.c503.ca.cadv.common.util.FileUtils;
import lombok.extern.slf4j.Slf4j;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * 读取电文的头文件
 *
 * @author btas3_mse
 *
 */
@Slf4j
public class IMSIUtils {

	/**
	 * 读取环境变量
	 */
	private final static String env = System.getenv("BTAS3");

	/**
	 * 遥控发令文件路径
	 */
	private final static String rpImsiPath = env + "/kjz/c/pub/include/ICNCMSGTYPE.h";

	/**
	 * 单目标头文件路径
	 */
	private final static String singleImsiPath = env + "/kjz/com/include/ICNIMSGTYPE_MISS.h";

	/**
	 * 多目标文件路径
	 */
	private final static String mutiImsiPath = env + "/com/include/ICNIMSGTYPE_COM.h";

	/**
	 * 电文号正则表达式
	 */
	private final static String regex = "^(#define[\\s]+)((IMSI_|IFLI_)[A-Z]+)([\\s]+)([0-9]+)(.*)";

	private static final String SPACE = " ";

	/**
	 * 读取电文号
	 * @param name
	 * @param defaultValue
	 * @return
	 */
	public static Integer readIMSI(String name,Integer defaultValue)  {
		//先读取多目标
		List<String> rpLines  = null;
		try {
			rpLines = FileUtils.readLines(new File(rpImsiPath), "utf8");
		} catch (IOException e) {
			log.error("读取文件错误:{}", e.getMessage());
		}
		Integer rpValue = regex(rpLines, regex, name);
		if(rpValue != 0) {
			return rpValue;
		}

		//先读取多目标
		List<String> mutiLines  = null;
		try {
			mutiLines = FileUtils.readLines(new File(mutiImsiPath), "utf8");
		} catch (IOException e) {
			log.error("读取文件错误:{}", e.getMessage());
		}
		Integer mutiValue = regex(mutiLines, regex, name);
		if(mutiValue != 0) {
			return mutiValue;
		}
		//后取单目标
		List<String> singleLines = null;
		try {
			singleLines = FileUtils.readLines(new File(singleImsiPath), "utf8");
		} catch (IOException e) {
			log.error("读取文件错误:{}", e.getMessage());
		}
		Integer singleValue = regex(singleLines, regex, name);
		if(singleValue != 0) {
			return singleValue;
		}
		return defaultValue;

	}

	/**
	 * 正则表达式匹配
	 * @param lines
	 * @param regex
	 * @return
	 */
	private static Integer regex(List<String> lines,String regex,String name) {

		if(lines == null || lines.isEmpty()) {
			return 0;
		}
		String valueStr = "0";
		for(String line: lines) {
			if(line.matches(regex)) {
				//去除//注释符
				line = line.replaceAll("\\//.*", "");
				String[] msgs = line.split(SPACE);
				if(msgs == null || msgs.length < 1) {
					continue;
				}
				List<String> values = new ArrayList<String>();
				//取第三个字符
				for(String str:Arrays.asList(msgs)) {
					if(!"".equals(str)) {
						values.add(str);
					}
				}
				if(name.equals(values.get(1))) {
					valueStr = values.get(2);
					break;
				}
			}
		}

		try {
			return Integer.parseInt(valueStr);
		} catch (Exception e) {
			return 0;
		}
	}

}
