package com.c503.ca.cadv.module.ge.domain.enums;

import com.baomidou.mybatisplus.annotation.IEnum;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonValue;
import com.google.common.collect.Maps;

import java.util.Map;

/**
 * 单个参数标识
 * @author jintch
 * @since 2021/2/20
 */

public enum ParamSignType implements IEnum<String> {

    V("V", "V:参数值"),
    D("D", "D:数据库标称值"),
    B("B", "B:读二进制文件"),
    F("F", "F:指定文件名"),
    T("T", "T:参数填表"),
    W("W", "W:读文本文件"),
    M("M", "M:读默认文件"),
    C("C", "C:指令名"),
    S("S", "S:S号"),
    A("A", "A:帧序列号自增");

    private static final Map<String, ParamSignType> enumMap = Maps.newHashMap();

    @JsonValue
    private final String value;

    private final String desc;

    ParamSignType(String value, String desc) {
        this.value = value;
        this.desc = desc;
    }

    public String getDesc() {
        return desc;
    }

    static {
        for (ParamSignType enumObj : values()) {
            enumMap.put(enumObj.getValue(), enumObj);
        }
    }

    public static ParamSignType valueOf(int value) {
        ParamSignType obj = enumMap.get(value);
        if (obj == null) {
            for (ParamSignType enumObj : values()) {
                enumMap.put(enumObj.getValue(), enumObj);
            }
            obj = enumMap.get(value);
        }
        return obj;
    }

    @JsonCreator(mode = JsonCreator.Mode.DELEGATING)
    public static ParamSignType forValue(String value) {
        ParamSignType result;
        try {
            result = valueOf(Integer.parseInt(value));
        } catch (NumberFormatException e) {
            result = forName(value);
        }
        return result;
    }

    public static ParamSignType forName(String enumName) {
        for (ParamSignType enumObj : values()) {
            if (enumObj.name().equals(enumName)) {
                return enumObj;
            }
        }
        return null;
    }

    @Override
    public String toString() {
        return this.getDesc();
    }

    @Override
    public String getValue() {
        //存入数据库内容
        return value;
    }

    public static boolean check(String value) {
        ParamSignType paramSignType = forValue(value);
        if (paramSignType == null) {
            return false;
        }
        return true;
    }
}
