/**
 * Copyright &copy; 2020-2021  c503-htphy All rights reserved.
 */
package com.c503.ca.cadv.module.ge.service;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.c503.ca.cadv.common.config.SystemProperties;
import com.c503.ca.cadv.common.mvc.PageQuery;
import com.c503.ca.cadv.common.mvc.QueryDetail;
import com.c503.ca.cadv.common.mvc.service.DataVOService;
import com.c503.ca.cadv.common.util.*;
import com.c503.ca.cadv.common.util.domain.PageModel;
import com.c503.ca.cadv.common.util.domain.QueryCondition;
import com.c503.ca.cadv.common.util.xml.XmlUtils;
import com.c503.ca.cadv.enhance.websocket.domain.CommonMessageType;
import com.c503.ca.cadv.enhance.websocket.domain.WebSocketMessage;
import com.c503.ca.cadv.enhance.websocket.util.WebSocketUtils;
import com.c503.ca.cadv.module.config.domain.UserDTO;
import com.c503.ca.cadv.module.ge.domain.Cmd;
import com.c503.ca.cadv.module.ge.domain.vo.CmdParamVO;
import com.c503.ca.cadv.module.ge.domain.vo.CmdVO;
import com.c503.ca.cadv.module.ge.domain.xdo.CmdParamXDO;
import com.c503.ca.cadv.module.ge.domain.xdo.CmdXDO;
import com.c503.ca.cadv.module.ge.domain.xdo.RootXDO;
import com.c503.ca.cadv.module.ge.repository.CmdRepository;
import com.c503.ca.cadv.module.info.domain.Target;
import com.c503.ca.cadv.module.info.service.TargetService;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import com.google.common.collect.Sets;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.Serializable;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.Set;
import java.util.concurrent.TimeUnit;

/**
 * <p>
 * 通用指令表 服务类
 * </p>
 *
 * @author jintch
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class CmdService extends DataVOService<CmdRepository, Cmd, CmdVO> {

    @Autowired
    private CmdParamService cmdParamService;

    @Autowired
    private TargetService targetService;

    @Override
    public CmdVO findOneVO(Serializable id) {
        CmdVO cmdVO = super.findOneVO(id);
        setCmdParamList(cmdVO);
        return cmdVO;
    }

    public void setCmdParamList(CmdVO cmdVO) {
        if (cmdVO == null) {
            return;
        }

        List<CmdParamVO> cmdParamVOList =  cmdParamService.findAllByCmdId(cmdVO.getId());
        cmdVO.setCmdParamVOList(cmdParamVOList);
    }


    public CmdVO getCmdWithParamByTargetIdAndCode(String targetId, String cmdCode) {
        Target target = targetService.findOne(targetId);
        return target == null ? null : getCmdWithParamByTargetAndCode(target.getTargetCode(), cmdCode);
    }

    public CmdVO getCmdWithParamByTargetAndCode(String objectCode, String cmdCode) {
        Cmd cmd = findOne(createQuery().lambda().eq(Cmd::getObjectCode, objectCode).eq(Cmd::getCmdCode, cmdCode));
        CmdVO cmdVO = copyBeanToVO(cmd);
        setCmdParamList(cmdVO);
        return cmdVO;
    }
}
