package com.c503.ca.cadv.module.log.util.interceptor;

import com.alibaba.fastjson.JSONArray;
import com.c503.ca.cadv.common.rest.handler.HttpErrorHandler;
import com.c503.ca.cadv.common.util.JacksonUtils;
import com.c503.ca.cadv.common.util.domain.RequestMethod;
import com.c503.ca.cadv.module.log.util.Logs;
import com.c503.ca.cadv.module.log.util.MyLog;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.core.NamedThreadLocal;
import org.springframework.util.AntPathMatcher;
import org.springframework.web.method.HandlerMethod;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.ModelAndView;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.Method;
import java.util.List;

/**
 * @author liaow
 */
public class OperateLogInterceptor implements HandlerInterceptor {

    private static final ThreadLocal<Long> START_TIME_THREAD_LOCAL = new NamedThreadLocal<Long>("StartTime");

    protected Logger logger = LoggerFactory.getLogger(getClass());

    private final AntPathMatcher pathMatcher = new AntPathMatcher();

    public static final ThreadLocal<List<Object>> PARAMS = new NamedThreadLocal<>("Params");

    public OperateLogInterceptor() {
    }

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response,
                             Object handler) throws Exception {

        //开始时间
        long beginTime = System.currentTimeMillis();
        START_TIME_THREAD_LOCAL.set(beginTime);
        return true;
    }

    @Override
    public void postHandle(HttpServletRequest request, HttpServletResponse response, Object handler,
                           ModelAndView modelAndView) throws Exception {
    }

    @Override
    public void afterCompletion(HttpServletRequest request, HttpServletResponse response,
                                Object handler, Exception ex) throws Exception {

        final String requestURI = request.getRequestURI();
        if (HttpErrorHandler.PATH.equals(requestURI)) {
            return;
        }

        String method = request.getMethod();
        //不记录get请求
        if (RequestMethod.GET.toString().equals(method)) {
            return;
        }
        // 不记录 ws 请求
        if (pathMatcher.match("**/ws/**/**/xhr_streaming", request.getRequestURL().toString())) {
            return;
        }
        HandlerMethod handlerMethod=(HandlerMethod) handler;
        Method logMethod = handlerMethod.getMethod();
        MyLog myLog= logMethod.getAnnotation(MyLog.class);
        long beginTime = START_TIME_THREAD_LOCAL.get();
        //结束时间
        long endTime = System.currentTimeMillis();

        if (myLog != null) {
            // 保存操作日志
            List<Object> objects = PARAMS.get();
            String paramsStr = JacksonUtils.toJsonString(objects);
            JSONArray jsonArray = (JSONArray) JSONArray.parse(paramsStr);
            Logs.saveOperate(request, paramsStr, beginTime, endTime, myLog, jsonArray);
        }

        START_TIME_THREAD_LOCAL.remove();
        PARAMS.remove();
    }
}
