/**
 * Copyright &copy; 2020-2021  c503-phy All rights reserved.
 */
package com.c503.ca.cadv.module.log.web;

import com.c503.ca.cadv.common.rest.base.DataVOResource;
import com.c503.ca.cadv.common.util.JacksonUtils;
import com.c503.ca.cadv.common.util.domain.CustomMessage;
import com.c503.ca.cadv.common.util.domain.PageModel;
import com.c503.ca.cadv.common.util.domain.ResultBuilder;
import com.c503.ca.cadv.module.config.annotation.RepeatSubmit;
import com.c503.ca.cadv.module.log.domain.vo.LogOperateVO;
import com.c503.ca.cadv.module.log.service.LogOperateService;
import com.c503.ca.cadv.module.log.util.MyLog;
import com.c503.ca.cadv.module.log.util.OperateType;
import com.fasterxml.jackson.databind.JsonNode;
import com.google.common.collect.Lists;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

/**
 * <p>
 * 用户操作日志控制器
 * </p>
 *
 * @author steveshang
 */
@RestController
@RequestMapping("/log/log-operate")
@Api(tags = "log_LogOperate")
public class LogOperateResource extends DataVOResource<LogOperateService, LogOperateVO> {
    public LogOperateResource(LogOperateService service) {
        super(service);
    }

    @GetMapping(value = "/")
    @ApiOperation(value = "获取列表", notes = "根据分页参数，获取分页列表")
    public ResponseEntity<CustomMessage<JsonNode>> getPage(PageModel pm) {
        service.findVOPage(pm);
        JsonNode rs = JacksonUtils.toJsonNodeWithDefaultMapper(pm);
        return ResultBuilder.buildObject(rs);
    }

    /**
     * 保存
     */
    @PostMapping(value = "/", produces = MediaType.APPLICATION_JSON_VALUE)
    @ApiOperation(value = "保存", notes = "")
    @RepeatSubmit
    public ResponseEntity<CustomMessage<LogOperateVO>> save(@Valid @RequestBody LogOperateVO vo) {
        service.saveOrUpdate(vo);
        return ResultBuilder.buildOk(vo, "保存成功");
    }

    /**
     * 批量删除
     */
    @PostMapping(value = "/del")
    @ApiOperation(value = "删除实体", notes = "批量删除，ID通过逗号分隔")
    @MyLog(value = "删除系统日志", type = OperateType.DELETE, model = "系统日志")
    public ResponseEntity<CustomMessage<String>> delete(@RequestBody String[] ids) {
        service.physicalRemoveByIds(Lists.newArrayList(ids));
        return ResultBuilder.buildOk("删除成功");
    }

}
