package com.c503.ca.cadv.module.plan.service;

import com.c503.ca.cadv.common.mvc.QueryDetail;
import com.c503.ca.cadv.common.mvc.domain.IdAutoEntity;
import com.c503.ca.cadv.common.mvc.service.DataVOService;
import com.c503.ca.cadv.module.plan.domain.CmdInst;
import com.c503.ca.cadv.module.plan.domain.vo.CmdInstVO;
import com.c503.ca.cadv.module.plan.domain.vo.CmdParamInstVO;
import com.c503.ca.cadv.module.plan.repository.CmdInstRepository;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.compress.utils.Lists;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author huyouxiang
 * @date 2023/12/13 14:02
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class CmdInstService extends DataVOService<CmdInstRepository, CmdInst, CmdInstVO> {

    @Resource
    private CmdParamInstService cmdParamInstService;


    public List<CmdInstVO> readByFileDataId(String fileDataId) {
        List<CmdInstVO> cmdInstVOList = findByFileDataId(fileDataId);
        if (cmdInstVOList != null && cmdInstVOList.size() > 0) {
            for (CmdInstVO cmdInstVO : cmdInstVOList) {
                List<CmdParamInstVO> cmdParamInstVOList = cmdParamInstService.read(cmdInstVO.getId());
                cmdInstVO.setCmdParamInstList(cmdParamInstVOList);
            }
        } else {
            cmdInstVOList = Lists.newArrayList();
        }
        return cmdInstVOList;
    }

    public List<CmdInstVO> findByFileDataId(String fileDataId) {
        return findAllVO(createQuery().lambda().eq(CmdInst::getFileDataId, fileDataId).orderByAsc(CmdInst::getSort));
    }

    public List<CmdInstVO> readByCmdParamInstId(String cmdParamInstId) {
        List<CmdInstVO> cmdInstVOList = findByCmdParamInstId(cmdParamInstId);
        if (cmdInstVOList != null) {
            for (CmdInstVO cmdInstVO : cmdInstVOList) {
                List<CmdParamInstVO> cmdParamInstVOList = cmdParamInstService.read(cmdInstVO.getId());
                cmdInstVO.setCmdParamInstList(cmdParamInstVOList);
            }
        } else {
            cmdInstVOList = Lists.newArrayList();
        }
        return cmdInstVOList;
    }

    public List<CmdInstVO> findByCmdParamInstId(String cmdParamInstId) {
        return findAllVO(createQuery().lambda().eq(CmdInst::getCmdParamInstId, cmdParamInstId).orderByAsc(CmdInst::getSort));
    }

    private static final Integer DEL_SUB_SIZE = 1000;

    public void physicalDelCmdInst(List<String> fileDataIdList) {
        List<String> cmdInstIdList = queryIdList(fileDataIdList);
        if (CollectionUtils.isNotEmpty(cmdInstIdList)) {
            int size = cmdInstIdList.size();
            List<String> subCmdInstIdList;
            for (int i = 0; i < size; i = i + DEL_SUB_SIZE) {
                subCmdInstIdList = cmdInstIdList.subList(i, Math.min(i + DEL_SUB_SIZE, size));
                cmdParamInstService.physicalDelCmdParamInst(subCmdInstIdList);
                physicalRemoveByIds(subCmdInstIdList);
            }
        }
    }

    public List<String> queryIdList(List<String> fileDataIdList) {
        QueryDetail<CmdInst> query = createQuery();
        query.lambda().in(CmdInst::getFileDataId, fileDataIdList);
        List<CmdInst> cmdInstList = findAll(query);
        if (CollectionUtils.isEmpty(cmdInstList)) {
            return null;
        }
        return cmdInstList.stream().map(IdAutoEntity::getId).collect(Collectors.toList());
    }

    public void physicalDelInnerCmdInst(List<String> cmdParamInstIdList) {
        List<String> innerCmdInstIdList = queryInnerIdList(cmdParamInstIdList);
        if (CollectionUtils.isNotEmpty(innerCmdInstIdList)) {
            List<String> innerCmdParamInstIdList = cmdParamInstService.queryIdList(innerCmdInstIdList);
            if (CollectionUtils.isNotEmpty(innerCmdParamInstIdList)) {
                cmdParamInstService.physicalRemoveByIds(innerCmdParamInstIdList);
            }
            physicalRemoveByIds(innerCmdInstIdList);
        }
    }

    public List<String> queryInnerIdList(List<String> cmdParamInstIdList) {
        QueryDetail<CmdInst> query = createQuery();
        query.lambda().in(CmdInst::getCmdParamInstId, cmdParamInstIdList);
        List<CmdInst> innerCmdInstList = findAll(query);
        if (CollectionUtils.isEmpty(innerCmdInstList)) {
            return null;
        }
        return innerCmdInstList.stream().map(IdAutoEntity::getId).collect(Collectors.toList());
    }

    public void setCmdParamFileValue(List<CmdInstVO> cmdInstList, String targetId) {
        if (CollectionUtils.isNotEmpty(cmdInstList)) {
            for (CmdInstVO cmdInstVO : cmdInstList) {
                setCmdParamFileValue(cmdInstVO, targetId);
            }
        }
    }

    public void setCmdParamFileValue(CmdInstVO cmdInstVO, String targetId) {
        if (null != cmdInstVO) {
            cmdParamInstService.setCmdParamFileValue(cmdInstVO.getCmdParamInstList(), targetId);
        }
    }
}
