package com.c503.ca.cadv.module.plan.service;

import com.c503.ca.cadv.common.config.SystemProperties;
import com.c503.ca.cadv.common.mvc.QueryDetail;
import com.c503.ca.cadv.common.mvc.domain.IdAutoEntity;
import com.c503.ca.cadv.common.mvc.service.DataVOService;
import com.c503.ca.cadv.common.util.FileUtils;
import com.c503.ca.cadv.common.util.IdGen;
import com.c503.ca.cadv.common.util.StringUtils;
import com.c503.ca.cadv.enhance.ftp.FtpUtils;
import com.c503.ca.cadv.module.check.util.CheckUtil;
import com.c503.ca.cadv.module.check.util.ThreadLocalUtil;
import com.c503.ca.cadv.module.fm.utils.DataConvertUtils;
import com.c503.ca.cadv.module.ge.domain.enums.ParamSignType;
import com.c503.ca.cadv.module.info.domain.Target;
import com.c503.ca.cadv.module.info.domain.enums.CatalogType;
import com.c503.ca.cadv.module.info.domain.vo.TargetZbVO;
import com.c503.ca.cadv.module.info.service.CatalogService;
import com.c503.ca.cadv.module.info.service.TargetService;
import com.c503.ca.cadv.module.info.service.TargetZbService;
import com.c503.ca.cadv.module.plan.domain.CmdParamInst;
import com.c503.ca.cadv.module.plan.domain.vo.CmdInstVO;
import com.c503.ca.cadv.module.plan.domain.vo.CmdParamInstVO;
import com.c503.ca.cadv.module.plan.repository.CmdParamInstRepository;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.compress.utils.Lists;
import org.apache.commons.net.ftp.FTPFile;
import org.apache.commons.net.ftp.FTPFileFilter;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import serv.jcml.JCML;

import javax.annotation.Resource;
import java.io.File;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author huyouxiang
 * @date 2023/12/13 14:04
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class CmdParamInstService extends DataVOService<CmdParamInstRepository, CmdParamInst, CmdParamInstVO> {

    @Resource
    private CmdInstService cmdInstService;

    @Resource
    private TargetService targetService;

    @Resource
    private TargetZbService targetZbService;

    @Resource
    private CatalogService catalogService;

    public List<CmdParamInstVO> read(String cmdInstId) {
        List<CmdParamInstVO> cmdParamInstVOList = findByCmdInstId(cmdInstId);
        if (cmdParamInstVOList != null) {
            for (CmdParamInstVO cmdParamInstVO : cmdParamInstVOList) {
                if (ParamSignType.S.equals(cmdParamInstVO.getSingleParamSign())) {
                    List<CmdInstVO> cmdInstVOList = cmdInstService.readByCmdParamInstId(cmdParamInstVO.getId());
                    cmdParamInstVO.setCmdInstList(cmdInstVOList);
                }
            }
        } else {
            cmdParamInstVOList = Lists.newArrayList();
        }
        return cmdParamInstVOList;
    }

    public List<CmdParamInstVO> findByCmdInstId(String cmdInstId) {
        return findAllVO(createQuery().lambda().eq(CmdParamInst::getCmdInstId, cmdInstId).orderByAsc(CmdParamInst::getDataNo));
    }

    private static final Integer DEL_SUB_SIZE = 1000;

    public void physicalDelCmdParamInst(List<String> cmdInstIdList) {
        List<String> cmdParamInstIdList = queryIdList(cmdInstIdList);
        if (CollectionUtils.isNotEmpty(cmdParamInstIdList)) {
            int size = cmdParamInstIdList.size();
            List<String> subCmdParamInstList;
            for (int i = 0; i < size; i = i + DEL_SUB_SIZE) {
                subCmdParamInstList = cmdParamInstIdList.subList(i, Math.min(i + DEL_SUB_SIZE, size));
                cmdInstService.physicalDelInnerCmdInst(subCmdParamInstList);
                physicalRemoveByIds(subCmdParamInstList);
            }
        }
    }

    public List<String> queryIdList(List<String> cmdInstIdList) {
        QueryDetail<CmdParamInst> query = createQuery();
        query.lambda().in(CmdParamInst::getCmdInstId, cmdInstIdList);
        List<CmdParamInst> cmdParamInstList = findAll(query);
        if (CollectionUtils.isEmpty(cmdParamInstList)) {
            return null;
        }
        return cmdParamInstList.stream().map(IdAutoEntity::getId).collect(Collectors.toList());
    }

    public void setCmdParamFileValue(List<CmdParamInstVO> cmdParamInstList, String targetId) {
        if (CollectionUtils.isNotEmpty(cmdParamInstList)) {
            for (CmdParamInstVO cmdParamInstVO : cmdParamInstList) {
                setCmdParamFileValue(cmdParamInstVO, targetId);
            }
        }
    }

    public void setCmdParamFileValue(CmdParamInstVO cmdParamInstVO, String targetId) {
        if (null != cmdParamInstVO) {
            Target target = targetService.findOne(targetId);
            if (target == null) {
                log.error("目标[{}]不存在", targetId);
                return;
            }
            TargetZbVO targetZbVO = targetZbService.findByTargetId(targetId);
            if (targetZbVO == null) {
                log.error("目标[{}]主备机未配置", target.getTargetNo());
                return;
            }
            String ftpId = targetZbVO.getMainId() != null ? targetZbVO.getMainId() : targetZbVO.getStandbyId();
            if (ftpId == null) {
                log.error("目标[{}]主备机的配置为空", target.getTargetNo());
                return;
            }
            Integer result = setCmdParamFileValue(ftpId, target.getTaskId(), targetId, cmdParamInstVO);
            //若在主机下载，但未找到值，则去备机下载
            if (ftpId.equals(targetZbVO.getMainId()) && result == -1 && targetZbVO.getStandbyId() != null) {
                ftpId = targetZbVO.getStandbyId();
                setCmdParamFileValue(ftpId, target.getTaskId(), targetId, cmdParamInstVO);
            }
        }
    }

    /**
     * @return 1--获取文件值成功，-1--获取文件值失败，0--无需获取文件值
     */
    public Integer setCmdParamFileValue(String ftpId, String taskId, String targetId, CmdParamInstVO cmdParamInstVO) {
        switch (cmdParamInstVO.getSingleParamSign()) {
            case D:
                if (StringUtils.isNotEmpty(cmdParamInstVO.getParaPath())) {
                    return setFilePathValueAndFile(ftpId, taskId, targetId, cmdParamInstVO);
                }
                return 0;
            case B:
            case W:
                //读orin下，为填表值的文件，文件内容与反算值比对
                return setOrinFileContentAndFile(ftpId, taskId, targetId, cmdParamInstVO);
            case M:
                //根据取值路径，取orin目录下文件，获取标签值，与反算文件参数值比较
                cmdParamInstVO.setValue(null);
                return setFilePathValueAndFile(ftpId, taskId, targetId, cmdParamInstVO);
            case F:
                //根据参数实例的取值路径与参数值，获取文件标签内容，与反算文件参数值比较
                return setFilePathValueAndFile(ftpId, taskId, targetId, cmdParamInstVO);
            case S:
                cmdInstService.setCmdParamFileValue(cmdParamInstVO.getCmdInstList(), targetId);
                return 0;
            default:
                return 0;
        }
    }

    private Integer setOrinFileContentAndFile(String ftpId, String taskId, String targetId, CmdParamInstVO cmdParamInstVO) {
        cmdParamInstVO.setFileValue(null);
        cmdParamInstVO.setFile(null);

        String value = cmdParamInstVO.getValue();
        value = StringUtils.isEmpty(value) ? cmdParamInstVO.getParaPath() : value;

        if (StringUtils.isEmpty(value)) {
            return -1;
        }

        if (!value.contains(".")) {
            String fileFullName = findOrinFileFullName(ftpId, taskId, targetId, value);
            if (StringUtils.isEmpty(fileFullName)) {
                return -1;
            } else {
                value = fileFullName;
            }
        }

        File localFile = createLocalOrinFile(value);
        String fileValue = getOrinFileContentAndFile(ftpId, taskId, targetId, cmdParamInstVO.getValue(), localFile);
        cmdParamInstVO.setFileValue(fileValue);
        if (localFile.exists() && localFile.length() > 0) {
            cmdParamInstVO.setParaFileLocalPath(localFile.getAbsolutePath());
        }
        return StringUtils.isNotEmpty(fileValue) ? 1 : -1;
    }

    private String getOrinFileContentAndFile(String ftpId, String taskId, String targetId, String fileName, File cacheFile) {
        if (cacheFile == null) {
            return null;
        }
        String serverPath = CheckUtil.getServerPath(taskId, targetId, CatalogType.ORIN, null);
        String remoteFilePath = serverPath + fileName;
        try {
            boolean success = FtpUtils.retrieve(ftpId, remoteFilePath, cacheFile);
            if (!success) {
                cacheFile.delete();
                log.error("文件[{}]下载失败(getOrinFileContentAndFile)", remoteFilePath);
                return null;
            }
            if (fileName.endsWith(".bin")) {
                byte[] bytes = FileUtils.readFileToByteArray(cacheFile);
                return DataConvertUtils.byteArrToHex(bytes);
            }
            return FileUtils.readFileToString(cacheFile, "UTF-8");
        } catch (Exception e) {
            cacheFile.delete();
            log.error("文件[{}]下载错误(getOrinFileContentAndFile): {}", remoteFilePath, e);
        }
        return null;
    }

    private FTPFile[] getParamFileList(String ftpId, String serverPath, FTPFileFilter ftpFileFilter) {
        try {
            if (ftpFileFilter != null) {
                return FtpUtils.listPathFiles(ftpId, serverPath, ftpFileFilter);
            }
            return FtpUtils.listPathFiles(ftpId, serverPath);
        } catch (Exception e) {
            log.warn("获取FTP[{}]路径[{}]下文件失败:{}", ftpId, serverPath, e);
            if (e instanceof RuntimeException) {
                throw new RuntimeException(e);
            }
            return null;
        }
    }

    private String findOrinFileFullName(String ftpId, String taskId, String targetId, String fileName) {
        FTPFile[] ftpFiles = getParamFileList(ftpId, CheckUtil.getServerPath(taskId, targetId, CatalogType.ORIN, null),
                ftpFile -> ftpFile.isFile() && fileName.equals(ftpFile.getName().substring(0, ftpFile.getName().lastIndexOf("."))));
        return ftpFiles != null && ftpFiles.length > 0 ? ftpFiles[0].getName() : null;
    }

    private Integer setFilePathValueAndFile(String ftpId, String taskId, String targetId, CmdParamInstVO cmdParamInstVO) {
        cmdParamInstVO.setFileValue(null);
        cmdParamInstVO.setFile(null);
        String[] paraPaths = cmdParamInstVO.getParaPath().split(StringUtils.SPLIT_DEFAULT);
        if (StringUtils.isNotEmpty(cmdParamInstVO.getValue())) {
            paraPaths[0] = cmdParamInstVO.getValue();
        }
        String fileValue = null;
        try {
            String serverPath = CheckUtil.getServerPath(taskId, targetId, CatalogType.ORIN, null);
            String fileFullName = paraPaths[0];
            String index = null;
            if (fileFullName.contains("@")) {
                String[] split = fileFullName.split("@");
                fileFullName = split[0];
                index = split[1];
            }
            String labelPath = null;
            if (paraPaths.length > 2) {
                labelPath = paraPaths[2];
            }
            File cacheFile = null;
            JCML jcml = (JCML) ThreadLocalUtil.getTempLocal(fileFullName);
            String result = ThreadLocalUtil.getResultLocal(fileFullName);
            if (jcml == null || StringUtils.isEmpty(result)) {
                try {
                    cacheFile = createLocalOrinFile(fileFullName);
                    String remoteFilePath = serverPath + fileFullName;
                    boolean success = FtpUtils.retrieve(ftpId, remoteFilePath, cacheFile);
                    ThreadLocalUtil.setTempLocal(fileFullName, jcml);
                    ThreadLocalUtil.setResultLocal(fileFullName, cacheFile.getAbsolutePath());
                } catch (Exception e) {
                    cacheFile.delete();
                    if (e instanceof RuntimeException) {
                        log.error("[{}]文件不存在", serverPath + fileFullName);
                    } else {
                        log.error("获取文件失败");
                    }
                }
            } else {
                cacheFile = new File(result);
            }
            fileValue = getFilePathValueAndFile(serverPath, labelPath, jcml, index);
            cmdParamInstVO.setFileValue(fileValue);
            if (cacheFile.exists() && cacheFile.length() > 0) {
                cmdParamInstVO.setParaFileLocalPath(cacheFile.getAbsolutePath());
            }
            if (null == fileValue && null == labelPath) {
                return 0;
            }
        } catch (Exception e) {
            log.error("获取文件值失败(setFilePathValueAndFile):{}", e.getMessage());
        }
        return StringUtils.isNotEmpty(fileValue) ? 1 : -1;
    }

    public File createLocalOrinFile(String fileFullName) {
        String fileName = fileFullName.substring(0, fileFullName.lastIndexOf(".")) + "_"
                + IdGen.randomLongStr() + fileFullName.substring(fileFullName.lastIndexOf("."));
        String filePath = SystemProperties.getInstance().getFileUploadPath() + "/orin/" + fileName;
        File file = new File(filePath);
        if (!file.getParentFile().exists()) {
            file.getParentFile().mkdirs();
        }
        try {
            file.createNewFile();
            return file;
        } catch (Exception e) {
            log.error("创建缓存文件[{}]失败:{}", fileName, e);
        }
        return null;
    }

    public String getFilePathValueAndFile(String serverPath, String praPath, JCML jcml, String index) {
        String value = null;
        try {
             if (praPath == null) {
                 return null;
             }
             if (index != null && praPath.contains("[") && praPath.contains("]")) {
                 praPath = praPath.substring(0, praPath.indexOf('[') + 1) + index + praPath.substring(praPath.indexOf(']'));
             }
             value = jcml.cmlGetString(praPath);
        } catch (Exception e) {
            log.error("文件[{}]内结构与取值路径[{}]不符", serverPath, praPath);
        }
        return value;
    }
}
