package com.c503.ca.cadv.module.plan.service;

import com.c503.ca.cadv.common.config.SystemProperties;
import com.c503.ca.cadv.common.mvc.service.DataVOService;
import com.c503.ca.cadv.common.util.FileUtils;
import com.c503.ca.cadv.module.plan.domain.ConfigFile;
import com.c503.ca.cadv.module.plan.domain.vo.ConfigFileVO;
import com.c503.ca.cadv.module.plan.domain.vo.FileDataVO;
import com.c503.ca.cadv.module.plan.repository.ConfigFileRepository;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * @author huyouxiang
 * @date 2023/12/19 8:53
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class ConfigFileService extends DataVOService<ConfigFileRepository, ConfigFile, ConfigFileVO> {

    @Resource
    private FileDataService fileDataService;

    public ConfigFileVO read(String configFileId) {
        ConfigFile configFile = findOne(configFileId);
        ConfigFileVO configFileVO = copyBeanToVO(configFile);
        if (configFileVO != null) {
            configFileVO.setVersion(configFile.getVersion());
            List<FileDataVO> fileDataVOList = fileDataService.read(configFileVO.getId());
            configFileVO.setFileDataList(fileDataVOList);
        }
        return configFileVO;
    }

    public boolean physicalDel(List<String> idList) {
        boolean success = physicalRemoveByIds(idList);
        if (success) {
            Thread thread = new Thread(() -> fileDataService.physicalDelFileData(idList));
            thread.setName(SystemProperties.getInstance().getSystemName() + "-del-configFile");
            thread.start();
        }
        return success;
    }

    public boolean generateFile(ConfigFile configFile, HttpServletResponse response) {
        try {
            FileUtils.downloadStrToFile(response, configFile.generateText(), configFile.getFileName() + ConfigFile.SUFFIX);
            return true;
        } catch (Exception e) {
            log.warn("注入配置文件[{}]下载失败", configFile.getFileName());
            return false;
        }
    }
}
