package com.c503.ca.cadv.module.plan.util;

import com.c503.ca.cadv.common.util.DateUtils;
import com.c503.ca.cadv.common.util.JacksonUtils;
import com.c503.ca.cadv.module.check.domain.CompResult;
import com.c503.ca.cadv.module.check.domain.StCmdPara;
import com.c503.ca.cadv.module.cmp.util.RevCalCompConfigUtils;
import com.c503.ca.cadv.module.ge.util.CmdParamUtil;
import com.c503.ca.cadv.module.plan.domain.vo.CmdParamInstVO;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;

import java.math.BigDecimal;
import java.math.MathContext;
import java.math.RoundingMode;
import java.util.Date;
import java.util.TimeZone;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

@Slf4j
public class CompUtils {

    public static final int PRECISION_THIRTEEN = 15;
    public static final int PRECISION_SEVEN = 7;

    public static final String ABS_TIME = "ABS_TIME";
    public static final String RLT_TIME = "RLT_TIME";
    public static final String FLOAT = "FLOAT";

    /**
     * 根据参数类型，参数值与反算值比对
     */
    public static boolean cmdParamValAndRsltFileVal(CmdParamInstVO cmdParamInstVO, StCmdPara stCmdPara) {
        String paramValue = cmdParamInstVO.getParamValue();
        String szCalcValue = stCmdPara.getSzCalcValue();
        switch (CmdParamUtil.getTypeCode(cmdParamInstVO.getType())) {
            case CmdParamUtil.SIGNED: // 0
            case CmdParamUtil.UNSIGNED: // 1
            case CmdParamUtil.STD_FLOAT: // 2
            case CmdParamUtil.USTD_FLOAT: // 3
            case CmdParamUtil.USTD_INT: // 9
            case CmdParamUtil.FLOAT_ABSTIME: // 10
            case CmdParamUtil.FILL_FOALT: // 17
                return numWithNum(cmdParamInstVO, stCmdPara, paramValue, szCalcValue);
            case CmdParamUtil.URLTTIME: // 4
                return rltTimeOrFloatWithRltTimeOrFloat(cmdParamInstVO, stCmdPara);
            case CmdParamUtil.DATABUF: // 5
                return strContainsStr(cmdParamInstVO, stCmdPara, paramValue, szCalcValue);
            case CmdParamUtil.FIX_POINT: // 6
                return false;
            case CmdParamUtil.US_FIX_POINT: // 7
                return false;
            case CmdParamUtil.RLTTIME: // 8
            case CmdParamUtil.FLOAT_RLTTIME: // 14
            case CmdParamUtil.TF_FLOAT_TIME: // 15
            case CmdParamUtil.BIT_ABSTIME: // 19
                return timeWithTime(cmdParamInstVO, stCmdPara);
            case CmdParamUtil.POINT_RLTTIME: // 11
                return false;
            case CmdParamUtil.US_FIX_ABSTIME: // 12
                return absTimeWithAbsTime(cmdParamInstVO, stCmdPara);
            case CmdParamUtil.CMDBUF: // 13
            case CmdParamUtil.GPS_WEEKTIME: // 16
                return strWithStr(cmdParamInstVO, stCmdPara, paramValue, szCalcValue);
            case CmdParamUtil.BIT_RLTTIME: // 18
            case CmdParamUtil.BIT8_RLTTIME: //20
                return rltTimeToFloatWithRltTimeToFloat(cmdParamInstVO, stCmdPara);
            default:
                log.error("cmdParamValAndRsltFileVal指令参数类型匹配失败");
        }
        return false;
    }

    private static boolean strContainsStr(CmdParamInstVO cmdParamInstVO, StCmdPara stCmdPara, String paramValue, String szCalcValue) {
        try {
            cmdParamInstVO.setCompValue(paramValue.toLowerCase());
            if (StringUtils.isEmpty(stCmdPara.getCompValue())) {
                stCmdPara.setCompValue(szCalcValue.toLowerCase());
            } else {
                stCmdPara.setCompValue(stCmdPara.getCompValue().toLowerCase());
            }
            boolean equal = stCmdPara.getCompValue().contains(cmdParamInstVO.getCompValue());
            if (!equal) {
                equal = RevCalCompConfigUtils.check(cmdParamInstVO.getCompValue(), stCmdPara.getCompValue(), cmdParamInstVO.getObjectCode(), cmdParamInstVO);
            }
            cmdParamInstVO.setCompResult(equal ? CompResult.COMP_EQUAL : CompResult.COMP_NO_EQUAL);
            stCmdPara.setCompResult(equal ? CompResult.COMP_EQUAL : CompResult.COMP_NO_EQUAL);
            return equal;
        } catch (Exception e) {
            log.error("strContainsStr:{},{}", paramValue, szCalcValue);
            setErrorResult(cmdParamInstVO, stCmdPara);
            return false;
        }
    }

    /**
     * 时间与时间的比较
     */
    private static boolean timeWithTime(CmdParamInstVO cmdParamInstVO, StCmdPara stCmdPara) {
        String paramValue = cmdParamInstVO.getParamValue();
        // 判断参数值为何种类型 --- 相对时，绝对时，浮点数
        String absOrRltOrFloat = isAbsOrRltOrFloat(paramValue);
        if (null == absOrRltOrFloat) {
            setErrorResult(cmdParamInstVO, stCmdPara);
        }
        if (ABS_TIME.equals(absOrRltOrFloat)) {
            return absTimeWithAbsTime(cmdParamInstVO, stCmdPara);
        } else {
            return rltTimeOrFloatWithRltTimeOrFloat(cmdParamInstVO, stCmdPara);
        }
    }

    /**
     * 绝对时与绝对时比较
     */
    private static boolean absTimeWithAbsTime(CmdParamInstVO cmdParamInstVO, StCmdPara stCmdPara) {
        try {
            String paramValue = cmdParamInstVO.getParamValue();
            String szCalcValue = stCmdPara.getSzCalcValue();
            cmdParamInstVO.setCompValue(paramValue);
            stCmdPara.setCompValue(szCalcValue);
            String pattern1 = "yyyy/MM/dd HH:mm:ss.SSSS" , pattern2 = "yyyy/MM/dd HH:mm:ss.SSSS";
            if (paramValue.contains("T")) {
                pattern1 = "yyyy-MM-dd HH:mm:ss.SSSS";
                paramValue = paramValue.replace("T" , " ");
            } else {
                paramValue = paramValue.replace("-" , " ");
            }
            if (szCalcValue.contains("T")) {
                pattern2 = "yyyy-MM-dd HH:mm:ss.SSSS";
                szCalcValue = szCalcValue.replace("T" , " ");
            } else {
                szCalcValue = szCalcValue.replace("-" , " ");
            }
            Date paramDate = DateUtils.parseDate(paramValue, pattern1, TimeZone.getTimeZone("Asia/Shanghai"));
            Date calDate = DateUtils.parseDate(szCalcValue, pattern2, TimeZone.getTimeZone("Asia/Shanghai"));
            boolean equal = Math.abs(paramDate.getTime() - calDate.getTime()) <= 5;
            if (!equal) {
                equal = RevCalCompConfigUtils.check(String.valueOf(paramDate.getTime()), String.valueOf(calDate.getTime()), cmdParamInstVO.getObjectCode(), cmdParamInstVO);
            }
            cmdParamInstVO.setCompResult(equal ? CompResult.COMP_EQUAL : CompResult.COMP_NO_EQUAL);
            stCmdPara.setCompResult(equal ? CompResult.COMP_EQUAL : CompResult.COMP_NO_EQUAL);
            return equal;
        } catch (Exception e) {
            setErrorResult(cmdParamInstVO, stCmdPara);
            return false;
        }
    }

    /**
     * 相对时或浮点数与相对时或浮点数比较
     */
    private static boolean rltTimeOrFloatWithRltTimeOrFloat(CmdParamInstVO cmdParamInst, StCmdPara stCmdPara) {
        String paramValue = cmdParamInst.getParamValue();
        String szCalcValue = stCmdPara.getSzCalcValue();
        try {
            // 判断是非为时间类型
            if (paramValue.contains("T") || paramValue.lastIndexOf("-") > 1) {
                // 时间转相对秒
                paramValue = rlTimeToFloat(paramValue);
            }
            if (szCalcValue.contains("T") || szCalcValue.lastIndexOf("-") > 1) {
                szCalcValue = rlTimeToFloat(szCalcValue);
            }
            return numWithNum(cmdParamInst, stCmdPara, paramValue, szCalcValue);
        } catch (Exception e) {
            log.error("rltTimeOrFloatWithRltTimeOrFloat:{},{}" , paramValue, szCalcValue);
            return false;
        }
    }

    private static boolean rltTimeToFloatWithRltTimeToFloat(CmdParamInstVO cmdParamInstVO, StCmdPara stCmdPara) {
        BigDecimal pv, sv;
        String paramValue = cmdParamInstVO.getParamValue();
        String szCalcValue = stCmdPara.getSzCalcValue();
        try {
            paramValue = rlTimeToFloat(paramValue);
            szCalcValue = rlTimeToFloat(szCalcValue);
            pv = new BigDecimal(paramValue);
            sv = new BigDecimal(szCalcValue);
        } catch (Exception e) {
            log.error("rltTimeToFloatWithRltTimeToFloat:{},{}" , paramValue, szCalcValue);
            setErrorResult(cmdParamInstVO, stCmdPara);
            return false;
        }

        cmdParamInstVO.setCompValue(pv.toPlainString());
        stCmdPara.setCompValue(sv.toPlainString());
        boolean equal = pv.subtract(sv).abs().subtract(new BigDecimal("0.0005")).compareTo(new BigDecimal("0")) == 1 ? false : true;
        if (!equal) {
            equal = RevCalCompConfigUtils.check(pv.toPlainString(), sv.toPlainString(), cmdParamInstVO.getObjectCode(), cmdParamInstVO);
        }
        cmdParamInstVO.setCompResult(equal ? CompResult.COMP_EQUAL : CompResult.COMP_NO_EQUAL);
        stCmdPara.setCompResult(equal ? CompResult.COMP_EQUAL : CompResult.COMP_NO_EQUAL);
        return equal;
    }

    /**
     * 数值之间比较
     */
    private static boolean numWithNum(CmdParamInstVO cmdParamInstVO, StCmdPara stCmdPara, String paramValue, String szCalcValue) {
        BigDecimal pv, sv;
        boolean isHex = false;
        try {
            if (paramValue.charAt(paramValue.length() - 1) != 'H' && !paramValue.matches("^[A-Fa-f0-9]+$")
                    && szCalcValue.contains("(") && szCalcValue.contains(")")) {
                String stCompValue = szCalcValue.substring(szCalcValue.lastIndexOf('(') + 1, szCalcValue.lastIndexOf(')'));

                cmdParamInstVO.setCompValue(paramValue);
                stCmdPara.setCompValue(stCompValue);

                boolean equal = paramValue.equals(stCompValue);
                cmdParamInstVO.setCompResult(equal ? CompResult.COMP_EQUAL : CompResult.COMP_NO_EQUAL);
                stCmdPara.setCompResult(equal ? CompResult.COMP_EQUAL : CompResult.COMP_NO_EQUAL);
                return equal;
            }

            // 判断十六进制
            if (szCalcValue.charAt(szCalcValue.length() - 1) == 'H' ) {
                szCalcValue = Long.valueOf(szCalcValue.substring(0, szCalcValue.length() - 1), 16) + "";
            } else if (szCalcValue.contains("(") && szCalcValue.endsWith(")") && szCalcValue.charAt(szCalcValue.indexOf("(") - 1) == 'H') {
                szCalcValue = Long.valueOf(szCalcValue.substring(0, szCalcValue.indexOf("(") - 1), 16) + "";
            }
            if (paramValue.charAt(paramValue.length() - 1) == 'H') {
                paramValue = Long.valueOf(paramValue.substring(0, paramValue.length() - 1), 16) + "";
                isHex = true;
            } else if  (paramValue.contains("(") && paramValue.contains(")") && paramValue.charAt(paramValue.indexOf("(") - 1) == 'H') {
                paramValue = Long.valueOf(paramValue.substring(0, paramValue.indexOf("(") - 1), 16) + "";
                isHex = true;
            }

            // 判断二进制
            if (szCalcValue.charAt(szCalcValue.length() - 1) == 'B' ) {
                szCalcValue = Long.valueOf(szCalcValue.substring(0, szCalcValue.length() - 1), 2) + "";
            } else if (szCalcValue.contains("(") && szCalcValue.endsWith(")") && szCalcValue.charAt(szCalcValue.indexOf("(") - 1) == 'B') {
                szCalcValue = Long.valueOf(szCalcValue.substring(0, szCalcValue.indexOf("(") - 1), 2) + "";
            }
            if (paramValue.charAt(paramValue.length() - 1) == 'B') {
                paramValue = Long.valueOf(paramValue.substring(0, paramValue.length() - 1), 2) + "";
                isHex = true;
            } else if  (paramValue.contains("(") && paramValue.contains(")") && paramValue.charAt(paramValue.indexOf("(") - 1) == 'B') {
                paramValue = Long.valueOf(paramValue.substring(0, paramValue.indexOf("(") - 1), 2) + "";
                isHex = true;
            }

            pv = new BigDecimal(paramValue);
            sv = new BigDecimal(szCalcValue);
            // 根据指令参数长度，获取有效位数
            if (cmdParamInstVO.getLen() != null && cmdParamInstVO.getLen() == 64) {
                pv = validDigital(pv, PRECISION_THIRTEEN);
                sv = validDigital(sv, PRECISION_THIRTEEN);
            } else {
                pv = validDigital(pv, PRECISION_SEVEN);
                sv = validDigital(sv, PRECISION_SEVEN);
            }
        } catch (Exception e) {
            log.error("numWithNum:{},{}" , paramValue, szCalcValue);
            setErrorResult(cmdParamInstVO, stCmdPara);
            return false;
        }

        if (isHex && StringUtils.isNotEmpty(cmdParamInstVO.getDimen())) {
            pv = pv.multiply(new BigDecimal(getDimen(cmdParamInstVO.getDimen())));
        }

        cmdParamInstVO.setCompValue(pv.toPlainString());
        stCmdPara.setCompValue(sv.toPlainString());
        boolean equal = pv.compareTo(sv) == 0;
        if (!equal) {
            equal = RevCalCompConfigUtils.check(pv.toPlainString(), sv.toPlainString(), cmdParamInstVO.getObjectCode(), cmdParamInstVO);
        }
        cmdParamInstVO.setCompResult(equal ? CompResult.COMP_EQUAL : CompResult.COMP_NO_EQUAL);
        stCmdPara.setCompResult(equal ? CompResult.COMP_EQUAL : CompResult.COMP_NO_EQUAL);
        return equal;
    }


    public static String getDimen(String str) {
        String number = getNumber(str);
        if (StringUtils.isEmpty(number)) {
            return "1";
        }
        return number;
    }

    public static String getNumber(String str) {
        if (StringUtils.isNotEmpty(str)) {
            String pReg = "(\\d+\\.\\d+)";
            Pattern p = Pattern.compile(pReg);
            Matcher m = p.matcher(str);
            if (m.find()) {
                str = m.group(1) == null ? "" : m.group(1);
            } else {
                pReg = "(\\d+)";
                p = Pattern.compile(pReg);
                m = p.matcher(str);
                if (m.find()) {
                    str = m.group(1) == null ? "" : m.group(1);
                } else {
                    str = "";
                }
            }
            return str;
        }
        return null;
    }


    private static boolean strWithStr(CmdParamInstVO cmdParamInstVO, StCmdPara stCmdPara, String paramValue, String szCalcValue) {
        try {
            cmdParamInstVO.setCompValue(paramValue);
            if (StringUtils.isEmpty(stCmdPara.getCompValue())) {
                stCmdPara.setCompValue(szCalcValue);
            }
            boolean equal = cmdParamInstVO.getCompValue().equals(stCmdPara.getCompValue());
            if (!equal) {
                equal = RevCalCompConfigUtils.check(cmdParamInstVO.getCompValue(), stCmdPara.getCompValue(), cmdParamInstVO.getObjectCode(), cmdParamInstVO);
            }
            cmdParamInstVO.setCompResult(equal ? CompResult.COMP_EQUAL : CompResult.COMP_NO_EQUAL);
            stCmdPara.setCompResult(equal ? CompResult.COMP_EQUAL : CompResult.COMP_NO_EQUAL);
            return equal;
        } catch (Exception e) {
            log.error("strWithStr:{},{}" , paramValue, szCalcValue);
            setErrorResult(cmdParamInstVO, stCmdPara);
            return false;
        }
    }

    /**
     * 判断参数值为何种类型 --- 相对时，绝对时，浮点数
     */
    private static String isAbsOrRltOrFloat(String paramValue) {
        try {
            if (paramValue.contains("0000/00/") || paramValue.contains("0000-00-") || (paramValue.contains("T") && paramValue.lastIndexOf("T") < 6) || (paramValue.contains("-") && paramValue.lastIndexOf("-") < 6)) {
                return RLT_TIME;
            } else if (paramValue.contains("T") || paramValue.lastIndexOf("-") > 1) {
                return ABS_TIME;
            } else {
                return FLOAT;
            }
        } catch (Exception e) {
            log.error("isAbsOrRltOrFloat({}):{}" , paramValue, e.toString());
        }
        return null;
    }

    /**
     * 相对时转相对秒（可带符号）
     */
    private static String rlTimeToFloat(String rltTime) {
        String ch;
        if (rltTime.contains("T")) {
            ch = "T";
            if ("-".equals(rltTime.charAt(7) + "")) {
                rltTime = rltTime.substring(8);
            }
        } else {
            ch = "-";
            if ("/".equals(rltTime.charAt(7) + "")) {
                rltTime = rltTime.substring(8);
            }
        }
        String day = rltTime.substring(0, rltTime.lastIndexOf(ch));
        String sign = "";
        if ("-".equals(day.substring(0, 1))) {
            sign = "-";
            day = day.substring(1);
        }
        String hour = rltTime.substring(rltTime.lastIndexOf(ch) + 1, rltTime.indexOf(":"));
        String min = rltTime.substring(rltTime.indexOf(":") + 1, rltTime.lastIndexOf(":"));
        String s = rltTime.substring(rltTime.lastIndexOf(":") + 1, rltTime.indexOf("."));
        String ms = rltTime.substring(rltTime.indexOf(".") + 1);
        return sign + (Integer.parseInt(day) * 24 * 60 * 60 + Integer.parseInt(hour) * 60 * 60 + Integer.parseInt(min) * 60 + Integer.parseInt(s)) + "." + ms;
    }

    /**
     * 保留指定位数的有效数字
     */
    public static BigDecimal validDigital(BigDecimal bigDecimal, int precision) {
        MathContext mathContext = new MathContext(precision, RoundingMode.HALF_UP);
        return bigDecimal.divide(BigDecimal.ONE, mathContext);
    }


    public static BigDecimal validDigital1(BigDecimal bigDecimal, int precision) {
        MathContext mathContext = new MathContext(precision, RoundingMode.DOWN);
        return bigDecimal.divide(BigDecimal.ONE, mathContext);
    }

    /**
     * 设置比对错误结果
     */
    public static void setErrorResult(CmdParamInstVO cmdParamInstVO, StCmdPara stCmdPara) {
        cmdParamInstVO.setCompValue(cmdParamInstVO.getParamValue());
        stCmdPara.setCompValue(stCmdPara.getSzCalcValue());
        cmdParamInstVO.setCompResult(CompResult.COMP_BREAK);
        stCmdPara.setCompResult(CompResult.COMP_BREAK);
        log.debug(">>>>>>>>>>>>>>>>>>>>>>>>>>>>>setError");
        log.debug(JacksonUtils.toJsonString(cmdParamInstVO));
        log.debug(JacksonUtils.toJsonString(stCmdPara));
    }
}