package com.c503.ca.cadv.module.plan.web;

import com.c503.ca.cadv.common.rest.base.DataVOResource;
import com.c503.ca.cadv.common.util.BeanConvertUtils;
import com.c503.ca.cadv.common.util.JacksonUtils;
import com.c503.ca.cadv.common.util.domain.CustomMessage;
import com.c503.ca.cadv.common.util.domain.PageModel;
import com.c503.ca.cadv.common.util.domain.ResultBuilder;
import com.c503.ca.cadv.module.log.util.MyLog;
import com.c503.ca.cadv.module.log.util.OperateType;
import com.c503.ca.cadv.module.plan.domain.ConfigFile;
import com.c503.ca.cadv.module.plan.domain.vo.ConfigFileVO;
import com.c503.ca.cadv.module.plan.service.ConfigFileService;
import com.fasterxml.jackson.databind.JsonNode;
import com.google.common.collect.Lists;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

/**
 * @author huyouxiang
 * @date 2023/12/19 8:58
 */
@RestController
@RequestMapping("/plan/config-file")
@Api(tags = "config-file")
public class ConfigFileResource extends DataVOResource<ConfigFileService, ConfigFileVO> {
    public ConfigFileResource(ConfigFileService service) {
        super(service);
    }

    @GetMapping(value = "/")
    @ApiOperation(value = "获取注入配置生成记录", notes = "根据分页参数，获取分页列表")
    public ResponseEntity<CustomMessage<JsonNode>> getPage(PageModel pm) {
        service.findVOPage(pm);
        JsonNode rs = JacksonUtils.toJsonNodeWithDefaultMapper(pm);
        return ResultBuilder.buildObject(rs);
    }

    /**
     * 批量删除
     */
    @PostMapping(value = "/del")
    @ApiOperation(value = "删除实体", notes = "批量删除，ID通过逗号分隔")
    @MyLog(value = "删除注入配置生成记录", type = OperateType.DELETE, model = "注入计划")
    public ResponseEntity<CustomMessage<String>> delete(@RequestBody String[] ids) {
        service.physicalDel(Lists.newArrayList(ids));
        return ResultBuilder.buildOk("删除成功");
    }


    @GetMapping(value = "/read")
    @ApiOperation(value = "读取数据库中的注入配置生成记录", notes = "")
    public ResponseEntity<CustomMessage<ConfigFileVO>> read(@RequestParam("configFileId") String configFileId) {
        ConfigFileVO configFileVO = service.read(configFileId);
        return ResultBuilder.buildObject(configFileVO);
    }

    /**
     * 下载注入配置文件
     */
    @GetMapping(value = "/downloadFile")
    @ApiOperation(value = "下载注入配置文件", notes = "")
    public ResponseEntity<CustomMessage<String>> downloadFile(@RequestParam("configFileId") String configFileId) {
        ConfigFileVO configFileVO = service.read(configFileId);
        if (configFileVO == null) {
            ResultBuilder.buildFailed("注入配置文件[" + configFileId + "]数据不存在");
        }
        boolean success = service.generateFile(BeanConvertUtils.map(configFileVO, ConfigFile.class), response());
        return success ? ResultBuilder.buildOk("下载注入配置文件成功") : ResultBuilder.buildFailed("下载注入配置文件失败");
    }

}
