package com.c503.ca.cadv.module.plan.web;

import com.c503.ca.cadv.common.rest.base.DataVOResource;
import com.c503.ca.cadv.common.util.CollectionsUtils;
import com.c503.ca.cadv.common.util.JacksonUtils;
import com.c503.ca.cadv.common.util.domain.CustomMessage;
import com.c503.ca.cadv.common.util.domain.PageModel;
import com.c503.ca.cadv.common.util.domain.ResultBuilder;
import com.c503.ca.cadv.module.log.util.MyLog;
import com.c503.ca.cadv.module.log.util.OperateType;
import com.c503.ca.cadv.module.plan.domain.FileData;
import com.c503.ca.cadv.module.plan.domain.vo.CmdInstVO;
import com.c503.ca.cadv.module.plan.domain.vo.FileDataVO;
import com.c503.ca.cadv.module.plan.service.CmdInstService;
import com.c503.ca.cadv.module.plan.service.FileDataService;
import com.fasterxml.jackson.databind.JsonNode;
import com.google.common.collect.Lists;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiModelProperty;
import io.swagger.annotations.ApiOperation;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author huyouxiang
 * @date 2023/12/13 14:12
 */
@RestController
@RequestMapping("/plan/file-data")
@Api(tags = "file-data")
public class FileDataResource extends DataVOResource<FileDataService, FileDataVO> {

    public FileDataResource(FileDataService service) {
        super(service);
    }

    @Resource
    private CmdInstService cmdInstService;

    @GetMapping(value = "/getPageWithoutCmdInst")
    @ApiOperation(value = "获取注入数据列表,不含指令实例信息", notes = "根据分页参数，获取分页列表")
    public ResponseEntity<CustomMessage<JsonNode>> getPageWithoutCmdInst(PageModel pm) {
        service.findVOPage(pm);
        JsonNode rs = JacksonUtils.toJsonNodeWithDefaultMapper(pm);
        return ResultBuilder.buildObject(rs);
    }

    @GetMapping(value = "/")
    @ApiOperation(value = "获取注入文件数据表列表", notes = "根据分页参数，获取分页列表")
    public ResponseEntity<CustomMessage<JsonNode>> getPage(PageModel pm) {
        service.findVOPage(pm);
        List<FileDataVO> fileDataVOList = pm.getData();
        if (CollectionsUtils.isNotEmpty(fileDataVOList)) {
            for (FileDataVO fileDataVO : fileDataVOList) {
                List<CmdInstVO> cmdInstVOList = cmdInstService.readByFileDataId(fileDataVO.getId());
                fileDataVO.setCmdInstList(cmdInstVOList);
            }
        }
        JsonNode rs = JacksonUtils.toJsonNodeWithDefaultMapper(pm);
        return ResultBuilder.buildObject(rs);
    }

    @PostMapping(value = "/del")
    @ApiOperation(value = "删除实体", notes = "批量删除，ID通过逗号分隔")
    @MyLog(value = "删除注入配置生成记录", type = OperateType.DELETE, model = "注入计划")
    public ResponseEntity<CustomMessage<String>> delete(@RequestBody String[] ids) {
        service.physicalDel(Lists.newArrayList(ids));
        return ResultBuilder.buildOk("删除成功");
    }

}
