package com.c503.ca.cadv.module.sys.service;

import com.c503.ca.cadv.common.mvc.QueryDetail;
import com.c503.ca.cadv.common.mvc.service.TreeVOService;
import com.c503.ca.cadv.common.util.BeanConvertUtils;
import com.c503.ca.cadv.common.util.StringUtils;
import com.c503.ca.cadv.module.sys.domain.Dict;
import com.c503.ca.cadv.module.sys.domain.query.DictTreeResult;
import com.c503.ca.cadv.module.sys.domain.vo.DictVO;
import com.c503.ca.cadv.module.sys.repository.DictRepository;
import com.google.common.collect.Lists;
import org.apache.commons.collections.CollectionUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

/**
 * Service class for managing dicts.
 */
@Service
public class DictService extends TreeVOService<DictRepository, Dict, DictVO> {

    public List<DictVO> findAllVOIsShowOrderBySortAsc(Integer isShow) {
        QueryDetail<Dict> queryDetail = createQuery();
        queryDetail.lambda().eq(Dict::getIsShow, isShow).orderByAsc(Dict::getSort);
        List<Dict> dicts = findAll(queryDetail.getWrapper());
        return BeanConvertUtils.mapAsList(dicts, DictVO.class);
    }

    @Transactional(readOnly = true, rollbackFor = Exception.class)
    public List<DictTreeResult> assembleTreeData(List<DictVO> dictList) {
        List<DictTreeResult> mapList = Lists.newArrayList();
        for (DictVO e : dictList) {
            DictTreeResult dictTreeResult = new DictTreeResult();
            dictTreeResult.setId(e.getId());
            dictTreeResult.setPid(StringUtils.isEmpty(e.getParentId()) ? "0" : e.getParentId());
            dictTreeResult.setName(e.getName());
            dictTreeResult.setCode(e.getCode());
            dictTreeResult.setValue(e.getVal());
            dictTreeResult.setLabel(dictTreeResult.getName());
            mapList.add(dictTreeResult);
        }
        return mapList;
    }

    @Override
    public boolean saveOrUpdate(Dict entity) {
        String newCode = entity.getCode();
        Dict originEntity = super.getById(entity.getId());
        String oldCode = null;
        if (originEntity != null) {
            oldCode = originEntity.getCode();
        }

        boolean success = super.saveOrUpdate(entity);
        if (success) {
            if (originEntity != null && !newCode.equals(oldCode)) {
                List<Dict> list = findAllByParentId(entity.getId());
                for (Dict e : list) {
                    e.setParentCode(newCode);
                }
                if (CollectionUtils.isNotEmpty(list)) {
                    saveOrUpdateBatch(list);
                }
            }
        }
        return success;
    }

}
