/**
 * Copyright &copy; 2020-2021  c503-htphy All rights reserved.
 */
package com.c503.ca.cadv.module.sys.web;

import com.alibaba.fastjson.JSONObject;
import com.c503.ca.cadv.common.util.CollectionsUtils;
import com.c503.ca.cadv.common.util.StringUtils;
import com.c503.ca.cadv.common.util.domain.CacheConstants;
import com.c503.ca.cadv.common.util.domain.CustomMessage;
import com.c503.ca.cadv.common.util.domain.ResultBuilder;
import com.c503.ca.cadv.enhance.redis.util.RedisUtils;
import com.google.common.collect.Maps;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 控制器
 * </p>
 *
 * @author jintch
 */
@RestController
@RequestMapping("/sys")
@Api(tags = "Sys")
@Slf4j
public class SysResource {

    /**
     * 获取系统时间
     */
    @GetMapping(value = "/getDate")
    @ApiOperation(value = "获取系统时间", notes = "获取系统时间")
    public ResponseEntity<CustomMessage<Date>> getDate() {
        return ResultBuilder.buildObject(new Date());
    }

    /**
     * 获取时间戳（1970）
     */
    @GetMapping(value = "/getTimeMillis")
    @ApiOperation(value = "获取时间戳（1970）", notes = "获取时间戳（1970）")
    public ResponseEntity<CustomMessage<Long>> getTimeMillis(@RequestParam(required = false) Date date) {
        return ResultBuilder.buildDataOk(date == null ? System.currentTimeMillis() : date.getTime());
    }

    /**
     * 获取环境
     */
    @GetMapping(value = "/getEnv")
    @ApiOperation(value = "获取环境", notes = "获取环境")
    public ResponseEntity<CustomMessage<String>> getEnv() {
        return ResultBuilder.buildDataOk(System.getenv("BTAS3"));
    }

}
