package serv.jcml;

import org.aspectj.util.FileUtil;

import java.io.File;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
public class JCML{
	public int rslt = 0;              //rslt<0:error;rslt>=0;true;
	private long handle = 0;
	private int MAX_ERRNO = -100000;

	private native static long jcmlLoadDocFromFile(String filename);
	private native static long jcmlLoadDocFromMem(byte[] content);
	private native static int  jcmlGetString(long handle,String path, JCML_DField o_value);
	private native static int  jcmlGetInt(long handle,String path, JCML_DField o_value);
	private native static int  jcmlGetBinary(long handle,String path, JCML_DField o_value);
	private native static void jcmlFreeDocHandler(long handle);
	private native static long jcmlNewEmptyDoc(String schemaname);
	private native static int  jcmlPutString(long handle,String path, byte[] content);
	private native static int  jcmlPutInt(long handle,String path, int value);
	private native static int  jcmlDumpDocToMem(long handle, JCML_DField o_data, int ifcompress);
	private native static int  jcmlDumpDocToFile(long handle,String fileName ,int ifcompress);
	private native static int  jcmlPutBinary(long handle,String path, byte[] content);

	/**
	 @brief 检查xml服务是否初始化
	 @param[in] null
	 @return null
	 */
	public boolean cmlIsInit() {
		///判断xml服务是否已初始化
		if(handle == 0) {
			return false;
		}
		rslt = 0;
		return true;
	}

	/**
	 @brief 加载xml格式文件
	 @param[in] String filename 文件名
	 @return true--success, false--failure
	 */
	public boolean cmlLoadDocFromFile(String filename) {
		///判断加载对象是否为空
		if(filename == null) {
			rslt = MAX_ERRNO - 1;
			return false;
		}
		if(handle != 0) {
			jcmlFreeDocHandler(handle);
		}
		///调用xml服务提供的接口加载xml格式文件
		handle = jcmlLoadDocFromFile(filename);
		if(handle == 0) {
			rslt = -1;
			return false;
		}
		rslt = 0;
		return true;
	}

	/**
	 @brief 加载字节数组数据格式
	 @param[in] byte[] content 字节数组
	 @return true--success, false--failure
	 */
	public boolean cmlLoadDocFromMem(byte[] content) {
		///判断加载对象是否为空
		if(content == null) {
			rslt = MAX_ERRNO - 1;
			return false;
		}
		if(handle != 0) {
			jcmlFreeDocHandler(handle);
		}
		///调用xml服务提供的接口加载字节数组
		handle = jcmlLoadDocFromMem(content);
		if(handle == 0) {
			rslt = -1;
			return false;
		}
		rslt = 0;
		return true;
	}

	/**
	 @brief 从xml里获取string型属性
	 @param[in] String path 目标属性路径
	 @return 目标属性--success, null--failure
	 */
	public String cmlGetString(String path) {
		///判断目标属性路径是否存在
		if(path == null) {
			rslt = MAX_ERRNO - 1;
			return null;
		}
		JCML_DField o_value = new JCML_DField();
		///调用xml服务提供的接口获取string型属性
		rslt = jcmlGetString(handle, path, o_value);
		if(rslt < 0) {
			return null;
		}
		if(o_value.b_value == null) {
			return "";
		}
		String value = new String(o_value.b_value, StandardCharsets.UTF_8);
		return value;
	}

	/**
	 @brief 从xml里获取byte型属性
	 @param[in] String path 目标属性路径
	 @return 目标属性--success, null--failure
	 */
	public byte[] cmlGetBinary(String path) {
		///判断目标属性路径是否存在
		if(path == null) {
			rslt = MAX_ERRNO - 1;
			return null;
		}
		JCML_DField o_value = new JCML_DField();
		///调用xml服务提供的接口获取byte型属性
		rslt = jcmlGetBinary(handle, path, o_value);
		if(rslt < 0) {
			return null;
		}
		return o_value.b_value;
	}

	/**
	 @brief 从xml里获取int型属性
	 @param[in] String path 目标属性路径
	 @return 目标属性--success, 0--failure
	 */
	public int cmlGetInt(String path) {
		///判断目标属性路径是否存在
		if(path == null) {
			rslt = MAX_ERRNO - 1;
			return -1;
		}
		JCML_DField o_value = new JCML_DField();
		rslt = jcmlGetInt(handle, path, o_value);///调用xml服务提供的接口获取int型属性
		if(rslt < 0) {
			return -1;
		}
		return o_value.i_value;
	}

	/**
	 @brief 从xml里获取Integer型属性
	 @param[in] String path 目标属性路径
	 @return 目标属性--success, 0--failure
	 */
	public Integer cmlGetInteger(String path) {
		///判断目标属性路径是否存在
		if(path == null) {
			rslt = MAX_ERRNO - 1;
			return null;
		}
		JCML_DField o_value = new JCML_DField();
		///调用xml服务提供的接口获取int型属性
		rslt = jcmlGetInt(handle, path, o_value);
		if(rslt < 0) {
			return null;
		}
		return o_value.i_value;
	}

	/**
	 @brief 从xml里获取Double型属性
	 @param[in] String path 目标属性路径
	 @return 目标属性--success, 0--failure
	 */
	public Double cmlGetDouble(String path) {
		///判断目标属性路径是否存在
		if(path == null) {
			rslt = MAX_ERRNO - 1;
			return null;
		}
		JCML_DField o_value = new JCML_DField();
		///调用xml服务提供的接口获取Double型属性
		rslt = jcmlGetString(handle, path, o_value);
		if(rslt < 0) {
			return null;
		}
		return Double.valueOf(new String(o_value.b_value,StandardCharsets.UTF_8));
	}

	/**
	 @brief 释放xml
	 @param[in] null
	 @return null
	 */
	public void cmlFreeDocHandler() {
		///调用xml服务提供的接口释放xml
		jcmlFreeDocHandler(handle);
		handle = 0;
		rslt = 0;
	}

	/**
	 @brief 新建xml
	 @param[in] String schemaname xml的结构名称
	 @return true--success, false--failure
	 */
	public boolean cmlNewEmptyDoc(String schemaname) {
		if(schemaname == null) {
			rslt = MAX_ERRNO - 1;
			return false;
		}
		if(handle != 0) {
			jcmlFreeDocHandler(handle);
		}
		///调用xml服务提供的接口新建xml
		handle = jcmlNewEmptyDoc(schemaname);
		if(handle == 0) {
			rslt = -1;
			return false;
		}
		rslt = 0;
		return true;
	}

	/**
	 @brief 设置xml字符串型属性
	 @param[in] String path 目标属性结构路径
	 @param[in] String content 目标属性值
	 @return >=0--success, <0--failure
	 */
	public int cmlPutString(String path, String content) {
		if(content == null || path == null) {
			rslt = MAX_ERRNO - 1;
			return rslt;
		}
		///调用xml服务提供的接口设置string型属性
		rslt = jcmlPutString(handle, path, content.getBytes());
		if(rslt < 0) {
			return rslt;
		}
		return rslt;
	}

	/**
	 @brief 设置xml字节型属性
	 @param[in] String path 目标属性结构路径
	 @param[in] String content 目标属性值
	 @return >=0--success, <0--failure
	 */
	public int cmlPutBinary(String path, byte[] content) {
		if(content == null || path == null) {
			rslt = MAX_ERRNO - 1;
			return rslt;
		}
		///调用xml服务提供的接口设置byte型属性
		rslt = jcmlPutBinary(handle, path, content);
		if(rslt < 0) {
			return rslt;
		}
		return rslt;
	}

	/**
	 @brief 设置xml整型属性
	 @param[in] String path 目标属性结构路径
	 @param[in] String content 目标属性值
	 @return >=0--success, <0--failure
	 */
	public int cmlPutInt(String path, int value) {
		if(path == null) {
			rslt = MAX_ERRNO - 1;
			return rslt;
		}
		///调用xml服务提供的接口设置int型属性
		rslt = jcmlPutInt(handle, path, value);
		if(rslt < 0) {
			return rslt;
		}
		return rslt;
	}

	/**
	 @brief 导出xml内容
	 @param[in] int ifcompress 压缩标志
	 @return >=0--success, <0--failure
	 */
	public byte[] cmlDumpDocToMem(int ifcompress) {
		JCML_DField o_data = new JCML_DField();
		///调用xml服务提供的接口导出xml内容
		rslt = jcmlDumpDocToMem(handle, o_data, ifcompress);
		if(rslt < 0) {
			return null;
		}
		return o_data.b_value;
	}

	public int cmlDumpDocToFile(String fileName ,int ifcompress) {
		if(fileName == null) {
			rslt = MAX_ERRNO - 1;
			return rslt;
		}
		rslt = jcmlDumpDocToFile(handle, fileName, ifcompress);
		return rslt;
	}
}
