package serv.jzf;

import serv.jcml.JCML;

public class JZF_FileInfo {
    public int uiFileType;                          /*<\brief 文件类型             */
    public int uiSubFileType;                       /*<\brief 文件子类型          */
    public int uiTaskCode;                          /*<\brief 任务码               */
    public int uiObjCode;                           /*<\brief 目标码               */
    public int ucDirectFlag;                        /*<\brief 仿真实战标识         */

    public int uiMsgLen;
    public byte[] sMsgCont;
    public JZF_FileMap[] FileList;
    public String FileNameS;                        /* 发送成功时回填基础服务生成的文件名 */


    public String filepath = "";

    public JZF_FileInfo(String filepath) {
        this.uiFileType = 0;
        this.uiSubFileType = 0;
        this.uiTaskCode = 0;
        this.uiObjCode = 0;
        this.uiMsgLen = 0;
        this.sMsgCont = null;
        this.FileList = null;
        this.FileNameS = null;
        if (filepath != null)
            this.filepath = filepath;

        if (this.filepath.length() > 0 && this.filepath.endsWith("/") == false) {
            this.filepath += "/";
        }
    }

    public JZF_FileInfo() {
        this.uiFileType = 0;
        this.uiSubFileType = 0;
        this.uiTaskCode = 0;
        this.uiObjCode = 0;
        this.uiMsgLen = 0;
        this.sMsgCont = null;
        this.FileList = null;
        this.FileNameS = null;
    }


    /**
     @brief 从电文里获取文件信息
     @param[in] byte[] msg 电文信息
     @return >=0--success, <0--failure
     */
    public int GetFromMsg(byte[] msg) {
        boolean boolret = false;
        JCML cml = new JCML();

        ///加载xml格式电文信息
        boolret = cml.cmlLoadDocFromMem(msg);
        if (boolret == false) {
            System.out.println("JZF_FileInfo--GetFromMsg 'jcmlLoadDocFromMem' ERROR !!!!");
            return -110001;
        }

        ///从电文里获取文件信息的每个属性
        int filenum = cml.cmlGetInt("/cmlRoot/uiFileNum");
        if (cml.rslt < 0) {
            System.out
                .println("JZF_FileInfo--GetFromMsg 'jcmlGetInt(/cmlRoot/uiFileNum)' (" + cml.rslt + ") ERROR !!!!");

            return cml.rslt;
        }

        if (filenum <= 0) {
            System.out.println("AC_DATBLDCMD--GetFromMsg 'zrfilenum < 0' (" + filenum + ") ERROR !!!!");

            return -110002;
        }

        this.FileList = new JZF_FileMap[filenum];
        String tmpstr = null;
        for (int i = 0; i < filenum; i++) {
            this.FileList[i] = new JZF_FileMap();
            tmpstr = cml.cmlGetString("/cmlRoot/szFileName[" + (i + 1) + "]");
            if (cml.rslt < 0) {
                System.out.println(
                    "AC_DATBLDCMD--GetFromMsg 'jcmlGetString(/cmlRoot/szFileName)' (" + cml.rslt + ") ERROR !!!!");
                return cml.rslt;
            }

            this.FileList[i].FileName = (this.filepath+tmpstr).getBytes();

        }

        this.FileNameS = cml.cmlGetString("/cmlRoot/szFileNameS");
        if (cml.rslt < 0) {
            System.out.println(
                "AC_DATBLDCMD--GetFromMsg 'jcmlGetString(/cmlRoot/szFileNameS)' (" + cml.rslt + ") ERROR !!!!");
            return cml.rslt;
        }

        this.uiMsgLen = cml.cmlGetInt("/cmlRoot/uiLen");
        if (cml.rslt < 0) {
            System.out.println("JZF_FileInfo--GetFromMsg 'jcmlGetInt(/cmlRoot/uiLen)' (" + cml.rslt + ") ERROR !!!!");

            return cml.rslt;
        }

        if (this.uiMsgLen > 0) {
            this.sMsgCont = cml.cmlGetBinary("/cmlRoot/szMsg");
            if (cml.rslt < 0) {
                System.out.println(
                    "JZF_FileInfo--GetFromMsg 'cmlGetBinary(/cmlRoot/szMsg)' (" + cml.rslt + ") ERROR !!!!");
                return cml.rslt;
            }
        }
        ///释放xml格式电文信息
        cml.cmlFreeDocHandler();
        return 0;
    }
}
